#include <stdio.h>
#include <string.h>
#include <math.h>

// encode electron density dxyz into unsigned short and decode it

float Dmin;
float Dmax;
float Range;
int Maxint = 0xfffff; // 20 bits 2^20-1

void setdminmax(float dmin,float dmax) {  // callable from Fortran
  Dmin = dmin;
  Dmax = dmax;
  Range = (Dmax-Dmin);
}

unsigned short encode (int temp) {
    if ( temp >= 0x8000 ) { // sign bit=1  0x8000 = 2^15  0xffff=2^16-1
          temp = temp >> 5;
	  return ((unsigned short)temp ^ 0xffff);          	  	   
    } else {
      return (unsigned short)temp;
    }
}	

int decode(unsigned short temp) {
  if ( temp & 0x8000 ) { // sign bit=1  0x8000 = 2^15  0xffff=2^16-1
    int tmp = (int)(temp ^ 0xffff);
    return (tmp << 5);          	  	   
  } else {
    return (int)temp;
  }
}	

unsigned short encodedxyz(float d) {   // callable from Fortran
  int dxyz = (int)((d-Dmin)/Range*Maxint); // compress d within Range
  unsigned short dx=encode(dxyz);  // ibm specific
  return byteoinv(dx);             // ibm specific 
  //  return encode(dxyz);
}

float decodedxyz(unsigned short d) {  // callable from Fortran
  unsigned short dx= byteoinv(d);  // ibm specific
  int dxyz =  decode(dx);             // ibm specific
  //int dxyz =  decode(d);
  return dxyz*Range/Maxint+Dmin;
}

void getdminmax(float dxyz[],int *nt, float *dmin,float *dmax) {
  *dmin = dxyz[0];
  *dmax = dxyz[0];
  //fprintf(stderr,"nt %d\n",nt);  
  int i;
  for (i = 1; i < *nt; i++) {
    if(dxyz[i] > *dmax)  *dmax=dxyz[i];
    if(dxyz[i] < *dmin)  *dmin=dxyz[i];
    //fprintf(stderr," %f ",dxyz[i]);
  }
  //fprintf(stderr,"dmin %f dmax %f",dmin,dmax);
}

// read binary data
void dxyzbinin(float dxyz[], int npix[], float ltconst[], char filename[]) { // callable from Fortran
  float dmin,dmax;
  FILE *fptr = fopen(filename,"rb");  // File pointer
  // read nt, dmin, dmax from the header (32 bites)

  char s1[40],s2[40],s3[31],s4[27];
  fread(s1,1,39,fptr); s1[39]='\0';
  fread(s2,1,39,fptr); s2[39]='\0';
  fread(s3,1,30,fptr); s3[30]='\0';
  fread(s4,1,26,fptr); s4[26]='\0';
 
  sscanf(s1,"%e %e %e",&ltconst[0],&ltconst[1],&ltconst[2]);  // read 36 bites a,b,c
  sscanf(s2,"%e %e %e",&ltconst[3],&ltconst[4],&ltconst[5]);  // read 36 bites alpha,beta,gamma
  sscanf(s3,"%d %d %d",&npix[0],&npix[1],&npix[2]);           // read 30 bites number of pixcels
  sscanf(s4,"%e %e",&dmin,&dmax);                             // read 24 bites

  int nt=npix[0]*npix[1]*npix[2];
  setdminmax(dmin,dmax);  // initialization for decodedxyz
  // read nt short int data
  unsigned short us;
  int i;
  for(i = 0; i < nt;  i++) {
    fread(&us,2,1,fptr);      // write one short int
    dxyz[i] = decodedxyz(us);
  }
  fclose(fptr);
}

// write binary data
// callable from Fortran

void dxyzbinout(float dxyz[], int npix[], float ltconst[], char filename[]) { 
  float dmin,dmax;
  int nt=npix[0]*npix[1]*npix[2];
  getdminmax(dxyz,&nt,&dmin,&dmax); 
  setdminmax(dmin,dmax);  // initialization for encodedxyz
  FILE *fptr = fopen(filename,"wb");  // File pointer
  //write nt, dmin, dmax at the header (32 bites)
  char s1[40],s2[40],s3[31],s4[27];
  sprintf(s1," %12.5e %12.5e %12.5e",ltconst[0],ltconst[1],ltconst[2]);  // write 39 bites a,b,c
  sprintf(s2," %12.5e %12.5e %12.5e",ltconst[3],ltconst[4],ltconst[5]);  // write 39 bites alpha,beta,gamma
  sprintf(s3," %9d %9d %9d",npix[0],npix[1],npix[2]);                    // write 30 bites number of pixcels
  sprintf(s4," %12.5e %12.5e",dmin,dmax);                                // write 26 bites
  fwrite(s1,1,39,fptr);
  fwrite(s2,1,39,fptr);
  fwrite(s3,1,30,fptr);
  fwrite(s4,1,26,fptr);

  // add nt short int data
  unsigned short us;
  int i;
  for(i = 0; i < nt;  i++) {
    us = encodedxyz(dxyz[i]);
    fwrite(&us,2,1,fptr);      // write one short int
  }
  fclose(fptr);
}

void dxyzascout_(float dxyz[], int npix[], float ltconst[], char filename[]) { 
  // .rho file (Wien2k file)
  float dmin,dmax;
  int nt=npix[0]*npix[1]*npix[2];
  getdminmax(dxyz,&nt,&dmin,&dmax); 
  setdminmax(dmin,dmax);  // initialization for encodedxyz
  FILE *fptr = fopen(filename,"w");  // File pointer
  fprintf(fptr," %s\n","cell");
  fprintf(fptr," %12.5e %12.5e %12.5e\n",ltconst[0],ltconst[1],ltconst[2]);
  fprintf(fptr," %12.5e %12.5e %12.5e\n",ltconst[3],ltconst[4],ltconst[5]);
  fprintf(fptr," %d %d %d",npix[0],npix[1],npix[2]);
  fprintf(fptr," %f %f %f\n",ltconst[0],ltconst[1],ltconst[2]);

  // add nt short int data
  unsigned short us;
  int i;
  for(i = 0; i < nt;  i++) {
    fprintf(fptr,"%12.5e ",dxyz[i]);
    if(i%5 == 0) {
      fprintf(fptr,"\n");
    }
  }
  fclose(fptr);
}

/*
main(int argc, char *argv[]) {
   if (argc != 1) {
      fprintf (stderr, "%s", "Usage: encodexyz\n\n"); 
      return 0;
   }

   float dmin = 30000;
   float dmax = 600000;
   setdminmax(dmin,dmax);  // callable from Fortran

   int n[] ={30000, 0x8000, 50000, 0x10000, 100000, 200000,400000,600000};
   int i;
   for(i=0; i < 8; i++) {
     fprintf(stderr," %d",n[i]);
     unsigned short m = encode(n[i]);
     fprintf(stderr," %d",m);
     int l = decode(m);
     fprintf(stderr," %d\n",l);
   }

   fprintf(stderr,"\n");
   //float m[] ={30000, 64511, 63973, 63487, 62410, 59285, 53035, 46785};
   float m[] ={30000, 0x8000, 50000, 0x10000, 100000, 200000,400000,600000};
   int i;
   for(i=0; i < 8; i++) {
     fprintf(stderr," %10.1f",m[i]);  // original
     unsigned short l = encodedxyz(m[i]);  // callable from Fortran
     fprintf(stderr," %d",l);   // encoded unsigned short
     float n = decodedxyz(l);              // callable from Fortran
     fprintf(stderr," %10.1f\n",n);     // dcoded value
   }
}
*/

unsigned short byteoinv(unsigned short fmap) {
  /* byte order inversion is necessary for dec, and intel */
  /* it is not needed for sun, hp */
  unsigned short high, low;
  unsigned short tmp = fmap;
  high = tmp >> 8;   /* high byte */
  low  = 0xff & tmp;  /* low byte */
  fmap= (low << 8) + high;
  return fmap;
}
