c     
c      This program generates a mem map, fourier map or patterson map
c          for crystals including modulated and composite crystals
c
c       input format as follows
c          key=value where key is key work shown below
c          for character value it should be enclosed by '
c            as flnm1='filename'
c          for numerical value it is like ndm=1 or f000=123.4
c     
c     (01) iwflag :
c     (02) flnm1 :  file name of hkl,Fobs,Fcal,phase,sig  for MEM
c                   file name of hkl,Fobs,Fcal,phase for Fourier
c     (03) flnm2 :  file name for nD density file (iplot=0)
c                   file name for 2D density file (iplot>0)
c                   file name for 3D density file (iplot<0) (.win file)
c     (1) title
c     (2) a,b,c,cosa,cosb,cosc
c         a,b,c in angstrome
c     (2') nsub,  zmtrx
c         nsub : number of subsystems
c         zmtrx: z matrix (permutation matrix for nsub >=2)
c     (3) ndm,nsymo,icent,ige,nctr
c         ndm: order of modulations  1 for one-dimensional modulation etc
c         (or extra dimension)
c         nsymo: number of symmetry operators
c         icent: 1 for centrosymmetric 0 for noncentrosymmetric
c         ige:   1 when generating elements are given else 0
c         nctr:  number of centering translation (excluding 0,0,0,0,0,0)
c         ********if ndm=0 skip (4)********
c     (4)  a*,b*,c* components of the wave vector icorc, ilm (*)
c         if incommensurate icorc=1, commensurate icorc=0
c         ilm  number of divisions of gauss integrals in structure
c         factor formula in incommensurate cases     3<=ilm<=100
c         ******caution:  satellite with order greater than****
c         *******ilm/5 is not calculated.  This means that ****
c         *******if second order satellite intensity must be***
c         *******necessary ilm >= 10 is necessary *************
c         =========== repeat (4) ndm times ============
c     
c         if nctr > 0 then (5)
c     (5) 'centering translation'
c         in the format of I.T.  1/2,1/2,1/2,0,0,0 etc
c     (6) 'symmetry operator'
c         in the format of I.T. x,y,z,t,u,v etc
c     (7) f000, sg00, sg0, sg1, fmag
c         f000: fc at the origin
c         (number of electrons in the unit cell of the first substructure)
c         sg00: sigma for f000
c         sg0:  average sigma
c         sg1:  sigma for strongest (Fcal) reflection
c         fmag: f000,sg00,sg0,sg1,fobs,fcal are
c               magnifiled by this factor
c         (fmag is the scale factor for the given structure factor)
c         if observed structure factor includes f000, this is replaced

c     (8) lgr: laue group: m3m,m3,4/mmm,4/m,mmm,2/m,-1,6/mmm,6/m,-3m,-31
c         laue group (number of equivalent positions)
c         m3m(48),m3(24),4/mmm(16),4/m(8),mmm(8),2/m(4),-1(2)
c         6/mmm(24),6/m(12),-3m(12),-31(6)
c         when lgr=2/m,(h2>=0 or h2<=0) and (h3>=0 or h3<=0) are assumed
c         when lgr=-1, h3>=0 or h3<=0 is assumed
c     
c     (9) jbc
c         jbc: kind of map  1 : no axis transformation
c                          -1: see (10) 
c                          -2: see (10')
c         (this program only write a plane normal to a*, b* or c*.
c         to write arbitrary plane, give jbc < 0 and 
c         specify transformation matrix (10) which defines
c         new a*,b*,c* from old ones.)

c         **if jbc !=-1 skip (10_1) (10_2)**
c     (10_1) transformation matrix for miller index itr(6,6) 
c                        (for superstructures)
c         itr(i,j),j=1,6 (*)
c         repeat 6 times
c         which is equal to transformation matrix for
c         unit vectors in direct space
c     (10_2) s1, s2, s3, s4 : orthogonal transformation matrix for q1-q6
c            ((s1(i,j),j=1,3),i=1,3) etc.
c             s1, s2 etc. should be in different lines
c             total 6x6 matrix is given by 
c                     s1,s3
c                     s4,s2

c         **if jbc !=-2  skip (10'_1)-(10'_3)**
c     (10'_1)  isetting
c           isetting : 1 c axis unique, 2 b axis unique, 3 a axis unique
c     (10'_2)  v
c           v : matrix ((v(i,j),j=1,3),i=1,6)
c           6x3 U matrix, which gives the a1-a3 coefficient of
c           6 unit vectors in reciprocal space, is calculated from
c           iseging and a*,b*,c*,cosa*,cosb*,cosc* and k1,k2,k3
c           6x3 V matrix, which gives the  a4-a6 coefficient of
c           6 unit vectors in reciprocal space, should be given
c           here

c           first 6x3 part of 6x6 R matrix is U while
c           second 6x3 part is V
c
c           in the standard setting for modulated structure
c           first 3x3 part of V is zero matrix while second 3x3
c           part is unit matrix for many cases, but may be
c           different for triclinic,monoclinic,hexagonal system
c           in idm=5
c           for example, k1 and k2 are in the a*-b* plane in hexagonal system,
c           we may take v(4,1)=a*,    v(4,2)=0,           v(4,3)=0
c                       v(5,1)=-a*/2, v(5,2)=sqrt(3)a*/2, v(5,3)=0
c                       v(6,1)=v(6,2)=v(6,3)=0
c           the best v(1,1)-v(3,3) are sample-dependent for which
c           the modulation wave should be parallel to the internal space
c           v(1,1)=v(1,2)=...=v(3,3)=0 gives the standard setting for
c           modulated structure but this is not appropriate for
c           semi-quasicrystals      
c     (10'_3) s1, s2, s3, s4 : orthogonal transformation matrix for q1-q6
c            ((s1(i,j),j=1,3),i=1,3) etc.
c             s1, s2 etc. should be in different lines
c             total 6x6 matrix is given by 
c                     s1,s3
c                     s4,s2

c     (11) iplot,hmax,pkmin,rwmin (for iplot=5)
c         iplot=1, -1:  Patterson map
c              =2, -2:  Fourier map
c              =3, -3:  difference Fourier map
c              =4, -4:  MEM map (starting flat electronndensity)
c              =5, -5:  MEM and LDEM starting from the result of LDEM
c              =6, -6:  MEM and LDEM starting from the result of LDEM
c                       f000 should be includel in observed structure factor
c                       calculated f is not normalized by f000
c             *****************new***************
c         for negative iplot, 3D density function is saved in a file flnm2
c
c         hmax(1-6): number of grid points along principal axes
c         (these sould be the multiple of the power of prime numbers
c         up to 21)
c         hmax(1)*hmax(2)*..*hmax(6) <=256*256*256
c         *****hmax(i) >= 1  (for i=1,2,..,6)******
c         pkmin : minimum peak height fraction to be printed in Fourier
c                           or Difference Fourier map
c         (real minimum peak hekgit is given by pkmax*pkmin
c          where pkmax is the maximum peak density)      
c         abs(peak density) > pkmax*pkmin is printed
c         rwmin  : rw factor below which LEM method works

c         *******(pkmin > 0)*******
c     
c     ***** if abs(iplot)=1,2,3 skip (12)
c     (12) ex1,ex2,nex1,nex2,icont
c         ex1,ex2:  constants needed for smooth convergence
c         nex1: number of iterations in imode=1 (Yamamoto)
c         nex2: number of iterations in imode=2 (Sakata)
c         icont : 1 continue from former results
c     ***** imode=1 mode is dangerous for accurate electron density map
c     ***** imode=2 mode should be used for MEM
c         data for plotter
c     (12') frmt
c         frmt: file format for hkl, fobs, fcal, phs, sig(f)
c         phs : phase in degree      
c         sig(f) is necessary only for MEM
c     (13) ldev,iwhkl
c         ldev : 1: B&W Postscript printer 2: color Postscript printer
c         iwhkl  : 0 no output, 1: output of hkl fo fc
c         obtained from MEM or Fourier map
c     
c         write strongest positive peak positions
c         for difference fourier (abs(iplot)=3)
c     
c     
c     (14) nsh: number of sheets for electron density map output
c     
c         *********if nsh=0 skip (15)-(23)*******
c     (15) ng ismth
c               ng : number of graphs in a sheet
c                ismth : 0=no smoothing
c                        1=3 point average
c                        2=5 point average
c     (16) xs: origin coordinate in each map (xs(1)-xs(6))
c         (coordinates of the map origin in the new coordinate system
c         specified at (10))
c     (17) iax: selection of axes (iax(1)-iax(6))
c         write (electron) density in iax(1)-iax(2) plane

c           if iplot > 0
c     (18) ax,bx,coscx (length of axes in cm and cos angles)
c         default ax,bx: a,b,c for iax=1,2,3, d=5 for iax=4
c         coscx,: cosc,cosa,cosb for ax,bx=1,2;2,3;3,1
c         coscx: calculated from k1,k2,k3 for ax=a,b,c and bx=d
c         to use default falues, give ax=bx=0
c     (19) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
c         xmin,..,ymax:coordinate reanges along x and y axes
c         hscl : scale for F
c         xdiv, ydiv : intervals of the scale in x and y axes

c           if iplot < 0
c     (18) ax,bx,cx,cosax,cosbx,coscx (length of axes in cm and cos angles)
c         default ax,bx,cx: a,b,c for iax=1,2,3, d=5 for iax=4
c         cosax,cosbx,coscx,: cosc,cosa,cosb for ax,bx=1,2;2,3;3,1
c         cosax,cosbx,coscx: calculated from k1,k2,k3 for ax=a,b,c and bx=d
c         to use default falues, give ax=bx=0
c     (19) xmin,xmax,ymin,ymax,zmin,zmax,hscl,xdiv,ydiv,zdiv
c         xmin,..,zmax:coordinate reanges along x,y and z axes
c         hscl : scale for F
c         xdiv, ydiv, zdiv : intervals of the scale in x and y axes

c         if iplot < 0  skip (20)-(23)      
c     (20) hmin,div,lf
c         hmin: minimum hight (density) in the division
c         div : interval of hight
c         lf  : number of contour lines
c     (21) amagx,amagy,shftxb,shftyb,shftxa,shftya
c         amagx:  magnification for x
c         amagy:  magnification for y
c         shftxb,shftyb: shifts before each copy
c         shftxa,shftya: shifts after each copy
c     (22) iwtitl,wx,wy,ws,angl
c         if iwtitl=1, title is written
c         wx,wy: coordinates for the title
c         ws size of letter in cm, angle is the angle
c         from the x axis in degree
c     (23) wtitle : title for the figure
c     ****************repeat (16)-(23) ng times****************
c     ****************repeat (15)-(24) nsh times***************
c
      implicit integer (h)
      common /const0/ pi2,dmgn,irs(6)
      character(80) title
      common /ctitle/ title
      common /cjbc/ jbc,xs(6),iax(6)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /cwght/ iwg
      common /kwave/ y(3,3,3),vc(3)
      common /cwflag/ iwflag(6)
      common /ctr/ itr(6,6)
c     common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      character(80) frmt
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq

      include 'nhmax.inc'
      common /cphs/ jp(nhmax),nphase ! phase for independent reflections
      real foi(nhmax)           ! fo for independent reflections
      real sgi(nhmax)           ! sigma for independent reflections
      integer hkli(6,nhmax)     ! hkl for independent reflections
      common /cfoi/ hkli,foi,sgi
      integer iphi(nhmax)
      common /ciphi/ iphi
      integer index(nhmax)      ! location of independent reflection in ar,ai
      include 'hklf.inc'       ! independent reflections
      real hscl
      include 'abcdri.inc'
      include 'nrmax.inc'
      include 'qsort.inc'      ! for use of t,ks,kp
      real dxy(513*513), dxz(513*513)
      character(80) wtitle
      common /name/ iwtitl,ws,angl,wtitle
      include 'drwmap.inc'

c     integer mm(7)
      integer hmax(6),hmax1(6)
      common /cnex/ nex1,nex2,ex1,ex2
      common /cdenmodMem/ ic,n1,n2,ex

      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      parameter (nbuf=2000)
      real hmin(10),div(10)
      integer lf(10)

      character(80) flnm1,flnm2,flnm3,flnm4
      common /cfilename/ flnm1,flnm2,flnm3,flnm4
      character(5) lgrp(11),lgr
      character(80) prefix
c     character brv
      integer defiwf(6)
      real xmn(6),xmx(6)
      logical isMem,isPatterson,isFourier,isDFourier
      integer getntf0,getntx0,getnt
      integer irefn(6)          ! refl. no. for fixed phase
      real tarray(2)
      integer getis
      character char
      data defiwf/0,1,1,4,2,3/
      data lgrp/'m3m','m3','4/mmm','4/m','mmm','2/m','-1',
     *     '6/mmm','6/m','-3m','-31'/
      data zmtrx /1,2,3,4,5,6,1,2,3,4,5,6,1,2,3,4,5,6/
      data nsub /1/
      data xmin /0./, xmax /1./, ymin /0./, ymax /1./
c      data eps / 0.000001/
      pi2=2*getpi()

      call setinp(5)
      call setout(6)
      call setskp(1)
      call getinp(inpu)
      call getout(iout)

      call freadiv('iwflag=',6,iwflag,1)
      if(iwflag(1).eq.0) then
         do i=1,6
            iwflag(i)=defiwf(i)
         end do
      end if
      write(iout,'(1h ,a,6i3)') 'output control flags  ', iwflag
      call freadc('flnm1=',1,flnm1,1)
      call freadc('flnm2=',1,flnm2,1)
      call getflnmprfx(flnm2,prefix,n)
      write(6,*) 'n,prefix=',n,',',prefix(1:n)
      flnm3(1:n+5)=prefix(1:n)//'.win'//char(0)
      flnm4(1:n+5)=prefix(1:n)//'.rho'//char(0)
      write(6,*) 'flnm3=',flnm3(1:n+5),',','flnm4=',flnm4(1:n+5)
      call rdlc()
      call freadi('nsub=',1,nsub,1)
      if(nsub.le.0.or.nsub.gt.3) call quit('*** nsub invalid***')
      if(nsub.ge.2) then
         call freadiv('zmtrx=',6,zmtrx(1,2),0)
      end if
      if(nsub.eq.3) then
         call freadiv('zmtrx=',6,zmtrx(1,3),0)
      end if
      if(iwflag(3).eq.1) then
         write(6,'(a,i2,(a,6i2))') ' nsub=',nsub,
     *        (' zmtrx=',(zmtrx(i,n),i=1,6),n=2,nsub)
      end if

      na=1
      call rdsymo(na)  ! for crystal
c     call setbrv(brv)
      call getgij(zmtrx,nsub)
      vol=gabx(1,1)*gabx(2,2)*gabx(3,3)
      write(6,*) 'vol=',vol
      call setvol(vol)
      call rdiob0(1)   ! 
      call setskp(0)
      call setinp(5)
c     read(5,'(a)') lgr
      call freadc('lgr=',1,lgr,1)
      write(6,*) 'lgr=',lgr
      iwg=0
      do i=1,11
         if(lgr.eq.lgrp(i)) then
            iwg=i
         end if
      end do
      if(iwg.eq.0) call quit('lgr is wrong')
      call freadi('jbc=',1,jbc,1)
      if(jbc.eq.0) jbc=1
c 501  continue
      if(dmgn.eq.0.) dmgn=1.
      write(6,*) 'jbc=',jbc
      call setjbc(jbc)

      if(jbc.eq.-1) then
         call readitr(det)
         call reads1234()       ! read s1,s2,s3,s4 matrices
      else if(jbc.eq.-2) then
         call freadi('isetting=',1,isetiing,1)
         call setumtrx(isetting)  ! set 6x3 u matrix
         call readVmtrx()
         call setdqmtrx()
         call setdr(dq)
         call reads1234()       ! read s1,s2,s3,s4 matrices
      end if

      call freadi('iplot=',1,iplot,1)
      call freadiv('hmax=',6,hmax,0)

      write(6,'(a,i2,a,6i4)') 'iplot=',iplot,' hmax=',hmax
      call getidm(idm)
      call setng(hmax)          ! for lodem
c     write(6,*) '****idm=',idm,'****'

      call calmm(hmax,idm)      ! set mm, hmax, idm
      call setitt(hmax)         ! set non primitive translation for grid points

      call freadr('pkmin=',1,pkmin,0)
      write(6,'(a,f10.5)') 'pkmin=',pkmin
      if(pkmin.le.0) then
         call quit('pkmin should be positive')
      end if

      if(iplot.eq.0) then       ! initial flat density
         call quit('do not support iplot=0')
      end if

      rwmin=0
      if(isMem()) then
         if(abs(iplot).eq.5) then
            call freadr('rwmin=',1,rwmin,0)
            write(6,*) 'rwmin=',rwmin
         end if
         iaster=0
      else if(isFourier().or.isDFourier()) then
         iaster=0
      end if
      call setrwmin(rwmin)

      nt=getnt()
      if(nt.gt.nts) then
         write(6,*) 'nts=',nts
         call quit('nt > nts')
      end if

      icont=0
      if(isMem()) then          ! MEM
         call freadr('ex1=',1,ex1,1)
         call freadr('ex2=',1,ex2,0)
         call freadi('nex1=',1,nex1,0)
         call freadi('nex2=',1,nex2,0)
         call freadi('icont=',1,icont,0)
         write(6,'(2(a,e10.3,3x),2(a,i3,3x),a,i1)')
     *        'ex1=',ex1,'ex2=',ex2,'nex1=',nex1,'nex2=',nex2,
     *        'icont=',icont
      else
         ex1=0
         ex2=0
      end if

      call freadc('frmt=',1,frmt,1)
      iln=istrln(frmt)
      write(6,*) 'frmt=',frmt(1:iln)

      call freadi('ldev=',1,ldev,1)
      call freadi('iwhkl=',1,iwhkl,0)
      write(6,'(3(a,i1))') 'ldev=',ldev, ' iwhkl=', iwhkl

      iwght=1
      call rdiob1(iwght,frmt,flnm1) ! read fobs fcal iphase
c     check whether the density file can open for MEM
      call openfileu(3,flnm2,'UNKNOWN') ! for unformatted file
      close(3)
c     end if

c     generate equivalent reflections
c     and set initial phase for ar,ai
      call symstf()             ! generate all equivalent reflections
c     f000=getf000()
      f000=getf0()
      call setindfindx()        ! set independent structure factor index
      call setindxindx()        ! set independent grid index
      write(6,*) 'ntf0=',getntf0(),' ntx0=',getntx0()
**************************for check
      if(isMem()) then          ! MEM initialization
         if(icont.eq.0) then 
            if(abs(iplot).eq.4) then
c     initial flat density and its independent part and set dmin dmax
               call initcr0(f000) ! this should be used for MEM
c     call initcr4(f000) ! for test
            else if(abs(iplot).eq.5) then
c     initial flat density and its independent part and set dmin dmax
c     call initcr4(f000) ! for test
               call initcr0(f000) ! this should be used for MEM
            else if(abs(iplot).eq.6) then
c     initial flat density and its independent part and set dmin dmax
c     call initcr4(f000) ! for test
               call initcr0(f000) ! this should be used for MEM
            else
               call quit('iplot=0 is not supported')
            end if
         else if(icont.eq.1) then
            call openfileu(3,flnm2,'OLD') ! for unformated files
            read(3) (br(j),j=0,nt-1)
            close(3)
            call setxr0()
            call getdmx()       ! get new dmin, dmax
         end if
      else
c     Fourier or Patterson map
         call initcr1() 
      end if
      if(isPatterson()) then    ! Patterson
         call wtlatticec()
      end if

      if(isMem()) then
         call chsdenmodf(2) 
         call memloop(f000,c2d,flnm2,rfact,rwfact)
      end if

      call getbrfromxr0()       ! get br for pksearch
      call pksearchnd(br,pkmin) ! peak search
      call getpeakcrd(br,xmn,xmx) ! peak coordinates
      write(6,'(a,f10.2,a,6f10.5)') 'dmin=',getdmin(),' xmn=',xmn
      write(6,'(a,f10.2,a,6f10.5)') 'dmax=',getdmax(),' xmx=',xmx
      if(isMem()) then          ! only for MEM map
         call openfileu(3,flnm2,'UNKNOWN') ! for unformatted file
         write(3) (br(j),j=0,nt-1)
         close(3)
         call rfct(c2d,rfact,rwfact,iwhkl)
      end if

      call drwmap(br,ldev)
      call exit(0)
      end


      integer function getjbc()
      common /cjbc/ jbc,xs(6),iax(6)
      getjbc=jbc
      return
      end

      subroutine setjbc(jbcd)
      common /cjbc/ jbc,xs(6),iax(6)
      jbc=jbcd
      return
      end

      subroutine readitr(det)
      common /ctr/ itr(6,6)
      real ritr(6,6),bt(6),piv(6)
      integer ipv(6),ind(2*6)
      data itr /1,0,0,0,0,0,
     *     0,1,0,0,0,0,
     *     0,0,1,0,0,0,
     *     0,0,0,1,0,0,
     *     0,0,0,0,1,0,
     *     0,0,0,0,0,1/
      do i=1,6
         read(5,*) (itr(i,j),j=1,6)
      end do
      write(6,'(6i3)') ((itr(i,j),j=1,6),i=1,6)
      call mcpyir(itr,ritr)
      call matinv(ritr,6,6,bt,0,det,ipv,ind,piv)
      call trnspi(itr)  ! take transpose for transformation matrix for x
      return
      end

      subroutine wtlatticec()
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      pi=3.141592
      a=sqrt(gab(1,1,1))
      b=sqrt(gab(2,2,1))
      c=sqrt(gab(3,3,1))
      cosa=gab(2,3,1)/(b*c)
      cosb=gab(3,1,1)/(c*a)
      cosc=gab(1,2,1)/(a*b)
      alpha=acos(cosa)*180/pi
      beta =acos(cosb)*180/pi
      gamma=acos(cosc)*180/pi
      write(6,*) 'a,b,c,alpha,beta,gamma=',
     *     a,b,c,alpha,beta,gamma
      return
      end

      function getxleng(x)  ! get vector length in A
      real x(3)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      real y(3)
      xleng=0
      do i=1,3
         do j=1,3
            xleng=xleng+x(i)*gab(i,j,1)*x(j)
         end do
      end do
      getxleng=sqrt(xleng)
      return
      end

      function getyllen(yl)  ! get reciplocal vector length in A
      real yl(3)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      real y(3)
      yllen=0
      do i=1,3
         do j=1,3
            yllen=yllen+yl(i)*gabn(i,j,1)*yl(j)
         end do
      end do
      getyllen=sqrt(yllen)
      return
      end

c      real function getf0()
c      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
c      getf0=f000
c      return
c      end

      subroutine drwmap(cr,ldev)
      implicit integer (h)
      real cr(0:*)
      include 'map.inc'
      include 'nhmax.inc'
      include 'hklf.inc'       ! independent reflections
      include 'drwmap.inc'
      parameter (ixmax1=ixmax+1)

      common /const0/ pi2,dmgn,irs(6)
      character(80) title
      common /ctitle/ title
      common /cjbc/ jbc,xs(6),iax(6)
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /kwave/ y(3,3,3),vc(3)
      common /cwflag/ iwflag(6)
      common /ctr/ itr(6,6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      character(80) frmt
      character(80) flnm1,flnm2,flnm3,flnm4
      common /cfilename/ flnm1,flnm2,flnm3,flnm4
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      
      real hscl
      character(80) wtitle
      common /name/ iwtitl,ws,angl,wtitle

      integer zmtrx
      common /czmtrx/ zmtrx(6,3), nsub
      real dxyz(ixmax1*ixmax1*5)   ! for 3D map
      real dxy(ixmax1*ixmax1), dxz(ixmax1*ixmax1) ! for 2D map
      integer hmax(6)
      parameter (nbuf=2000)
      real hmin(10),div(10)
      integer lf(10)
      integer getnt
      character(5) lgrp(11),lgr
      real ltconst(6)
      integer npix(3)
c     character brv
      integer defiwf(6)
      real tarray(2)
      data defiwf/0,1,1,4,2,3/
      data lgrp/'m3m','m3','4/mmm','4/m','mmm','2/m','-1',
     *     '6/mmm','6/m','-3m','-31'/
c     pi2=2*getpi()

      jxmax=ixmax1*ixmax1*5     ! for 3D map      

      ndiv=1
      call freadi('nsh=',1,nsh,1)
      write(6,*) 'nsh=',nsh
      call setnfil(nsh)
      ismth=0                   ! no smoothing
      call setsmt(ismth)
      if(nsh.eq.0) call exit(1)

      do nsd=1,nsh
         call freadi('ng=',1,ng,1)
         call freadi('ismth=',1,ismth,0)
         call setsmt(ismth)
         write(6,'(2(a,i3))') 'ng=',ng,' ismth=',ismth
         call plots(0.0,0.0,ldev)
         call newpen(1)

         do ig=1,ng
c     read(5,104) xs
            call freadrv('xs=',6,xs,1)
            write(6,'(a,6f8.5)') 'xs=',(xs(ii),ii=1,6)
c     read(5,100) iax
            call freadiv('iax=',6,iax,1)
            write(6,*) 'jbc1 in drwmap',jbc
c     call freadi('iwmlp=',1,iwmlp,0)
            iwmlp=1
            write(6,221) xs,iwmlp,iax
c     read(5,111) ax,bx,coscx
            if(iplot.ge.0) then ! including mem map
               call rdaxbx()
            else if(iplot.lt.0) then
               call rdaxbxcx()
            end if
c           write(6,224) ldev,iwhkl
c     read(5,113) (hmin(i),div(i),lf(i),i=1,ndiv)
            
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
            izm=(zmax-zmin)/zdiv+0.001
            ixyzm=(ixm+1)*(iym+1)*(izm+1)
            write(6,*) 'xmin=',xmin,' ymin=',ymin,' zmin=',zmin
            write(6,*) 'xmax=',xmax,' ymax=',ymax,' zmax=',zmax
            write(6,*) 'xdiv=',xdiv,' ydiv=',ydiv,' zdiv=',zdiv
            write(6,*) 'ixm=',ixm,' iym=',iym,' izm=',izm
            if(ixyzm.gt.jxmax) then
               write(6,*) 'jxmax=',jxmax
               call quit('(ixm+1)*(iym+1)*(izm+1)>jxmax')
            end if
            
            if(iplot.gt.0) then
               do ii=1,ndiv
                  call freadr('hmin=',1,hmin(ii),1)
                  call freadr('div=',1,div(ii),0)
                  call freadi('lf=',1,lf(ii),0)
                  write(6,225) hmin(ii),div(ii),lf(ii)
               end do
            end if
            write(6,*) 'jbc2 in drwmap',jbc
            if(iplot.ge.0) then !  2D map
               call rdpltmap2(ax,bx,coscx,wx,wy)
               call cxtoxy(cr,dxy,ixm,iym)
c               write(6,*) 'jbc3 in drwmap',jbc
c               if(ixm*iym.gt.ixmax1**2) then
c                  write(6,*) 'ixmax1,ixm,iym',ixmax1,ixm,iym
c                  call quit('ixmax1<ixm*iym')
c               end if
               call map01(hmin,div,lf,ndiv,iw,dxy,ixm,iym) ! for crystal
               write(6,*) 'jbc4 in drwmap',jbc
               nchr=istrln(wtitle)
               call symbol(wx,wy,ws,wtitle,angl,nchr)
            else if(iplot.lt.0) then ! 3D map
c              call rdpltmap3(iax,wx,wy)
               call cxtoxyz(cr,dxyz,ixm,iym,izm)
c              call wtdxyz(dxyz,ixm,iym,izm) ! 3D data 
               call setltcnst(ltconst)
               call setnpix(npix,ixm,iym,izm)
c              call dxyzbinout(dxyz,npix,ltconst,flnm3) ! 3D data for vend
               call dxyzascout(dxyz,npix,ltconst,flnm4) ! 3D data for vend
            end if
         end do
         call plote('end of plot')
      end do
 221  format(1h ,'coordinates of origin',6f12.6
     *     /1h ,'iwmlp=',i3,
     *     /1h ,'first to sixth axis in map',6i3)
 224  format(1h ,'ldev=',i3,2x,'iwhkl=',i3)
 225  format(1h ,'hmin=',f12.6,2x,'div=',f12.6,2x,'lf=',i3)
      return
      end

      subroutine setnpix(npix,ixm,iym,izm)
      integer npix(*)
      npix(1)=ixm
      npix(2)=iym
      npix(3)=izm
      return
      end

      subroutine setltcnst(ltconst)
      include 'drwmap.inc'
      real ltconst(*)
      ltconst(1)=ax
      ltconst(2)=bx
      ltconst(3)=cx
      ltconst(4)=todeg(acos(cosax))
      ltconst(5)=todeg(acos(cosbx))
      ltconst(6)=todeg(acos(coscx))
      return
      end

      subroutine rdpltmap2(ax,bx,coscx,wx,wy)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      character(80) wtitle
      common /name/ iwtitl,ws,angl,wtitle
      call freadr('amagx=',1,amagx,1)
      call freadr('amagy=',1,amagy,0)
      call freadr('shftxb=',1,shftxb,0)
      call freadr('shftyb=',1,shftyb,0)
      call freadr('shftxa=',1,shftxa,0)
      call freadr('shftya=',1,shftya,0)
      if(amagx.eq.0) amagx=1
      if(amagy.eq.0) amagy=1
      call setamg(amagx,amagy)
      write(6,226) amagx,amagy,shftxb,shftyb,shftxa,shftya
c     read(5,114) iwtitl,wts,angl
      call freadi('iwtitl=',1,iwtitl,1)
      call freadr('wx=',1,wx,0)
      call freadr('wy=',1,wy,0)         
      call freadr('ws=',1,ws,0)
      call freadr('angl=',1,angl,0)
      write(6,227) iwtitl,wx,wy,ws,angl
      call freadc('wtitle=',1,wtitle,1)
c     if(ixm.gt.ixmax) then
c     write(6,*) 'ixm=',ixm,' ixmax=',ixmax
c     call quit('ixm > ixmax')
c     else if(iym.gt.ixmax) then
c     write(6,*) 'iym=',iym,' ixmax=',ixmax
c     call quit('iym > ixmax')
c     end if
      irl=0
c     call setxyt(1.,1.,0.,0)
      call setxyt(ax,bx,coscx,irl)
 226  format(1h ,'amagx=',f12.6,2x,'amagy=',f12.6,2x,'shftxb=',f12.6,2x,
     *     'shftyb=',f12.6,2x,'shftxa=',f12.6,2x,'shftya=',f12.6)
 227  format(1h ,'iwtitl=',i1,2x,'wx=',f10.5,2x,'wy=',f10.5,2x,
     *     'ws=',f10.5,2x,'angl=',f10.5)
      return
      end

      subroutine setaxbx()
      common /mtrtns/ gabx(3,3),gaby(3,3),gab(3,3,3),gabn(3,3,3)
      common /cjbc/ jbc,xs(6),iax(6)
      common /kwave/ y(3,3,3),vc(3)
      include 'drwmap.inc'
      a=sqrt(gab(1,1,1))
      b=sqrt(gab(2,2,1))
      c=sqrt(gab(3,3,1))
      cosa=gab(2,3,1)/(b*c)
      cosb=gab(3,1,1)/(c*a)
      cosc=gab(1,2,1)/(a*b)
      d=5
      if(iax(1).eq.1) ax=a
      if(iax(1).eq.2) ax=b
      if(iax(1).eq.3) ax=c
      if(iax(1).eq.4) ax=d
      if(iax(2).eq.1) bx=a
      if(iax(2).eq.2) bx=b
      if(iax(2).eq.3) bx=c
      if(iax(2).eq.4) bx=d
c     a*coscx=-y(1,1,1)*d etc.
      if(iax(1).eq.1.and.iax(2).eq.2) coscx=cosc
      if(iax(2).eq.1.and.iax(1).eq.2) coscx=cosc
      if(iax(1).eq.2.and.iax(2).eq.3) coscx=cosa
      if(iax(2).eq.2.and.iax(1).eq.3) coscx=cosa
      if(iax(1).eq.3.and.iax(2).eq.1) coscx=cosb
      if(iax(2).eq.3.and.iax(1).eq.1) coscx=cosb
      if(iax(1).eq.1.and.iax(2).eq.4) coscx=-y(1,1,1)*d/a
      if(iax(1).eq.4.and.iax(2).eq.1) coscx=-y(1,1,1)*d/a
      if(iax(1).eq.2.and.iax(2).eq.4) coscx=-y(2,1,1)*d/b
      if(iax(1).eq.4.and.iax(2).eq.2) coscx=-y(2,1,1)*d/b
      if(iax(1).eq.3.and.iax(2).eq.4) coscx=-y(3,1,1)*d/c
      if(iax(1).eq.4.and.iax(2).eq.3) coscx=-y(3,1,1)*d/c
      return
      end

      subroutine rdaxbx()
      include 'drwmap.inc'

      call freadr('ax=',1,ax,1)
      call freadr('bx=',1,bx,0)
      call freadr('coscx=',1,coscx,0)
      write(6,*) 'ax,bx,coscx=',ax,bx,coscx
c     read(5,*) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
      call freadr('xmin=',1,xmin,1)
      call freadr('xmax=',1,xmax,0)
      call freadr('ymin=',1,ymin,0)
      call freadr('ymax=',1,ymax,0)
      call freadr('hscl=',1,hscl,0)
      call freadr('xdiv=',1,xdiv,0)
      call freadr('ydiv=',1,ydiv,0)
      write(6,'(7(a,f8.2))') 'xmin=',xmin,' xmax=',xmax,
     *     ' ymin=', ymin,' ymax=',ymax,' hscl=',hscl,
     *     ' xdiv=',xdiv,' ydiv=',ydiv
      zdiv=1.0
c     set default
      if(ax.eq.0.and.bx.eq.0) then
         call setaxbx()
      end if
c      write(6,222) ax,bx,coscx
c 222  format(1h ,3(a,f12.6,2x))
      return
      end

      subroutine rdaxbxcx()
      include 'drwmap.inc'
      call freadr('ax=',1,ax,1)
      call freadr('bx=',1,bx,0)
      call freadr('cx=',1,cx,0)
      call freadr('cosax=',1,cosax,0)
      call freadr('cosbx=',1,cosbx,0)
      call freadr('coscx=',1,coscx,0)
      write(6,*) 'ax,bx,cx,cosax,cosbx,coscx=',
     *     ax,bx,cx,cosax,cosbx,coscx
c     read(5,*) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
      call freadr('xmin=',1,xmin,1)
      call freadr('xmax=',1,xmax,0)
      call freadr('ymin=',1,ymin,0)
      call freadr('ymax=',1,ymax,0)
      call freadr('zmin=',1,zmin,0)
      call freadr('zmax=',1,zmax,0)
      call freadr('hscl=',1,hscl,0)
      call freadr('xdiv=',1,xdiv,0)
      call freadr('ydiv=',1,ydiv,0)
      call freadr('zdiv=',1,zdiv,0)
      write(6,'(6(a,f8.2))') 'xmin=',xmin,' xmax=',xmax,
     *     ' ymin=',ymin,' ymax=',ymax,' zmin=',zmin,
     *     ' zmax=',zmax
      write(6,'(4(a,f8.2))') ' hscl=',hscl,' xdiv=',xdiv,
     *     ' ydiv=',ydiv,' zdiv=',zdiv
c      ax=(xmax-xmin)
c      bx=(ymax-ymin)
c      zx=(zmax-zmin)
c      cosax=0
c      cosbx=0
c      coscx=0
c      write(6,223) ax,bx,cx,cosax,cosbx,coscx
c 223  format(1h ,6(f12.6,2x))
      return
      end

      function getfmax()
      common /cfmax/ fmax
      getfmax=fmax
      return
      end

      integer function getnind()
      common /cnind/ nind       ! number of independent reflections used
      getnind=nind
      return
      end

      function getf0()
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      getf0=f000
      return
      end

      subroutine symstf() ! generate all equivalent reflections
      implicit integer (h)
      include 'abcdri.inc'
      real dmgn
      common /const0/ pi2,dmgn,irs(6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl

      include 'nhmax.inc'
      include 'hklf.inc'       ! independent reflections
      include 'symstfsort.inc'

      common /cfmax/ fmax
      integer nfmax
      common /chkl/ nfmax
      common /cnind/ nind       ! number of independent reflections used

      integer hd(6),hr(6)
      complex fctmp,fctmp0
      integer hmax(6)
      integer gethkltoj,getnt
      integer hkl(6)
      integer mmd(7)
      logical isMem,isPatterson,isFourier,isDFourier
      integer caliph
      call getidm(idm)
      call gethmax(hmax)
      call getnsymo(nsymo)
      call geticent(icent)
      call getiplt(iplot)
      nt=getnt()
      write(6,*) 'iplot=',iplot,' nsymo=',nsymo,' icent=',icent
      j=0
c     initialization
      call zerovr(br,nt)
      call zerovr(bi,nt)
c     call checkIsFunction()
      if(isDFourier()) then
         br(0)=0
      else
         br(0)=f000             ! set this first after that this may override
      end if
      fmax=0

      do i=1,is                 ! number of independent reflections
         fod(i)=fobs(i)
c     write(6,*) 'fod=',fod(i)
      end do

      call qsortd(fod,st,jp,is) ! sort from strong to weak reflections
      nfmax=multpl(hkln(1,jp(1))) ! number of strongest reflections

      nref=0
      nind=0
      nskp=0
c     write(6,*) 'is=',is
      write(6,*) 'nsymo=',nsymo
      do jd=1,is                ! independent reflection
         j=jp(jd)
         call vcpyi(hkln(1,j),hd,6)
c        call wtvctri('hd',hd,6)
         sgobs=sig(j)
         fctmp0=cmplx(phsr(j),phsi(j))
         iph(j)=caliph(phsr(j),phsi(j))
         do ns=1,nsymo
            call gethr(hd,hr,ns,0)
c           call wtvctrif('hr=',hr,6,'(6i4)')
            if(iskip(hr).eq.1) then
               nskp=nskp+1
               go to 30
            end if
            yt=getyt(hd,ns,ic)
c     write(6,*) 'yt=',yt
                                !  strucutre factor for equivalent reflections
            fctmp=fctmp0*cexp(pi2*yt*(0.,-1.))
            argr=real(fctmp)
            argi=aimag(fctmp)
            do ic=0,1           ! generage equivalent reflections
               if(ic.eq.1) argi=-argi
               if(ic.eq.1) call vnegatei(hr,6)
               ao=fobs(j)*argr  ! real part of observed structure factor
               bo=fobs(j)*argi  ! imaginary part of observed structure factor
               if(isDFourier()) then ! difference fourier
                  ac=fcal(j)*argr ! real part of calculated structure factor
                  bc=fcal(j)*argi ! imaginary part of calculated structure factor
                  ao=ao-ac
                  bo=bo-bc
               end if
               fmax=max(fmax,fobs(j))
               l=gethkltoj(hr)
               nref=nref+1
c              write(6,*) 'l,nref=',l,nref
               call setbeq(ao,bo,l) ! set br,bi
               if(ns.eq.1.and.ic.eq.0) then
                  call setari0(ao,bo,sgobs,nind,l)
               end if
            end do
 30         continue
         end do
 20      continue
      end do
      f000=br(0)                ! this may be a value given by Fobs
      write(6,*) 'f000 used in the calculation =',f000
c     call wtvctrr('ar0',ar0,nind)
c     call wtvctrr('ai0',ai0,nind)
      return
      end

      subroutine wthklfo(idm,hr,br,bi,frmt)
      integer hr(*)
      real br,bi
      character(*) frmt
      write(6,frmt) (hr(j),j=1,idm),br,bi
      return
      end

      subroutine saxis(ih)
      dimension ih(6)
      return
      end

      subroutine setbrv_1(brv)
      character brv,brvs
      common /cbrv/ brvs
      brvs=brv
      return
      end

      subroutine getbrv_1(brv)
      character brv,brvs
      common /cbrv/ brvs
      brv=brvs
      return
      end

      subroutine ihkln(ih,hkln)
      integer ih(6),hkln,hm
      common /chkln/ iex(6),hm(6)
      ih(1)=hkln/iex(1)-hm(1)
      do i=2,6
         ih(i)=mod(hkln,iex(i-1))/iex(i)-hm(i)
      end do
      return
      end

      subroutine rdfofc         ! read independent hkl fobs
      common /cwflag/ iwflag(6)

      include 'nhmax.inc'
      include 'hklf.inc'        ! independent reflections

      integer nslct,ih(6)
      integer h4,hd
c     common /islct/ hd(3),h4(3,40),nslct
      common /islct/ hd(6),h4(3,40),nslct
      common /cnsti/ ix,ib,if,itvr,itst,nds,na4
      real afobs
      character(80) fmt
      integer hm
      common /chkln/ iex(6),hm(6)
      call getidm(idm)
      call getnsymo(nsymo)
      call geticent(icent)
      call getinp(inpu)
      call getout(iout)

      call freadi('ngr=',1,nslct,1)
      do i=1,nslct
         call freadiv('h3+i=',nde,h4(1,i),1)
      end do
      if(iwflag(3).eq.1) then
         write(iout,'(a,i3,a,20i3)') ' ngr=',nslct,' h3+i=',
     *        ((h4(i,j),i=1,nde),j=1,nslct)
      end if
      if(nde.eq.3) go to 211
      j=nde+1
      do k=1,nslct
         do  i=j,3
            h4(i,k)=0
         end do
      end do
 211  continue
      call freadr('sg0=',1,sg0,0)
      call freadr('sg1=',1,sg1,0)
      call freadiv('hm=',6,hm,0)
      if(iwflag(3).eq.1) then
         write(iout,201) fmag,sg0
      end if
c     read(inpu,100) inp,iorf,fmt
      call freadi('inp=',1,inp,1)
      call freadc('fmt=',1,fmt,0)
      if(iwflag(3).eq.1) then
         write(iout,202) inp,iorf,fmt
      end if
      do i=1,6
         if(hm(i).ne.0) go to 20
      end do
      do i=1,3
         j=i+3
         hm(i)=24
         hm(j)=10
      end do
 20   iex(6)=1
      do i=1,5
         j=6-i
         iex(j)=iex(j+1)*(hm(j+1)*2+1)
      end do
 111  i=0
      iw1=0
      if(inp.ne.5) rewind 3
c******read at least once even if nds=0 ******
c     inde=max(1,nds)
      inde=1
c*******************************
      i=0
 10   continue
      ih(4)=0
      ih(5)=0
      ih(6)=0
      if(icent.eq.1) then
         if(inp.eq.5) read(inpu,fmt) (ih(j),j=1,idm),afobs,asig
         if(inp.ne.5) read(3,fmt) (ih(j),j=1,idm),afobs,asig
      else
         if(inp.eq.5) read(inpu,fmt) (ih(j),j=1,idm),afobs,asig
         if(inp.ne.5) read(3,fmt) (ih(j),j=1,idm),afobs,asig
      end if
      if(ih(1).eq.999) go to 11
      iabsn=0
      if(nslct.gt.0) then
         if(ihslct(ih).eq.0) go to 10
      end if
      i=i+1
 16   continue
      do j=1,6
         hkln(j,i)=ih(j)
      end do
      go to 10
 11   is=i
      return
 100  format(2i2,19a4)
 102  format(6f12.6)
 103  format(22i3)
 104  format(18i3)
 105  format(2f12.6,6i3)
 200  format(1h ,'number of total reflections',i5,5x,
     *     'number of reflections used',i5/1h ,
     *     'maximum miller indices',6i5)
 201  format(1h ,'fmag=',f12.6,3x,'sg0=',f12.6)
 202  format(1h ,'inp=',i3,3x,'iorf=',i3,3x,'format=',a)
      end

      subroutine fdmin(i1,i2,i,m1,dmin,a,b,ii)
      dimension a(ii),b(ii)
c     find point nearest to i-th one within i1-12
      m1=i1
      do j=i1,i2
         d=(a(j)-a(i))**2+(b(j)-b(i))**2
         if(j.eq.i1) dmin=d
         if(d.ge.dmin) go to 10
         dmin=d
         m1=j
 10      continue
      end do
      return
      end

      subroutine intch(x1,y1,x2,y2)
      x=x1
      y=y1
      x1=x2
      y1=y2
      x2=x
      y2=y
      return
      end

      subroutine getcas(ic)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      ic=icase
      return
      end

      subroutine getplt(iplt)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      iplt=iplot
      return
      end

      subroutine gauss(p,n)
c     parameter (ndata=20000,ns=2,ntotal=(ndata-1)*10+1)
      include 'map.inc'
      real p(ns,ndata)
      common /spl1/ a(ndata,3),b(ns,ndata),u(ns,ndata),xl(ndata)

      do j=2,n-1
         a(j,1)=xl(j)
         a(j,2)=2.0*(xl(j)+xl(j-1))
         a(j,3)=xl(j-1)
c         do k=1,is
         do k=1,ns
            b(k,j)=(3.0*(xl(j-1)*xl(j-1)*(p(k,j+1)-p(k,j)))
     &           +3.0*xl(j)*xl(j)*(p(k,j)-p(k,j-1)))/(xl(j)*xl(j-1))
         end do
      end do

      do i=2,n
         d=a(i-1,2)/a(i,1)
         a(i,1)=0.0
         a(i,2)=a(i,2)*d-a(i-1,3)
         a(i,3)=a(i,3)*d
         do k=1,ns
            b(k,i)=b(k,i)*d-b(k,i-1)
         end do
         q=a(i,2)
         a(i,2)=1.0
         a(i,3)=a(i,3)/q
         do k=1,ns
            b(k,i)=b(k,i)/q
         end do
      end do
      do k=1,ns
         u(k,n)=b(k,n)
         do j=1,n-1
            u(k,n-j)=b(k,n-j)-a(n-j,3)*u(k,n+1-j)
         end do
      end do
      end

      subroutine extncr(ih,jskip)
      integer ih(*)
      common /cctr/ ctr(6,0:48),nctr
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      data pi2/6.28318/, eps/0.0001/
      integer hr(6)
c     extinction rules due to symmetry operators
      call getidm(idm)
      call getnsymo(nsymo)
      call geticent(icent)
      jskip=0
      do ns=1,nsymo
         do ic=0,icent
            call gethr(ih,hr,ns,ic)
            if(icomp(ih,hr).eq.0) go to 20
c     write(6,*) 'ih=',(ih(j),j=1,idm)
            fm=0
            do j=1,idm
               fm=fm+ih(j)*tt(j,ns,1)
            end do
            fm=fm+sign(eps,fm)
            fm=mod(fm,1.)
            if(abs(fm).gt.eps*10) then
c     write(6,*) 'jskip=1',(ih(j),j=1,idm)
               jskip=1
               return
            end if
 20         continue
         end do
      end do
c     extinction rules due to centering translations
      if(nctr.eq.0) return
      fctr=1
      fcti=0
      do i=1,nctr
         fm=0
         do j=1,idm
            fm=fm+ih(j)*ctr(j,i)
         end do
         fctr=fctr+cos(pi2*fm)
         fcti=fcti+sin(pi2*fm)
      end do
      if(abs(fctr).lt.eps.and.abs(fcti).lt.eps) then
         jskip=1
      else
         jskip=0
      end if
      return
      end

      function irefcd(ih)
      integer ih(*)
c     common /extcr/ nextnc,icef(7,48)
      common /cctr/ ctr(6,0:48),nctr
      integer rr
      common /sym/ rr(6,6,48,3),tt(6,48,3),nsymo,icent
      integer hr(6)
      complex extf,cexpf
      data pi2/6.28318/, eps/0.0001/
c     remove reflections violating extinction rules 
c     due to bravais type
      call getidm(idm)
      call getnsymo(nsymo)
      call geticent(icent)
      irefcd=0
      extf=(0.,0.)
      do i=0,nctr
         arg=0
         do j=1,idm
            arg=arg+ctr(j,i)*ih(j)
         end do
         extf=extf+cexp(pi2*(0.,1.)*arg)
c     extf=extf+cexpf(pi2*(0.,1.)*arg)  
      end do
      extf=extf/(nctr+1)
      if(abs(extf).lt.eps) return
c     remove reflections violating extinction rules due to screw and glide
      extf=(0.,0.)
      nn=0
      do ic=0,icent
         do ns=1,nsymo
            call gethr(ih,hr,ns,ic)
            if(icomp(ih,hr).ne.1) go to 20
c     symmetry operator leaving diffraction vector invariant
            xx=0
            if(ic.eq.0) then
               do i=1,idm
                  xx=xx+ih(i)*tt(i,ns,1)
               end do
            else
               do i=1,idm
                  xx=xx-ih(i)*tt(i,ns,1)
               end do
            end if
            extf=extf+cexp(pi2*(0.,1.)*xx)
c     extf=extf+cexpf(pi2*(0.,1.)*xx)
            nn=nn+1
 20         continue
         end do
      end do
c     factor of structure factor
      if(nn.eq.0) then
         write(6,*) 'ih=',(ih(i),i=1,idm)
         call quit('nn=0 in irefcd')
      endif
      extf=extf/nn
      if(abs(extf).le.eps) return
      irefcd=1
      return
      end

      integer function multpl(h)
      integer h(*)
      integer hr(6)
      integer hs
      common /cmult/ hs(6,48)
      call getnsymo(nsymo)
      call geticent(icent)
      mul=0
      do ns=1,nsymo
         do j=0,1
            call gethr(h,hr,ns,j)
            do k=1,mul
               if(icomp(hr,hs(1,k)).eq.1) then
                  go to 100
               end if
            end do
            mul=mul+1
            do k=1,6
               hs(k,mul)=hr(k)
            end do
 100        continue
         end do
      end do
      multpl=mul
      return
      end

      subroutine rdiob0(iwght)
      common /cwflag/ iwflag(6)
      include 'nhmax.inc'
      include 'hklf.inc'       ! independent reflections

      integer nslct,hm
      integer h4,hd
c     common /islct/ hd(3),h4(3,40),nslct
      common /islct/ hd(6),h4(3,40),nslct
      common /filen/ flfobs,flfcal
      common /cnsti/ ix,ib,if,itvr,itst,nds,na4
      character(80) fmt
      common /crdiob/ fmin, wg0,inp,iorf,fmt
      character(26) flfobs,flfcal
      common /chkln/ iex(6),hm(6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      call getidm(idm)
      call getnsymo(nsymo)
      call geticent(icent)
      call getinp(inpu)
      call getout(iout)
      call freadr('f000=',1,f000,1)
      call freadr('sg00=',1,sg00,0)
      call freadr('sg0=',1,sg0,0)
      call freadr('sg1=',1,sg1,0)
      call freadr('fmag=',1,fmag,0)
      if(iwflag(3).eq.1) then
         write(iout,201) f000,sg00,sg0,sg1,fmag
      end if
      f000=f000*fmag
      sg00=sg00*fmag
      sg0=sg0*fmag
      sg1=sg1*fmag
      return
 100  format(2i2,19a4)
 102  format(6f12.6)
 103  format(22i3)
 104  format(18i3)
 105  format(2f12.6,6i3)
 200  format(1h ,'number of total reflections',i5,5x,
     *     'number of reflections used',i5/1h ,
     *     'maximum miller indices',6i5)
 201  format(1h ,'f000=',f12.6,
     *     ' sg00=',f12.6,' sg0=',f12.6,' sg1=',f12.6,
     *     ' fmag=',f12.6)
 202  format(1h ,'inp=',i3,3x,'iorf=',i3,3x,'format=',a)
      end

      subroutine rdiob1(iwght,frmt,flnm1) ! read independent reflections
                                ! read hkl fo fc iph sig
      character(*) frmt,flnm1
      common /cwflag/ iwflag(6)

      include 'nhmax.inc'
      include 'hklf.inc'       ! independent reflections

      integer rhkln(6)
      real rfobs,rfcal
      character(26) flfobs,flfcal
      character(80) fmt
      common /crdiob/ fmin, wg0,inp,iorf,fmt
      common /chkln/ iex(6),hm(6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      character cidm
      logical isMem,isPatterson,isFourier,isDFourier
      data eps /0.00001/
      data pi  /3.141592/
      call getiplt(iplot)
      call getidm(idm)
c     write(cidm,'(i1)') idm
c     write(6,*) 'cidm=',cidm
c     frmt='('//cidm//'i4,4f10.1)'
      istr=istrln(frmt)
      write(6,*) 'frmt=',frmt(1:istr)
      call openfile(2,flnm1,'OLD')

      i=0
 10   continue
      if(isMem()) then
         read(2,frmt,end=20) (rhkln(j),j=1,idm),rfobs,rfcal,ang,rsig
         if(rhkln(1).eq.999) go to 11
         write(6,frmt) (rhkln(j),j=1,idm),rfobs,rfcal,ang,rsig
      else if(isFourier().or.isDFourier()) then
         read(2,frmt,end=20) (rhkln(j),j=1,idm),rfobs,rfcal,ang
         if(rhkln(1).eq.999) go to 11
         write(6,frmt) (rhkln(j),j=1,idm),rfobs,rfcal,ang
      end if
c     sig=0
      i=i+1
      if(i.gt.nhmax) go to 11
      call vcpyi(rhkln,hkln(1,i),idm)
      cs=cos(torad(ang))
      sn=sin(torad(ang))
      fobs(i)=rfobs*fmag
      fcal(i)=rfcal*fmag
      phsr(i)=cs
      phsi(i)=sn
c     wght(i)=rsig*fmag
      sig(i)=rsig*fmag
c      write(6,*) 'i,hkl,fosb,fcal,ang,wght',i,rhkln(1),rhkln(2),
c     *     rhkln(3),fobs(i),fcal(i),ang,wght(i)
      go to 10
 11   continue
 20   continue
      is=i
      write(6,*) 'number of reflections=',is
      close(2)
      return
      end

      subroutine setsmt(ismth)
      common /csmth/ ismths
      ismths=ismth
      return
      end

      subroutine getsmt(ismth)
      common /csmth/ ismths
      ismth=ismths
      return
      end

      subroutine getiplt(iplotd)
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      iplotd=iplot
      return
      end

      subroutine reduc(x)
      real x(*)
      data eps /0.00001/
      call getidm(idm)
      do ix=1,idm
         x(ix)=amod(x(ix),1.0)
         if(x(ix).le.-0.5+eps) then
            x(ix)=x(ix)+1.0
         else if(x(ix).gt.0.5)  then
            x(ix)=x(ix)-1.0
         end if
      end do
      return
      end

      integer function rcpmp1(x,rx)
c     1 for rx equivalent to x else 0
      real x(*),rx(*)
      common /cctr/ ctr(6,0:48),nctr
      data eps/0.00001/

      call getidm(idm)
      call getsym(nsymo,icent,nctr)
      rcpmp1=0
      do j=1,nctr
         do i=1,idm
c     xx=abs(x(i)-rx(i)-trc(i,j))+eps/2
            xx=abs(x(i)-rx(i)-ctr(i,j))+eps/2
            xx=mod(xx,1.)
            if(xx.gt.eps) go to 110
         end do
         rcpmp1=1
         return
 110     continue
      end do
      return
      end

      subroutine sitesy1(na,mpl,x,isk,irs) 
c     real mpl(*),x(3,*)
      real mpl(*),x(6,*)
c     integer isk(0:48,*),irs(0:48,*)
      parameter (ieqmax=300)
      integer isk(0:ieqmax,*),irs(0:ieqmax,*)
      real rx(3)
c     real rxs(3,48),td(3)
      real rxs(6,300),td(6)
      integer rcpmp1
      data td/6*0./

      call getidm(idm)
      call getnsymo(nsymo)
      call geticent(icent)
      do i=1,na
         nsym=0
         nrx=1
         isk(nrx,i)=1
         call savex(x(1,i),rxs(1,nrx))
         do ic=0,icent
            do ns=1,nsymo
               call getrx(x(1,i),rx,ns,ic)
               call reduc(rx)
               do irx=1,nrx
                  if(rcpmp1(rx,rxs(1,irx)).eq.1) go to 120
               end do
               nrx=nrx+1
               if(nrx.gt.ieqmax) then
                  call quit('number of equivalent peak > ieqmax')
               end if
               call savex(rx,rxs(1,nrx))
               if(ic.eq.0) then
                  isk(nrx,i)=ns
               else
                  isk(nrx,i)=ns+nsymo
               end if
 120           continue
               if(rcpmp1(rx,x(1,i)).eq.1) then
                  nsym=nsym+1
                  if(ic.eq.0) then
                     irs(nsym,i)=ns
                  else
                     irs(nsym,i)=ns+nsymo
                  end if
               end if
            end do
         end do
         isk(0,i)=nrx
         irs(0,i)=nsym
         mpl(i)=1

c     if(icent.eq.1) then
c     c     compress symmetry element > nsymo
c     n=0
c     do 200 j=1,isk(0,i)
c     if(isk(j,i).gt.nsymo) go to 201
c     n=n+1
c     200        continue
c     201        continue
c     c     n: number of equivalent positions generated by given operators
c     c     isk(0,i): number of equivalent positions generated by given
c     c     symmetry operators and inversion
c     mpl(i)=float(isk(0,i))/(float(n)*2)
c     isk(0,i)=n
c     end if

      end do
      write(6,*) 'site symmetry group'
      do i=1,na
         write(6,'(a,i4,a,(30i3))') 'atom',i,'  ',
     *        (irs(j,i),j=1,irs(0,i))
      end do
      write(6,*) 'equivalent position'
      do i=1,na
         write(6,'(a,i4,a,(30i3))') 'atom',i,'  ',
     *        (isk(j,i),j=1,isk(0,i))
      end do
      write(6,*) 'multiplicity'
      do i=1,na
         write(6,'(a,i4,a,f10.5)') 'atom',i,'  ',mpl(i)
      end do
      return
      end

      subroutine setvol(vold)
      common /cvol/ vol
      vol=vold
      return
      end

      function getvol()
      common /cvol/ vol
      getvol=vol
      return
      end


      subroutine setsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtrd=ismtr
      return
      end

      subroutine getsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtr=ismtrd
      return
      end

      subroutine smtrh(h,hd,ind) ! dummy subroutine for crystals
      integer h(*),hd(*)
      integer s,t,ht(6)
      common /csmtrh/ s(6,6),t(6,6)
      return
      end

      subroutine smtrx(x,xd,ind) ! dummy subroutine for crystals
      real x(*),xd(*)
      call vcpyr(x,xd,6)
      return
      end

      subroutine getsys(isysd)  ! subroutine for crystals
      common /cisys/ isys
      isysd=0
      return
      end            

      subroutine setsys(isysd)  ! subroutine for crystals
      common /cisys/ isys
      isys=isysd
      return
      end            

      subroutine ctopx(x)       ! dummy subroutine for crystals
      real x(*)
      return
      end

      subroutine ptocx(x)       ! dummy subroutine for crystals
      real x(*)
      return
      end

      subroutine cxtoxyz0c(cr,dxyz,ixm,iym,izm)  ! c order
      implicit integer (h)
      real cr(0:*)
      real dxyz(0:*)
      integer hmax(6)
      include 'drwmap.inc'
      real hscl
      real x(6),xd(6),rx(6)
c     integer hd(6)
c     common /ctr/ itr(6,6)
      common /cjbc/ jbc,xs(6),iax(6)
      real tt(6)
      data tt /6*0.0/
      data eps /0.0001/
c      write(6,*) 'jbc in cxtoxyz0c=',jbc
c      stop

      call gethmax(hmax)
      call getnsymo(nsymo)
      call geticent(icent)
      divx=(xmax-xmin)/(ixm)    ! reset xdiv,ydiv,zdiv
      divy=(ymax-ymin)/(iym)
      divz=(zmax-zmin)/(izm)
c     write(6,*) 'divx,divy,divz=',divx,divy,divz
      write(6,*) 'ixm,iym,izm=',ixm,iym,izm
c     write(6,*) (cr(i),i=1,10)
      if(icent.eq.0) then
         icd=1
      else
         icd=2
      end if
c     write(6,*) '****iax(1),iax(2)=',iax(1),iax(2)
      hkl=0
      do h=0,ixm-1
         do k=0,iym-1
            do l=0,izm-1
               call zerovr(x,6)
               call zerovr(rx,6)
               x(iax(1))=xmin+h*divx
               x(iax(2))=ymin+k*divy
               x(iax(3))=zmin+l*divz ! x  : user coordinates
               call getxtoxd(x,xd,jbc) ! xd : nD coordinates
               call vaddr(xd,xs,xd,6) ! origin shift in nD
               dxyz(hkl)=0
               call grad(cr,xd,dnsty) ! density at xd    
c              call wtvctrr('xd',xd,6)
               dxyz(hkl)=dxyz(hkl)+dnsty*hscl
               hkl=hkl+1
            end do
         end do
      end do
c     write(6,*) (dxyz(i),i=1,10)
      return
      end

      subroutine cxtoxyz0f(cr,dxyz,ixm,iym,izm)  ! fortran order
      implicit integer (h)
      real cr(0:*)
      real dxyz(0:*)
      integer hmax(6)
      include 'drwmap.inc'
      real hscl
      real x(6),xd(6),rx(6)
c     integer hd(6)
      common /ctr/ itr(6,6)
      common /cjbc/ jbc,xs(6),iax(6)
      real tt(6)
      data tt /6*0.0/
      data eps /0.0001/
      call gethmax(hmax)
      call getnsymo(nsymo)
      call geticent(icent)
      divx=(xmax-xmin)/(ixm)
      divy=(ymax-ymin)/(iym)
      divz=(zmax-zmin)/(izm)
c     write(6,*) (cr(i),i=1,10)
      if(icent.eq.0) then
         icd=1
      else
         icd=2
      end if
c     write(6,*) '****iax(1),iax(2)=',iax(1),iax(2)
      hkl=0
      do l=0,izm-1
         do k=0,iym-1
            do h=0,ixm-1
               call zerovr(x,6)
               call zerovr(rx,6)
               x(iax(1))=xmin+h*divx
               x(iax(2))=ymin+k*divy
               x(iax(3))=zmin+l*divz
               call getxtoxd(x,xd,jbc)
               call vaddr(x,xs,xd,6) ! origin translation
               call grad(cr,xd,dnsty) ! density at xd    
               dxyz(hkl)=0
               dxyz(hkl)=dxyz(hkl)+dnsty*hscl
               hkl=hkl+1
            end do
         end do
      end do
      return
      end

      subroutine getxtoxd(x,xd,jbc)
      real x(*),xd(*)
      if(jbc.eq.1) then
         call vcpyr(x,xd,6)
      else if(jbc.eq.-1) then
         call ottrfm1(x,xd)        ! transformed coodinates
      else if(jbc.eq.-2) then
         call ottrfm0(x)        ! q coodinates transformation
         call getxe(x,xd)       ! original lattice coodinates
      else
         write(6,*) 'jbc=',jbc
         call quit('jbc should be 1,-1,or -2')
      end if
      return
      end

c     transformation of coordinates by axis transformation
      subroutine ottrfm1(x,xd)
      real x(6),xd(6)
      common /ctr/ itr(6,6)
c     call wtmtrxi('itr',itr,6)
      do i=1,6
         xd(i)=0
         do j=1,6
c     xd(i)=xd(i)+x(j)*itr(j,i)
            xd(i)=xd(i)+itr(i,j)*x(j)
         end do
      end do
      return
      end

      subroutine ottrfm0(q)
c     orthogonal transformation in external and internal spaces
      real q(6),qd(6)
      common /otrfm/ si(6,6),sr(6,6),ti(6,6),tr(6,6)
      integer getjbc
      call mtmplmno(q,sr,qd,1,6,6)
      if(getjbc().eq.-1) then
         call vcpyr(qd,q,6)
      else if(getjbc().eq.-2) then
         call mtmplmno(qd,tr,q,1,6,6)
      else
         write(6,*) 'jbc=',getjbc()
         call quit('strange jbc in ottrfm0')
      end if
      return
      end

      subroutine cxtoxyz(cr,dxyz,ixm,iym,izm)
      implicit integer (h)
      real cr(0:*)
      real dxyz(0:*)
      call cxtoxyz0c(cr,dxyz,ixm,iym,izm)  ! c order
      return
      end

      subroutine cxtoxy(cr,dxy,ixm,iym)
      implicit integer (h)
      real cr(0:*)
      real dxy(0:ixm,0:iym)
      integer hmax(6)
      include 'drwmap.inc'
      real hscl
      real x(6),xd(6),rx(6)
      common /ctr/ itr(6,6)
      common /cjbc/ jbc,xs(6),iax(6)
      real tt(6)
      data tt /6*0.0/
      data eps /0.0001/
      write(6,*) 'jbc in cxtoxy=',jbc
      call gethmax(hmax)
      call getnsymo(nsymo)
      call geticent(icent)
      divx=(xmax-xmin)/(ixm)
      divy=(ymax-ymin)/(iym)
c     write(6,*) (cr(i),i=1,10)
      if(icent.eq.0) then
         icd=1
      else
         icd=2
      end if
c     write(6,*) '****iax(1),iax(2)=',iax(1),iax(2)
      do h=0,ixm-1
         do k=0,iym-1
            call zerovr(x,6)
            call zerovr(rx,6)
            x(iax(1))=xmin+h*divx
            x(iax(2))=ymin+k*divy    ! x  : user coordinates
c           write(6,*) 'jbc in loop=',jbc
            call getxtoxd(x,xd,jbc)  ! xd : nD coordinates
            call vaddr(x,xs,xd,6)    ! origin shift in nD
            call grad(cr,xd,dnsty)            
            dxy(h,k)=dnsty*hscl
c           write(6,*) 'dxy=',dxy(h,k),' dnsty=',dnsty,' hscl=',hscl
         end do
      end do
      do k=0,iym-1
         dxy(ixm,k)=dxy(0,k)
      end do
      do h=0,ixm
         dxy(h,iym)=dxy(h,0)
      end do
c     do h=0,ixm
c     write(6,'(10f8.2)') (dxy(h,k),k=0,9)
c     end do
c     write(6,*) ' '
      return
      end

      subroutine getac(al,cl)
      common /cabd/ a,b,c,cosa,cosb,cosc
      al=a
      cl=c
      return
      end

      subroutine gethkllen(hkl,qe,nt)
      integer hkl(6,*)
      real qe(*)
      real yl(3)
      do i=1,nt
         call getyl(hkl(1,i),yl,1)
         qe(i)=getyllen(yl)   ! get reciplocal vector length in A
      end do
      return
      end


