c
c      Fourier or MEM map of quasi-crystals and semi-quasicrystals
c
c      This program generates a mem map, fourier map or patterson map
c            (********isys=0 does not work at present*****)
c
c       input format as follows
c          key=value where key is key work shown below
c          for character value it should be enclosed by '
c            as file1='filename1'
c          for numerical value it is like ndm=1 or f000=123.4
c
c
c                            input data
c     (00) file1
c         file1:name for hkl-fo-fc-phase-sig file
c     (01) file2
c         file2: file name for nD density file for mem
c     (1) title
c            **** for semi-quasicrystals (isys=0)
c     (2_1) isys,a,b,c,cosa,cosb,cosc (for isys=0)
c         isys :  0 semi-quasicrystals
c              ******* if isys>0 skip (2') and (2")
c     (2_2) ndm
c          a*,b*,c* components of a wave vector
c     (2_3) k
c          k: a*,b*,c* components of a wave vector
c          ********** repeat (2_3) ndm times
c     (2_4)  isetting
c           isetting : 1 c axis unique, 2 b axis unique, 3 a axis unique
c     (2_5)  v
c           v : matrix ((v(i,j),j=1,3),i=1,6)
c           6x3 U matrix, which gives the a1-a3 coefficient of
c           6 unit vectors in reciprocal space, is calculated from
c           iseging and a*,b*,c*,cosa*,cosb*,cosc* and k1,k2,k3
c           6x3 V matrix, which gives the  a4-a6 coefficient of
c           6 unit vectors in reciprocal space, should be given
c           here

c           first 6x3 part of 6x6 R matrix is U while
c           second 6x3 part is V
c
c           in the standard setting for modulated structure
c           first 3x3 part of V is zero matrix while second 3x3
c           part is unit matrix for many cases, but may be
c           different for triclinic,monoclinic,hexagonal system
c           in idm=5
c           for example, k1 and k2 are in the a*-b* plane in hexagonal system,
c           we may take v(4,1)=a*,    v(4,2)=0,           v(4,3)=0
c                       v(5,1)=-a*/2, v(5,2)=sqrt(3)a*/2, v(5,3)=0
c                       v(6,1)=v(6,2)=v(6,3)=0
c           the best v(1,1)-v(3,3) are sample-dependent for which
c           the modulation wave should be parallel to the internal space
c           v(1,1)=v(1,2)=...=v(3,3)=0 gives the standard setting for
c           modulated structure but this is not appropriate for
c           semi-quasicrystals      
c            **** for quasicrystals (isys>0)
c     (2_1) isys,a,c             for quasicrystals (isys>0)
c         isys: 1:icosahedral 2:icosahedral (cubic)
c               3:decagonal 4:octagonal 5: dodecagonal
c         a: lattice parameter (in angstrom)
c         c: lattice parameter (in angstrom) for dihedral lattice
c     (3) nsymo,icent,brv
c         nsymo: number of generators in symmetry operations
c         icent: give 1 for centrosymmetryc
c                give 0 otherwise
c         brv  : 'p','i' or 'f'
c              ****** see (10) ******
c     (4) symmetry operator in the format of i.t.
c             for example x,y,z,t,u,v; x+1/2,y,z+1/2,t,u,v etc.
c             ********    repeat (4) nsymo times
c     (5) f000,sg00,sg0,sg1, fmag
c            f000: fc at the origin
c            if observed structure factor includes f000, this is replaced
c            sg00: sigma for f000  (sg0 : sigma=sqrt(f000)*sg00 is used)
c            sg0: sigma=sg0     (sg0 : sigma=sqrt(fo)*sg0 is used)
c            sg1: sigma=sg1   for strongest (Fcal) reflections
c         fmag: f000,sg00,sg0,sg1,fobs,fcal are
c               magnifiled by this factor
c     (6) jbc
c         jbc: kind of map  1 : axes are unit vectors for ext. and int. spaces
c                          -1 : axes specified in (7) (lattice vectors)
c
c         **if jbc=-1 or jbc=-2 then give (7) else skip (7)**
c     (7)  s1, s2, s3, s4 : orthogonal transformation matrix for q1-q6
c            ((s1(i,j),j=1,3),i=1,3) etc.
c             s1, s2 etc. should be in different lines
c             total 6x6 matrix is given by 
c                     s1,s3
c                     s4,s2
c          for 3-fold map  (isys=2 external and internal spaces // x and y)
c             s1= 0.5773 0.5773 0.5773 0.7071 -0.7071 0. 0.4082 0.4082 -0.8165
c             s2= 0.5773 0.5773 0.5773 0.7071 -0.7071 0. 0.4082 0.4082 -0.8165
c             s3= 0. 0. 0. 0. 0. 0. 0. 0. 0.
c             s4= 0. 0. 0. 0. 0. 0. 0. 0. 0.
c          other possiblilities
c          for 5-fold map (sys=1 two 5-fold axes in 6D // x and y)
c             s1=1. 0. 0. 0. 1. 0. 0. 0. 0.7071
c             s2=1. 0. 0. 0. 1. 0. 0. 0. 0.7071
c             s3=0. 0. 0. 0. 0. 0. 0. 0. -0.7071
c             s4=0. 0. 0. 0. 0. 0. 0. 0. 0.7071
c          for 2-fold map (sys=2 two 2-fold axes in 6D // x and y)
c             s1=0.8506 0. 0. 0. 1. 0. 0. 0. 1.
c             s2=0.8506 0. 0. 0. 1. 0. 0. 0. 1.
c             s3=-0.5257 0. 0. 0. 0. 0. 0. 0. 0.
c             s4=0.5257 0. 0. 0. 0. 0. 0. 0. 0.
c          for 3-fold map (sys=2 two 3-fold axes in 6D // x and y)
c             t1=0.8650 t2=0.5257 
c             s1=0.4994 0.4994 0.4994 0.7071 -0.7071 0. 0.4082 0.4082 -0.8165
c             s2=0.4994 0.4994 0.4994 0.7071 -0.7071 0. 0.4082 0.4082 -0.8165
c             s3=-0.3034 -0.3034 -0.3034 0. 0. 0. 0. 0. 0.
c             s4= 0.3034  0.3034  0.3034 0. 0. 0. 0. 0. 0.
c               (see  (13))
c          if jbc=-2 then give (7') else skip (7') (for projected structures)
c     (7')  t1, t2, t3, t4 : orthogonal transformation matrix for q1'-q6'
c           which is transformed by s matrix defined by (7) (q'=sq)
c           for 5-fold map   24.095 = arctan(1/sqrt(5))
c                            cost=cos(24.095) sint=sin(24.095)
c           t1= 1 0 0 0 1 0 0 0  cost
c           t2= 1 0 0 0 1 0 0 0  cost
c           t3= 0 0 0 0 0 0 0 0 -sint
c           t4= 0 0 0 0 0 0 0 0  sint

c           for 2-fold or 3-fold  axis
c           cost=     sint=   (2-fold)
c           cost=     sint=   (3-fold)
c
c           t1= cost 0 0 0 1 0 0 0 1
c           t2= cost 0 0 0 1 0 0 0 1
c           t3= sint 0 0 0 0 0 0 0 0
c           t4=-sint 0 0 0 0 0 0 0 0
c
c     (8) iplot, hmax, ismtr,pkmin,rwmin (for iplot=5)
c         iplot =1, -1:  patterson map
c               =2, -2:  fourier map
c               =3, -3:  difference fourier map
c               =4, -4:  Mem map starting from flat electron density
c               =5, -5:  MEM and LDEM starting from the result of LDEM
c         positive and negative iplot generate a 2D and 3D maps
c         if iplot < 0 .win and .rho file is generated

c          hmax(1-6): number of points along principal axes
c          ismtr : order of similarity transformations
c              qext => qext*tau**ismtr
c              qint => qint*tau**-ismtr
c              xext => xext*tau**-ismtr
c              xint => xint*tau**ismtr
c          pkmin : minimum peak height fraction to be printed in  Fourier
c                           or Difference Fourier map
c         (real minimum peak hekgit is given by pkmax*pkmin
c          where pkmax is the maximum peak density)      
c                  abs(peak density) > pkmax*pkmin is printed
c          rwmin  : rw factor below which LEM method works
c                  ******(pkmin > 0)********        
           
c     ***** if abs(iplot)=1,2,3  skip (9)*****
c     (9) ex1,ex2,nex1,nex2
c            ex1,ex2:  constants needed for smooth convergence
c            nex1: number of iterations in imode=1 (Yamamoto)
c            nex2: number of iterations in imode=2 (Sakata)
c     ***** imode=1 mode is dangerous for accurate electron density map
c     ***** imode=2 mode should be used for MEM

c     (9') icont
c            icont: 0  initial nD map is calculated
c                   1: initial nD map is read from a file

c     (10) qimax, frmt
c          qimax: maximum q vector to be used to draw map
c          frmt: file format for hkl, fobs, fcal, phase, sig 

c     (11) ldev iwhkl iprj
c          ldev : 1: B&W Postscript printer 2: color Postscript printer
c          iwhkl  : 0 no output, 1: output of hkl fo fc obtained from MEM
c                 : -1: output of hkl fc fc for all calculated points
c                 ******  fo=fc in the present version ******
c          iprj  : 0 for a map in a cut plane, iprj=n means nD projection
c               ******  if iprj=0 skip (11')
c     (11') h 
c           h : Miller indices for projection axis
c          ****** repeat (11') iprj times *****
c          for 5-fold projection in external space iprj=2
c          h=1 0 0 0 0 0 and h=0 1 1 1 1 1   (isys=1)
c          for 2-fold projection in external space iprj=2
c          h=0 0 0 -1 0 1 and h=0 1 -1 0 0 0 (isys=2)

c     (12)  ng   :    number of graphs in a sheet  (ng < 10)
c                  ****if ng=0 no map is written****
c     (13) iax ipers
c         iax(1),iax(2),,,iax(6) : q coordinates
c         ipers : 0 no perspective veiw 1 add psespective view 

c         density in 3D space spanned by iax(1),iax(2),iax(3) axes is calculated

c         if jbc > 0 or jbc < 0
c         for drawing 3D density map.
c         for 5-fold map iax=3 6 1 2 4 5 (isys=1)
c         for 2-fold map iax=1 4 2 3 5 6 (isys=2)
c         for 3-fold map iax=1 4 2 3 5 6 (isys=2) (see (7))
c         for 5-fold map in external space iax=1 2 3 4 5 6 (isys=1)
c         for 2-fold map in external space iax=1 2 3 4 5 6 (isys=2)
c           
      
c     (14) iaver, ismth
c ****** note that iaver or ismth >0 takes much time for 3D maps (iplot<0)*****
c          iaver : 0 noaverage 1 average 2 equivalent points in the unit cell
c                    in the calculation of the density
c          ismth : 0 no smoothing
c                  1 3 point smoothing (average)
c                  2 5 point smoothing (average)
c     (11) x       :coordinates of origin
c     (12) xe, u   :coordinates of origin and displacement
c     (13) xi, v   :coordinates of origin and displacement
c
c     (14) q       :origin shift in external and internal spaces

c           
c            iplot > 0      
c     (15) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv 
c          xmin:  minimum coordinate along x-axis (in a)
c          xmax:  maximum coordinate along x-axis (in a)
c          ymin:  minimum coordinate along y-axis (in a)
c          ymax:  maximum coordinate along y-axis (in a)
c
c            unit vectors (ax,.. bx.. ) in the external and internal spaces
c            are as follows
c            1) for icosahedral case
c                  ax=a*a1 ay=a*a2 az=a*a3 bx=a*a4 by=a*a5 bz=a*a6
c            2) for decagonal case
c                  ax=2a*a1/sqrt(5) ay=2a*a2/sqrt(5) az=c*a5
c                  bx=2a*a3/sqrt(5) by=2a*a4/sqrt(5) 
c            3) for octagonal case
c                  ax=a*a1 ay=a*a2 az=c*a5
c                  bx=a*a3 by=a*a4 
c            4) for dodecagonal case
c                  ax=2a*a1/sqrt(6) ay=2a*a2/sqrt(6) az=c*a5
c                  bx=2a*a3/sqrt(6) by=2a*a4/sqrt(6) 
c                xmin=1 means ax A
c          hscl: hight scale
c          xdiv: length of a mesh edge in the unit of ax, etc.
c          ydiv: length of a mesh edge in the unit of ax, etc.
c          ri1,ri2: radii for the first (x) and second (y) axes
c          hscl: scale factor for the electron density
c          dtheta: interval of theta in polar coordinate system
c          dphi  : interval of phi in polar coordinate system

c           if iplot < 0 else skip (15)
c     (15) xmin,xmax,ymin,ymax,zmin,zmax,hscl,xdiv,ydiv,zdiv
c         xmin,..,zmax:coordinate reanges along x,y and z axes
c         hscl : scale for F
c         xdiv, ydiv, zdiv : intervals of the scale in x and y axes

c          if iplot < 0 skip (16)-(17) else skip (16)-(17)
c     (16) hmin,div,lf
c         hmin: minimum hight (density) in the division
c         div : interval of hight
c         lf  : number of contour lines
c     (17) amagx,amagy,shftxb,shftyb,shftxa,shftya
c          amagx:  magnification for x
c          amagy:  magnification for y
c          shftxb: shift of origin along x before plot in cm
c          shftyb: shift of origin along y before plot in cm
c          shftxa: shift of origin along x after each plot in cm
c          shftya: shift of origin along y after each plot in cm
c      **0*****repeat (9)-(17) ng times********
      program qcfour
      include 'nhmax.inc'
      include 'ixmax.inc'
      parameter (ixmax1=ixmax+1)
      include 'drwmap.inc'

      real det
      character(80) flnm1,flnm2,flnm3,flnm4
      common /cfilename/ flnm1,flnm2,flnm3,flnm4
      character(80) prefix

      character(40) frmt
      character brv
c      include 'nhmax.inc'
c      include 'symstfsort.inc'
      real foi(nhmax)           ! fo for independent reflections
      real sgi(nhmax)           ! sigma for independent reflections
      integer hkli(6,nhmax)     ! hkl for independent reflections
      common /cfoi/ hkli,foi,sgi
      include 'nrmax.inc'
      include 'qsort.inc'
c     include 'nhklt.inc'
      include 'abcdri.inc'
      
      real cxy(ixmax1*ixmax1),dxy(ixmax1*ixmax1) ! 2D array for 2D map
      real hmin(1),div(1),x0(6),q(6)
      real qd(6),xe(6),xi(6),xd(6),xf(6),qe(6),qi(6),x1(6)
      integer hmax(6)
      integer lf(1)
      character(80) title
      integer pcol,sctmdl
      common /cpcol/ pcol
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq

      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      common /cjbc/ jbc,xs(6),iax(6),ipers
      real s1(3,3),s2(3,3),s3(3,3),s4(3,3)
      common /otrfm/ si(6,6),sr(6,6),ti(6,6),tr(6,6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
c
      common /ccol/ icol
      common /cwflag/ iwflag(7)
      integer q10,qm,qc
      common /cdrwed/ q10(6),qm(6),qc(6)

      real am(0:5000),bm(0:5000)
      character(100) rfform
      common /const0/ pi2,dmgn,irs(6)
      data ic1/1/,ic2/1/,ic3/0/
      common /cproj/ iproj, px, py, pz, shftz
      dimension ipv(6),ind(6,2),piv(6),b(6)
      integer mm(7)
      real xmn(6),xmx(6)
      common /cdenmodMem/ ic,n1,n2,ex
      common /cnex/ nex1,nex2,ex1,ex2

      integer hprj(6,6)
      common /chprj/ iprj, hprj

      logical isMem,isFourier,isDFourier,isPatterson
      integer getntf0,getntx0,getnt
      character char
      data iax /1,2,3,4,5,6/
      parameter (tau=1.618034)
      data q10/0,0,0,0,1,0/
      data qm/1,0,0,-1,0,0/
      data qc/1,1,1,1,0,0/
      data pi/3.141592/
c     data eps / 0.000001/
      pi2=2*getpi()

      pcol=1
      icol=10
c
      call setinp(5)
      call setout(6)

      sctmdl=1
      call freadc('file1=',1,flnm1,1)
      write(6,'(2a)') 'hkl-fo-fc file:',flnm1
      call freadc('file2=',1,flnm2,1)
      write(6,'(2a)') 'density file:',flnm2
      call getflnmprfx(flnm2,prefix,n) ! fileneme prefix
      flnm3(1:n+5)=prefix(1:n)//'.win'//char(0)
      flnm4(1:n+5)=prefix(1:n)//'.rho'//char(0)
c      write(6,*) 'flnm3=',flnm3
      write(6,*) 'flnm3=',flnm3(1:n+5),',','flnm4=',flnm4(1:n+5)
      lfn2=istrln(flnm2)
c     read(5,'(a)') title
      call freadc('title=',1,title,1)
      call settitle(title)  ! set title for writetitle
      write(6,'(a)') title

      call freadi('isys=',1,isys,1)
      call setsys(isys)

      if(isys.eq.0) then
         call readlc()
         call rdkvctr(ndm)
      else
         call freadr('a=',1,al,0)
         cl=al
         if(isys.ge.3) then
            call freadr('c=',1,cl,0)
         end if
         call setac(al,cl)
      end if
      if(isys.le.2) then
         write(6,*) 'isys=',isys,' a=',al
      else 
         write(6,*) 'isys=',isys,' a=',al,' c=',cl
      end if

      call setsmh(isys)
      call setsmx(isys) 
      eps1=0
      eps2=0
      eps3=0
      eps4=0
      call seteps(eps1,eps2,eps3,eps4)
      call setd()
      call setdei()
      call intskp()
      if(isys.le.2) cl=al
c     call mtrtns(al,cl)
      call mtrtns()
c     call getdet(det)
      if(isys.le.2) then
         idm=6
         nde=3
      else
         idm=5
         nde=2
      end if

c       sds: deformation matrix for unit vectors in 6d space
c       d,sds sent to getq through common area
c***
      call setiwflag(1)     ! printout symmetry operators
      call rdsymo(na,brv)
      if(isys.le.2) then
         call setidm(6)
      else
         call setidm(5)
      end if
      call setbrv(brv)
      call freadr('f000=',1,f000,1)
      call freadr('sg00=',1,sg00,0)
      call freadr('sg0=',1,sg0,0)
      call freadr('sg1=',1,sg1,0)
      call freadr('fmag=',1,fmag,0)
      write(6,'(a,f8.2,3x,a,f8.4,3x,a,f8.4,3x,a,f8.4,a,3x,f12.4)')
     *     'f000=',f000,' sg00=',sg00,' sg0=',sg0,' sg1=',sg1,
     *     ' fmag=',fmag

      call freadi('jbc=',1,jbc,1)
      write(6,*) 'jbc=',jbc
      call setjbc(jbc)

      if(isys.eq.0) then        ! for semicrystals
         call freadi('isetting=',1,isetiing,1)
         call setumtrx(isetting)  ! set 6x3 u matrix
         call readVmtrx()
         call setdqmtrx()
         call setdr()
      else
         call setdr()
      end if
      call getdet(det)
c     write(6,*) 'det after setdr',det

      if (jbc.lt.0) then
         call readst1234() ! read s and t matrices
      end if

      call freadi('iplot=',1,iplot,1)
      call freadiv('hmax=',6,hmax,0)

      write(6,'(a,i2,a,6i4)') 'iplot=',iplot,' hmax=',hmax
      if(isys.ne.0) then
         call freadi('ismtr=',1,ismtr,0)
         write(6,*) 'ismtr=',ismtr
      else
         ismtr=0
      end if

      call getidm(idm)
      call setng(hmax)          ! for lodem
      
      call calmm(hmax,idm)      ! set mm, hmax, idm
      call setitt(hmax)         ! set non primitive translation for grid points
      
      call freadr('pkmin=',1,pkmin,0)
      write(6,'(a,f10.6)') ' pkmin=',pkmin
      if(pkmin.le.0) then
         call quit('pkmin should be positive')
      end if
      
      if(iplot.eq.0) then       ! initial flat density
         call quit('do not support iplot=0')
      end if
      
      rwmin=0
      if(isMem()) then
         if(abs(iplot).eq.5) then
            call freadr('rwmin=',1,rwmin,0)
            write(6,*) 'rwmin=',rwmin
         end if
         iaster=0
      else if(isFourier().or.isDFourier()) then
         iaster=0
      end if
      call setrwmin(rwmin)
      
      nt=getnt()
      if(nt.gt.nts) then
         write(6,*) 'nt=',nt,' nts=',nts
         call quit('nt > nts')
      end if
      
      if(isMem()) then          ! MEM
         call freadr('ex1=',1,ex1,1)
         call freadr('ex2=',1,ex2,0)
         call freadi('nex1=',1,nex1,0)
         call freadi('nex2=',1,nex2,0)
         write(6,'(2(a,e10.3,3x),2(a,i3,3x),a,i1)')
     *        'ex1=',ex1,'ex2=',ex2,'nex1=',nex1,'nex2=',nex2
      else
         ex1=0
         ex2=0
      end if
      call freadi('icont=',1,icont,1)
      write(6,*) 'icont=',icont
      call setvol()
c     call quit('after setvol()')

      call setsmtr(ismtr)
      if(isys.le.2.and.brv.eq.'p'.and.mod(ismtr,3).ne.0) then
         call quit('ismtr must be 3*n for brv=p isys=1,2')
      end if
      
      call freadr('qimax=',1,qimax,1)
      call freadc('frmt=',1,frmt,0)
      write(6,*) 'qimax=',qimax, ' frmt=',frmt
      
      call freadi('ldev=',1,ldev,1)
      call freadi('iwhkl=',1,iwhkl,0)
      call freadi('iprj=',1,iprj,0)
      write(6,'(3(a,i3))') 'ldev=',ldev, ' iwhkl=', iwhkl,' iprj=',iprj
      do i=1,iprj
         call freadiv('h=',6,hprj(1,i),1)
         call wtvctrif('h=',hprj(1,i),6,'(6i3)')
      end do

      call rdhklq(qimax,frmt,iaster,flnm1)   ! read hkl fo fc phase
c     if(iplot.eq.0) then
c     check whether the density file can open for MEM
      call openfileu(3,flnm2,'UNKNOWN') ! for unformatted file
      close(3)
c     end if
      
c     generate equivalent reflections
c     write(6,*) 'f000=',f000
c     stop

      call symstf()
c     f000=getf0()
      call setsmopprm()         ! set symmetry operators for primitive lattice
      call setindfindx()        ! set independent structure factor index
      call setindxindx()        ! set independent grid index
c     write(6,*) 'ntf0=',getntf0(),' ntx0=',getntx0()
**************************for check
      if(isMem()) then          ! MEM initialization
         if(icont.eq.0) then
            if(abs(iplot).eq.4) then
c     initial flat density and its independent part and set dmin dmax
               call initcr0(f000)
c     call initcr4(f000) ! for test
            else if(abs(iplot).eq.5) then
c     initial non-flat density and its independent part and set dmin dmax
               call initcr0(f000)
c     call initcr4(f000) ! for test
            end if
         else if(icont.eq.1) then
c     initial density is read from a file
            call readdxyz(flnm2,br,nt)
            call setxr0()
            call getdmx()       ! get new dmin, dmax
         else
            write(6,*) 'icont=',icont
            call quit('icont should be 0 or 1')
         end if
      else
c     Fourier or Patterson map
         if(icont.eq.0) then
            call initcr1()      ! calculate Fourier or Patterson map
         else if(icont.eq.1) then
c     initial density is read from a file
            call readdxyz(flnm2,br,nt)
            call setxr0()
            call getdmx()       ! get new dmin, dmax
         else
            write(6,*) 'icont=',icont
            call quit('icont should be 0 or 1')
         end if
      end if
      
      if(isMem()) then
c     choose density modification function for MemLde
         call chsdenmodf(2)
         call memloop(f000,c2d,flnm2,rfact,rwfact)
      end if
      
      call getbrfromxr0()       ! get br for pksearch
      call pksearchnd(br,pkmin) ! peak search
      call getpeakcrd(br,xmn,xmx) ! peak coordinates
      write(6,'(a,f10.2,a,6f10.5)') 'dmin=',getdmin(),' xmn=',xmn
      write(6,'(a,f10.2,a,6f10.5)') 'dmax=',getdmax(),' xmx=',xmx
      call wrtdxyz(flnm2,br,nt) ! write nD density into a file

      if(isMem()) then
         call rfct(c2d,rfact,rwfact,iwhkl)
      end if
c     write(6,*) 'isys0=',isys
      call getsys(isys)
c     write(6,*) 'isys1=',isys
      call drwmap(br,ldev)      ! draw maps
      call exit(0)
      stop
      end

      subroutine readdxyz(flnm2,br,nt)  ! read nD density file
      character(*) flnm2
      real br(0:*)
      call openfileu(3,flnm2,'OLD') ! for unformatted file
      read(3) (br(i),i=0,nt-1)
      close(3)
      return
      end

      subroutine wrtdxyz(flnm2,br,nt)
      character(*) flnm2
      real br(0:*)
      call openfileu(3,flnm2,'UNKNOWN') ! for unformatted file
      write(unit=3) (br(i),i=0,nt-1)
      close(3)
      return
      end

      integer function getixmax()
      include 'ixmax.inc'
      getixmax=ixmax
      return
      end

      subroutine setjbc(jbcd)
      common /cjbc/ jbc,xs(6),iax(6),ipers
      jbc=jbcd
      return
      end

      integer function getipers()
      common /cjbc/ jbc,xs(6),iax(6),ipers
      getipers=ipers
      return
      end

      subroutine readitr(det)
      common /ctr/ itr(6,6)
      integer getjbc
      data itr /
     *     1,0,0,0,0,0,
     *     0,1,0,0,0,0,
     *     0,0,1,0,0,0,
     *     0,0,0,1,0,0,
     *     0,0,0,0,1,0,
     *     0,0,0,0,0,1/
      call unitmtrxi(itr,6)  ! itr : 6x6 unit matrix
      jbc=getjbc()
      if(jbc.lt.0) then
         do i=1,6
            read(5,*) (itr(i,j),j=1,6)
         end do
         write(6,'(6i3)') ((itr(i,j),j=1,6),i=1,6)
      end if
      call trnspi(itr)  ! take transpose for transformation matrix for x      
      return
      end

      integer function getjbc()
      common /cjbc/ jbc,xs(6),iax(6),ipers
      getjbc=jbc
      return
      end

      real function getf0()
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      getf0=f000
      return
      end

      subroutine drwmap(cr,ldev)  ! draw electron density map
      real cr(0:*)
      include 'nhmax.inc'
      include 'ixmax.inc'
      include 'drwmap.inc'
      parameter (ixmax1=ixmax+1)
      real det

      real dxyz(ixmax1*ixmax1*5)   ! for 3D map
      real cxy(ixmax1*ixmax1),dxy(ixmax1*ixmax1) ! for 2D map

      character(80) flnm1,flnm2,flnm3,flnm4
      common /cfilename/ flnm1,flnm2,flnm3,flnm4

      real hmin(1),div(1),x0(6),q(6)
      real qd(6),xe(6),xi(6),xd(6),xf(6),qe(6),qi(6),x1(6)
c     integer hmax(6)
      integer lf(1)
      integer pcol,sctmdl
      common /cpcol/ pcol
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq

      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      common /cjbc/ jbc,xs(6),iax(6),ipers
      real s1(3,3),s2(3,3),s3(3,3),s4(3,3)
      common /otrfm/ si(6,6),sr(6,6),ti(6,6),tr(6,6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
c
      common /ccol/ icol
      common /cwflag/ iwflag(7)
      integer q10,qm,qc
      common /cdrwed/ q10(6),qm(6),qc(6)
      integer npix(3)
      real am(0:5000),bm(0:5000)
      character(100) rfform
      common /const0/ pi2,dmgn,irs(6)
      data ic1/1/,ic2/1/,ic3/0/
      common /cproj/ iproj, px, py, pz, shftz

      dimension ipv(6),ind(6,2),piv(6),b(6)
      integer mm(7)
      integer getnt
      parameter (tau=1.618034)
      real ltconst(6)
c     pi=getpi()

      jxmax=ixmax1*ixmax1*5     ! for 3D map      
      call plots(0.0,0.0,ldev)
      call freadi('ng=',1,ng,1)
      call getsys(isys)
c     write(6,*) 'isys1=',isys 
      do nplt=1,ng
         call freadiv('iax=',6,iax,1)
c        write(6,'(a,6i2,a,i2)') 'iax=',iax
         call freadi('ipers=',1,ipers,0)
         write(6,'(a,6i2,a,i2,a,i2)') 'iax=',iax

         call freadi('iaver=',1,iaver,1)
         call freadi('ismth=',1,ismth,0)
         write(6,'(4(a,i2))') 'iaver=',iaver,' ismth=',ismth
         call setsymavr()       !  set symmetry operator for averaged density
         call setavr(iaver)
         call setsmt(ismth)
c     read(5,*) x1,x2,x3,x4,x5,x6
         call freadrv('x=',6,x0,1)
         call freadrv('xe=',6,xe,1)
         call freadr('u=',1,u0,0)
         call freadrv('xi=',6,xi,1)
         call freadr('v=',1,v0,0)
         call freadrv('q=',6,q,1)
         write(6,'(a,6f10.5)') 'x=',x0
         write(6,303) 'xe=', (xe(j),j=1,6),'u=',u0
         write(6,303) 'xi=', (xi(j),j=1,6),'v=',v0
 303     format(a,6f8.5,2x,a,f8.5)
         write(6,'(a,6f10.5)') 'q=',q

         call getxe(q,xd)
c     read(5,'(9f8.4)') xmax,ymax,hscl,xdiv,ydiv
         if(iplot.ge.0) then    ! 2D map
            call readxyminmax()
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
            izm=0
            ixyzm=(ixm+1)*(iym+1)
c           write(6,*) 'ixm,iym=',ixm,iym
c           stop
            if((ixm+1)*(iym+1).gt.ixmax1*ixmax1) then
               write(6,*) '(ixm+1)*(iym+1)=',(ixm+1)*(iym+1)
               write(6,*) 'imax1*imax1=',ixmax1*ixmax1
               call quit('(ixm+1)*(iym+1)>ixmax1*ixmax1)') 
            end if
         else if(iplot.lt.0) then ! 3D map
            call readxyzminmax()
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
            izm=(zmax-zmin)/zdiv+0.001
            ixyzm=(ixm+1)*(iym+1)*(izm+1)
            write(6,'(a,3i4)') 'ixm,iym,izm=',ixm,iym,izm
            if(ixyzm.gt.jxmax) then
               write(6,*) 'jxmax=',jxmax
               call quit('(ixm+1)*(iym+1)*(izm+1)>jxmax')
            end if
         end if

         nt=getnt()
         write(6,*) 'nt=',nt
         if(ixyzm.gt.nt) then
            write(6,*) 'ixm*iym*izm=',ixyzm
            write(6,*) 'nt=',nt
            call quit('ixm*iym*izm > nt')
         end if

         call setxyt(1.,1.,0.,0) ! use rectangular coordinate system
         call setxs(x0,xe,xi,xd,u0,v0) ! set origin shift

c        read(5,113) (hmin(i),div(i),lf(i),i=1,ndiv)
         ndiv=1
         if(iplot.gt.0) then
            call freadr('hmin=',1,hmin(1),1)
            call freadr('div=',1,div(1),0)
            call freadi('lf=',1,lf(1),0)
            write(6,225) hmin(1),div(1),lf(1)
 225        format('hmin=',f12.6,2x,'div=',f12.6,2x,'lf=',i3)
         end if
c     hmin(i): minimum hight in i-th range
c     div(i) : increase of hight in i-th range
c     lf(i)  : number of contour lines in i-th range
c     ndiv: number of ranges
c     ixm  : number of lattice points along x-axis
c     iym  : number of lattice points along y-axis
c     am,bm : work array to store the x,y coordinates on a contour line
c             (size=5000)
c     xy: hight of the function at the lattice point (i,j)
c     ixm,iym : size of to dimensional array xy
c
c     read(5,111) amagx,amagy,shftxb,shftyb,shftxa,shftya
         iw=0
         if(iplot.gt.0) then    ! 2D map
            call rdpltmap2(iax,iqx,iqy)
            call setxyt(1.,1.,0.,0) ! use rectangular coordinate system
            call rdpxyzsft(ipers,px,py,pz,shftz)
            call cxtoxy(cr,dxy,ixm,iym)
            call map1(hmin,div,lf,ndiv,iw,dxy,ixm,iym,cxy)
            call setaxy()
         else if(iplot.lt.0) then ! 3D map
            call rdpltmap3(iax)
            call setaxyz()
            call cxtoxyz(cr,dxyz,ixm,iym,izm)
            call setltcnst(ltconst)
            call setnpix(npix,ixm,iym,izm)
            call dxyzbinout(dxyz,npix,ltconst,flnm3) ! 3D data for vend
            call dxyzascout(dxyz,npix,ltconst,flnm4) ! 3D data for vend
         end if
      end do
      call plote('end of plot')
      return
      end

      subroutine setltcnst(ltconst)
      include 'drwmap.inc'
      real ltconst(*)
      ltconst(1)=xmax-xmin
      ltconst(2)=ymax-ymin
      ltconst(3)=zmax-zmin
      ltconst(4)=90.
      ltconst(5)=90.
      ltconst(6)=90.
      return
      end
            
      subroutine setnpix(npix,ixm,iym,izm)
      integer npix(*)
      npix(1)=ixm
      npix(2)=iym
      npix(3)=izm
      return
      end
            
      subroutine setaxy()
      include 'drwmap.inc'
      ax=(xmax-xmin)
      bx=(ymax-ymin)
      coscx=0
      return
      end

      subroutine setaxyz()
      include 'drwmap.inc'
      ax=(xmax-xmin)
      bx=(ymax-ymin)
      cx=(zmax-zmin)
      cosax=0
      cosbx=0
      coscx=0
      return
      end

      subroutine rdpxyzsft(ipers,px,py,pz,shftz)
c      if(ipers.eq.1) then
cc     read(5,*) px,pz,py,shftz
c         call freadr('px=',1,px,1)
c         call freadr('py=',1,py,0)
c         call freadr('pz=',1,pz,0)
c         call freadr('shftz=',1,shftz,0)
c      else
         px=0.1
         py=-0.1
         pz=0.2
         shftz=-100
c      end if
c      write(6,'(a,4f10.4)') 'direction of projection and shftz',
c     *     px,py,pz,shftz
      return
      end

      function getfact(isys)
      call getsys(isys)
      if(isys.le.2) then
         fact=1
      else if(isys.eq.3) then
         fact=2/sqrt(5.)
      else if(isys.eq.4) then
         fact=1
      else if(isys.eq.5) then
         fact=2/sqrt(6.)
      else
         call quit('invalid isys')
      end if
      getfact=fact
      return
      end

      subroutine getxs(xsd)
      real xsd(6)
      common /cjbc/ jbc,xs(6),iax(6),ipers
      call vcpyr(xs,xsd,6)
      return
      end

      subroutine setxs(x0,xe,xi,xd,u0,v0)
      real x0(*),xe(*),xi(*),xd(*)
      common /cjbc/ jbc,xs(6),iax(6),ipers
      real x1(6)
      real qe(6),qi(6)
      call getsys(isys)
      if(isys.le.2) then
         call getqe0(xe,qe)
      else if(isys.eq.3.or.isys.eq.4) then
         call tr6to5e(xe,x1)
         call getqe0(x1,qe)
      else if(isys.eq.5) then
         call getqe0(xe,qe)
      end if
c     call getqe0(xe,qe)
      qe(4)=0
      qe(5)=0
      qe(6)=0
      call getxe(qe,xe)
      
      if(isys.le.2) then
         call getqe0(xi,qi)
      else if(isys.eq.3.or.isys.eq.4) then
         call tr6to5e(xi,x1)
         call getqe0(x1,qi)
      else if(isys.eq.5) then
         call getqe0(xi,qi)
      end if
c     call getqe0(xi,qi)
      qi(1)=0
      qi(2)=0
      qi(3)=0
      call getxe(qi,xi)
      do i=1,6
         xs(i)=x0(i)+xe(i)*u0+xi(i)*v0-xd(i)
      end do
      return
      end

      subroutine rdpltmap2(iax,iqx,iqy)  ! for 2D map
      integer iax(*)
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      call freadr('amagx=',1,amagx,1)
      call freadr('amagy=',1,amagy,0)
      call freadr('shftxb=',1,shftxb,0)
      call freadr('shftyb=',1,shftyb,0)
      call freadr('shftxa=',1,shftxa,0)
      call freadr('shftya=',1,shftya,0)
      write(6,'(a,6f10.5)')
     *     'amagx,amagy,shftxb,shftyb,shftxa,shftya=',
     *     amagx,amagy,shftxb,shftyb,shftxa,shftya
      if(amagx.eq.0) amagx=1
      if(amagy.eq.0) amagy=1
      call setamg(amagx,amagy)
      call setqxqy(iax,iqx,iqy)
      return
      end

      subroutine rdpltmap3(iax) ! for 3D map
      integer iax(*)
      amagx=1
      amagy=1
      amagz=1
      call setamg3(amagx,amagy,amagz)
      call setqxqyqz(iax,iqx,iqy,iqz)
      return
      end

      subroutine readxyminmax()
      include 'drwmap.inc'
      call freadr('xmin=',1,xmin,1)
      call freadr('xmax=',1,xmax,0)
      call freadr('ymin=',1,ymin,0)
      call freadr('ymax=',1,ymax,0)
      call freadr('hscl=',1,hscl,0)
      call freadr('xdiv=',1,xdiv,0)
      call freadr('ydiv=',1,ydiv,0)
      write(6,'(a,7f10.5)')
     *     'xmin,xmax,ymin,ymax,hscl,xdiv,ydiv=',
     *     xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
      return
      end

      subroutine  readxyzminmax()
      include 'drwmap.inc'
      call freadr('xmin=',1,xmin,1)
      call freadr('xmax=',1,xmax,0)
      call freadr('ymin=',1,ymin,0)
      call freadr('ymax=',1,ymax,0)
      call freadr('zmin=',1,zmin,0)
      call freadr('zmax=',1,zmax,0)
      call freadr('hscl=',1,hscl,0)
      call freadr('xdiv=',1,xdiv,0)
      call freadr('ydiv=',1,ydiv,0)
      call freadr('zdiv=',1,zdiv,0)
      write(6,'(a,10f10.5)')
     *     'xmin,xmax,ymin,ymax,zmin,zmax,hscl,xdiv,ydiv,zdiv=',
     *     xmin,xmax,ymin,ymax,zmin,zmax,hscl,xdiv,ydiv,zdiv
      return
      end

      subroutine setsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtrd=ismtr
      return
      end

      subroutine getsmtr(ismtr)
      common /csmtr/ ismtrd
      ismtr=ismtrd
      return
      end

      subroutine setavr(iaver)
      common /caver/ iavers
      iavers=iaver
      return
      end

      subroutine getavr(iaver)
      common /caver/ iavers
      iaver=iavers
      return
      end

      subroutine setsmt(ismth)
      common /csmth/ ismths
      ismths=ismth
      return
      end

      subroutine getsmt(ismth)
      common /csmth/ ismths
      ismth=ismths
      return
      end

      subroutine vcovr(x,y)
      real x(6),y(6)
      common /cmtrt/ gab(6,6)
c      get covariant elements y from contravariant elements
c      gab : metric tensor

      do i=1,6
         y(i)=0
         do j=1,6
            y(i)=y(i)+gab(i,j)*x(j)
         end do
      end do
      return
      end

      subroutine setsymavr()    ! symmetry operation for averaged map
      integer rg(6,6,2)
      real    tg(6,2)
      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      integer imps(2)
      character(60) crri(2),crrd(2)
      logical isZerov
      data crri /'x,z,t,u,v,y',         ! for icosahedral
     *           '-u,y,z,-v,-x,-t'/
      data crrd /'y,z,t,-x-y-z-t,u,v',  ! for decagonal
     *           't,z,y,x,u,v'/
      call getsys(isys)
      if(isys.le.0.or.isys.gt.5) then
         write(6,*) 'isys in setsymavr=',isys
         call quit('invalid isys')
      end if
      nsymo=2
      do k=1,nsymo
         if(isys.le.2) then
            call decode(crri(k),rg,tg,k) ! icosahedral
            idm=6
         else if(isys.eq.3) then
            call decode(crrd(k),rg,tg,k) ! decagonal
            idm=5
         else if(isys.eq.4) then
            call quit('octagonal has not been implemented')
            call decode(crrd(k),rg,tg,k) ! octagonal
            idm=5
         else if(isys.eq.5) then
            call quit('dodecagonal has not been implemented')
            call decode(crrd(k),rg,tg,k) ! dodecagonal
            idm=5
         else
            call quit('invalid isys')
         end if
      end do

c      do k=1,nsymo
c         do i=1,6
c            write(6,'(6i3,f10.5)') (rg(i,j,k),j=1,6),tg(i,k)
c         end do
c         write(6,*) ' '
c      end do

      call ordr(rg,imps,nsymo,idm) ! get order of generators
      write(6,*) 'imp=',imps

      impt=1
      call zerovr(avrt(1,1),6)
      call unitmtrxi(avrr(1,1,1),6)
      do ns=1,nsymo
         imp=imps(ns)
c        write(6,*) 'ns,imp=',ns,imp
c     modified to include the pure translation
         if(imp.eq.1) then
            if(.not.isZerov(tg(1,ns))) then
               imp=2
            end if
         end if
         do i=2,imp
            do j=1,impt
               mp1=j+(i-2)*impt
               mp2=j+(i-1)*impt
               call mpso(rg,tg,ns,avrr,avrt,mp1,mp2,idm)
            end do
         end do
         impt=impt*imp
      end do
      nsymo=impt
      call wtsymo(avrr,avrt,nsymo,idm)  ! write symmetry operator
c     write(6,*) 'nsymo in setsymavr()',nsymo
      return
      end

      function getfmax()
      common /cfmax/ fmax
      getfmax=fmax
      return
      end

      integer function getnind()
      common /cnind/ nind       ! number of independent reflections used
      getnind=nind
      return
      end

c      function getf0()
c      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
c      getf0=f000
c      return
c      end

      subroutine symstf()       ! generate equivalent reflections
      include 'abcdri.inc'
c     generate equivalent reflections
      common /const0/ pi2,dmgn,irs(6)
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      include 'nhmax.inc'
      include 'hklf.inc'        ! independent reflections
      include 'symstfsort.inc'
      common /cnind/ nind       ! number of independent reflections used
      real dmgn
      common /csc/ sc
      complex fctmp,fctmp0,cexp

      common /cfmax/ fmax

      integer hr(6),hi(6),hd(6),hrd(6)
      integer h,k,l,m,n,o
      character brv
      integer mm(7)
c     integer hmax(6)
      integer gethkltoj,getnt
      logical isMem,isFourier,isDFourier,isPatterson
      nt=getnt()
      call getidm(idm)
c     call gethmax(hmax)
      call getsmtr(ismtr)
      call getnsymo(nsymo)
      call geticent(icent)
      call getbrv(brv)
      call getiplt(iplot)
c     fact=getbrvfact(brv)
      fact=1
      sg00=sg00*fact
      sg0=sg0*fact
      sg1=sg1*fact
      f000=f000*fact

c     stop
c     initialization
      call zerovr(br,nt)
      call zerovr(bi,nt)
      if(isDFourier()) then
         br(0)=0
      else
         br(0)=f000             ! set this first after that this may override
      end if
      write(6,*) 'nt,is,nsymo,f000,fact=',nt,is,nsymo,f000,fact

      fmax=0
      do i=1,is                 ! is : number of observed reflections
         if(isPatterson()) then
            fod(i)=fobs(i)
         else
            fod(i)=fcal(i)
         end if
      end do

c     write(6,*) 'number of observed reflections=',is

      if(is.gt.nhmax) then
         call quit('is > nhmax in symstf')
      else if(is.eq.0) then
         call quit('is =0 in symstf')
      end if

      call qsortd(fod,st,jp,is)  ! sort according to intensity
      nref=0
      nskp=0
      nind=0                    ! number of independent reflections used
      do jd=1,is                ! is : number of independent reflections
         j=jp(jd)               ! original order of hkl,fobs
         call vcpyi(hkln(1,j),hd,6)
c     call wtvctrif('hd=',hd,6,'(6i3)')
         fctmp0=cmplx(phsr(j),phsi(j)) ! complex phase for ind. reflection
         sgobs=sig(j)
         phsr0=real(fctmp0)
         phsi0=aimag(fctmp0)
         phsd=todeg(atan3(phsi0,phsr0))
         iph(j)=phsd+sign(0.0001,phsd)
         do ns=1,nsymo
            call gethr(hd,hi,ns,0)
            yt=getyt(hd,ns,ic)

            if(isprj(hi).eq.1) go to 30 ! for projected structures
            call smtrh(hi,hr,ismtr) ! simirality transformation
            call ctoph(hr)      ! indices in primitive cell
            if(iskip(hr).eq.1) then
               nskp=nskp+1
               go to 30
            end if

c     strucutre factor for equivalent reflections
            fctmp=fctmp0*cexp(pi2*yt*(0.,-1.)) ! phase for equivalent refl.
            argr=real(fctmp)
            argi=aimag(fctmp)
            do ic=0,1           ! generage equivalent reflections
               if(ic.eq.1) argi=-argi
               if(ic.eq.1) call vnegatei(hr,6)
               ao=fobs(j)*argr  ! real part of observed structure factor
               bo=fobs(j)*argi  ! imaginary part of observed structure factor
               if(isDFourier()) then
                  ac=fcal(j)*argr ! real part of calculated structure factor
                  bc=fcal(j)*argi ! imaginary part of calculated structure factor
                  ao=ao-ac
                  bo=bo-bc
               end if
               fmax=max(fmax,fobs(j))
               l=gethkltoj(hr)  ! nD to 1D address for primitive unit cell
               if(l.eq.0) then
                  write(6,*) 'hr=',hr
                  write(6,*) 'if hr=0 0 0 0 0 0 note that blank line'
                  write(6,*) 'in flnm1 is inhibited'
                  call quit('index=0 in symstf')
               end if
               nref=nref+1
               call setbeq(ao,bo,l) ! set br,bi
               if(ns.eq.1.and.ic.eq.0) then
                  call setari0(ao,bo,sgobs,nind,l)
               end if
            end do
 30         continue
         end do
 20      continue
      end do
      f000=br(0)                ! this may be a value given by Fobs
      write(6,*) 'f000 used in the calculation =',f000
      write(6,*) 'number of reflections used   =',nref
      write(6,*) 'number of skipped reflections=',nskp
      if(nref.eq.0) call quit('nref=0')
      return
      end

      subroutine wthklfo(hr,br,bi,ismtr)
      integer hr(6)
      real ar,ai,br,bi
      integer hd(6),hi(6)
      call vcpyi(hr,hd,6)
      call ptoch(hd)
      call smtrh(hd,hi,-ismtr)  ! simirality transformation
c     write(6,'(6i4,4f12.6)') hi,ar,ai,br,bi
      write(6,'(6i4,2f12.6)') hi,br,bi
      return
      end

      subroutine nrmxyz(cr,hmax,dmin,dmax,f000)
      real cr(0:*)
      integer hmax(*)
      character brv
      integer getnt
      nt=getnt()
      vol=getvol()
      call getbrv(brv)
c
c     normalization of cr > 0
      eps=f000/1000
      fact=(f000/vol)/(f000/vol+abs(dmin)+eps)
      do i=0,nt-1
         cr(i)=(cr(i)+abs(dmin)+eps)*fact
      end do
      return
      end

      subroutine intdns(cr,dmin)
      real cr(0:*)
c     integer hmax(6)
      character brv
      integer getnt
      data eps /1./
c     call gethmax(hmax)
      call getbrv(brv)
      nt=getnt()
c
c     initial electron density cr > dmin
      do i=0,nt-1
         if(cr(i).lt.dmin) then
            cr(i)=dmin
         end if
      end do
      return
      end

      function getfct(brv)
      character brv
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
      getfct=fact
      return
      end

      subroutine getbrbi0()
      include 'nhmax.inc'
      include 'hklf.inc'
c     include 'qsort.inc'
      include 'abcdri.inc'      
      integer hd(6),hi(6)
      integer gethkltoj
      call getidm(idm)
c     write(frmt,'(a,i1,a)') '(',idm,'i4,4e12.4)'
      call getsmtr(ismtr)
      do i=1,is
         call vcpyi(hkln(1,i),hd,6)
         call smtrh(hd,hi,ismtr)
         call ctoph(hi)         ! centered to primitive lattice
         j=gethkltoj(hi)
         br0(i)=br(j)
         bi0(i)=bi(j)
      end do
      return
      end

      subroutine saxis(ih)
      dimension ih(6)
      return
      end

      subroutine phsngb(bhkl,xs)
      complex bhkl(0:*)
      real xs(6)
      real zbhkl,rbhkl,ibhkl
      complex ctmp,phsfct,cexp
      common /const0/ pi2,dmgn,irs(6)
c     integer hmax(6)
      integer hkl(6)
      integer getnt
      call getidm(idm)
c     call gethmax(hmax)
      zsum=0
      zdif=0
      nt=getnt()
      do i=0,nt-1
         call getitohkl(i,hkl)
         yt=vinpir(hkl,xs,6)
         phsfct=cexp(pi2*yt*(0.,-1.))
         zbhkl=abs(bhkl(i))
         ctmp=bhkl(i)*phsfct
         rbhkl=real(ctmp)
         ibhkl=aimag(ctmp)
         argr=1.0
         argi=0.0
         if(zbhkl.ne.0.0) then      
            argr=rbhkl/zbhkl
            argi=ibhkl/zbhkl
         end if
         bhkl(i)=cmplx(zbhkl*argr,zbhkl*argi)
      end do
      return
      end

      subroutine rdhklq(qimax,frmt,iaster,flnm1)  ! read hkl fo fc phase
      character(*) frmt, flnm1
      character aster
c     integer hmax(6)
      integer h(6),hd(6),h1(6)
      include 'nhmax.inc'
      include 'hklf.inc'
c     include 'qsort.inc'
      common /tofofc/ f000,fmag,sg00,sg0,sg1,scl
      real q(6)
      integer getiplot
      logical isMem,isPatterson,isFourier,isDFourier
      character(50) frmt1
      data pi /3.141592/
      f000=f000*fmag
      sg00=sg00*fmag
      sg0=sg0*fmag
      sg1=sg1*fmag

      iplot=getiplot()
c     call gethmax(hmax)
      call geticent(icent)
      call getsmtr(ismtr)
      call openfile(2,flnm1,'OLD')
      call getidm(idm)
c     write(6,*) 'idm in rdhklq=',idm
                                !      write(6,*) 'icent=',icent
                                !      write(6,*) 'flnm1=',flnm1
      call getsys(isys)
      call zerovi(h1,6)
      write(frmt1,'(a,i1,a)') '(',idm,'i4,4f10.2)'
c     write(6,*) 'idm,frmt1=',idm,frmt1
c     call quit('stop in rdhklq')

      i=0
      nskip=0
 100  continue
      if(isMem()) then
         read(2,frmt,end=300) (h1(j),j=1,idm),fo,fc,phs,sg
         write(6,fmt=frmt1) (h1(j),j=1,idm),fo,fc,phs,sg
c     write(6,*) (h1(j),j=1,idm),fo,fc,phs,sg
      else if(isFourier().or.isDFourier()) then
         read(2,frmt,end=300) (h1(j),j=1,idm),fo,fc,phs
         write(6,fmt=frmt1) (h1(j),j=1,idm),fo,fc,phs
c     write(6,*) (h1(j),j=1,idm),fo,fc,phs
      end if
      if(h1(1).eq.999) go to 300
c     call smtrh(h1,h,ismtr)    ! simirarity transformation
                                ! write(6,'(a,6i4)') 'h=',h
      call vcpyi(h1,h,6)
c     write(6,*) 'iaster=',iaster
      if(iaster.eq.0.or.
     *     (iaster.eq.1.and.aster.eq.' ').or.
     *     (iaster.eq.2.and.aster.eq.'*')) then
         call getqi0(h1,q)
         if(isys.le.2) then
            qi=vabs(q(4),3)
         else
            qi=vabs(q(4),2)
         end if
         call vcpyi(h,hd,6)
         call ctoph(hd)         ! indices in primitive cell
         if(qi.le.qimax) then
            i=i+1
            fobs(i)=fo*fmag
            fcal(i)=fc*fmag
            th=torad(phs)
            phsr(i)=cos(th)
            phsi(i)=sin(th)
            call vcpyi(h1,hkln(1,i),6)
            sig(i)=sg*fmag
c     write(6,*) fobs(i),fcal(i),phs
         else
            nskip=nskip+1
         end if
      end if
      go to 100
 300  continue
      is=i
      write(6,'(a,i5)') 'number of reflections used  =',is
      write(6,'(a,i5)') 'number of skiped reflections=',nskip
      close(2)
      return
      end

      subroutine read2
      character(80) str
c      skip up to the header Nrefl  h k l etc.
      rewind 2
 100  continue
      read(2,'(a)',end=200) str
      if(str(1:6).eq.' Nrefl') then
            go to 76
      else
            go to 100
      end if
 76   continue
c      skip one line
      read(2,*)
      return
 200  call quit(' Nrefl not found')
      return
      end

      integer function check(ih,ik,h,k)
      integer ih,ik
      real h,k
      integer jh,jk
      jh=h*10
      jk=k*10
      if(h*10-jh.gt.0.5) then
          jh=jh+1
      else if(h*10-jh.lt.-.5) then
          jh=jh-1
      end if
      if(k*10-jk.gt.0.5) then
          jk=jk+1
      else if(k*10-jk.lt.-.5) then
          jk=jk-1
      end if
      if(ih.eq.jh.and.ik.eq.jk) then
          check=1
      else
          check=0
      end if
      return
      end

      subroutine ottrfm(q)
      real q(6)
      call ottrfm0(q)
c      call ottrfm1(q)
      return
      end

c     transformation of coordinates by axis transformation
      subroutine ottrfm1(x)
      real x(6),xd(6)
      common /ctr/ itr(6,6)
      do i=1,6
         xd(i)=0
         do j=1,6
c            xd(i)=xd(i)+x(j)*itr(j,i)
            xd(i)=xd(i)+itr(i,i)*x(j)
         end do
      end do
      call vcpyr(xd,x,6)
      return
      end
      
      subroutine ottrfm0(q)
c     orthogonal transformation in external and internal spaces
      real q(6),qd(6)
      common /otrfm/ si(6,6),sr(6,6),ti(6,6),tr(6,6)
      integer getjbc
      call mtmplmno(q,sr,qd,1,6,6)
      if(getjbc().eq.-1) then
         call vcpyr(qd,q,6)
      else if(getjbc().eq.-2) then
         call mtmplmno(qd,tr,q,1,6,6)
      else
         write(6,*) 'jbc=',getjbc()
         call quit('strange jbc in ottrfm0')
      end if
      return
      end

      subroutine savexc(xc,xold,ndiff)
      real xc(*),xold(3,*)
      xold(1,ndiff)=xc(1)
      xold(2,ndiff)=xc(2)
      xold(3,ndiff)=xc(3)
      return
      end

      subroutine savehr(hr,hold,ndiff)
      integer hr(*),hold(6,*)
      do i=1,6
         hold(i,ndiff)=hr(i)
      end do
      return
      end

      integer function icomp(h,hr)
      integer h(*),hr(*)
      icomp=0
      do i=1,6
         if(h(i).ne.hr(i)) return
      end do
      icomp=1
      return
      end

      subroutine savehc(hc,hold,ndiff)
      real hc(*),hold(6,*)
      do i=1,6
         hold(i,ndiff)=hc(i)
      end do
      return
      end

      subroutine smthxy0(cxy,ixm,iym,dxy)
      real cxy(0:ixm,0:iym)
      real dxy(0:ixm,0:iym)
      do i=1,ixm-1
         do j=1,iym-1
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=0,0
         do j=1,iym-1
            dxy(i,j)=(cxy(i,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=ixm,ixm
         do j=1,iym-1
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=1,ixm-1
         do j=0,0
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=1,ixm-1
         do j=iym,iym
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j))/8
         end do
      end do
      dxy(0,0)=cxy(0,0)
      dxy(ixm,0)=cxy(ixm,0)
      dxy(0,iym)=cxy(0,iym)
      dxy(ixm,iym)=cxy(ixm,iym)
      do i=0,ixm
         do j=0,iym
            cxy(i,j)=dxy(i,j)
         end do
      end do
      return
      end

      subroutine fdmin(i1,i2,i,m1,dmin,a,b)
      dimension a(0:*),b(0:*)
      m1=i1
      do j=i1,i2
         d=(a(j)-a(i))**2+(b(j)-b(i))**2
         if(j.eq.i1) then
            dmin=d
            m1=j
         else if(d.lt.dmin) then
            dmin=d
            m1=j
         end if
      end do
      return
      end

      subroutine intch(x1,y1,x2,y2)
      x=x1
      y=y1
      x1=x2
      y1=y2
      x2=x
      y2=y
      return
      end

      subroutine getcas(ic)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      ic=icase
      return
      end

      subroutine gauss(p,n)
      parameter (ndata=200000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata)
      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

      do j=2,n-1
         a(j,1)=xl(j)
         a(j,2)=2.0*(xl(j)+xl(j-1))
         a(j,3)=xl(j-1)
         do k=1,is
            b(k,j)=(3.0*(xl(j-1)*xl(j-1)*(p(k,j+1)-p(k,j)))
     &           +3.0*xl(j)*xl(j)*(p(k,j)-p(k,j-1)))/(xl(j)*xl(j-1))
         end do
      end do

      do i=2,n
         d=a(i-1,2)/a(i,1)
         a(i,1)=0.0
         a(i,2)=a(i,2)*d-a(i-1,3)
         a(i,3)=a(i,3)*d
         do k=1,is
            b(k,i)=b(k,i)*d-b(k,i-1)
         end do
         q=a(i,2)
         a(i,2)=1.0
         a(i,3)=a(i,3)/q
         do k=1,is
            b(k,i)=b(k,i)/q
         end do
      end do
      do k=1,is
         u(k,n)=b(k,n)
         do j=1,n-1
            u(k,n-j)=b(k,n-j)-a(n-j,3)*u(k,n+1-j)
         end do
      end do
      end

      subroutine getwmij(amij,wmij,ctoh,htoc)
      real amij(3,3),wmij(3,3),ctoh(6,6),htoc(6,6),tmp(6,6)
      do i=1,3
         do j=1,3
            tmp(i,j)=0
            do k=1,3
               tmp(i,j)=tmp(i,j)+amij(i,k)*ctoh(k,j)
            end do
         end do
      end do
      do i=1,3
         do j=1,3
            wmij(i,j)=0
            do k=1,3
               wmij(i,j)=wmij(i,j)+htoc(i,k)*tmp(k,j)
            end do
         end do
      end do
      return
      end

      subroutine getsys(isysd)
      common /cisys/ isys
c     write(6,*) 'isys in getsys=',isys
      isysd=isys
      return
      end            

      subroutine setsys(isysd)
      common /cisys/ isys
      isys=isysd
c     write(6,*) 'isys in setsys=',isys
      return
      end            

c*********************************
      subroutine trcs(x,y,n1,n2,n3)
      dimension x(*),y(*),n1(*),n2(*),n3(*)
      real n1,n2,n3
c*****transformation of coordinate systems********************
c     x: coordinates in primary system
c     y: coordinates in plotter system
c     y(1),y(2) are x and y coordinates in plotter system
      y(1)= (n1(1)*x(1)+n1(2)*x(2)+n1(3)*x(3))
      y(2)= (n2(1)*x(1)+n2(2)*x(2)+n2(3)*x(3))
      y(3)= (n3(1)*x(1)+n3(2)*x(2)+n3(3)*x(3))
      return
      end

      subroutine tr5to6(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *     0,1,0,0,-1,0,
     *     0,0,1,0,-1,0,
     *     0,0,0,1,-1,0,
     *     1,1,1,1, 1,0,
     *     0,0,0,0, 0,1/
      do j=1,6
         fi(j)=0.
         if(j.le.5) then
            do k=1,4
               fi(j)=fi(j)+sd(j,k)*ei(k)
            end do
         else if(j.eq.6) then
            fi(6)=ei(5)
         end if
      end do
      return
      end

      subroutine tr6to5e(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 4,-1,-1,-1, 1, 0,
     *     -1, 4,-1,-1, 1, 0,
     *     -1,-1, 4,-1, 1, 0,
     *     -1,-1,-1, 4, 1, 0,
     *     -1,-1,-1,-1, 1, 0,
     *     0, 0, 0, 0, 0, 1/
      do j=1,6
         fi(j)=0.
         if(j.le.4) then
            do k=1,5
               fi(j)=fi(j)+sd(j,k)*ei(k)/5
            end do
         else if (j.eq.5) then
            fi(5)=ei(6)
         end if
      end do
      return
      end

      subroutine tr6to5i(ei,fi)
      integer fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *     0,1,0,0,-1,0,
     *     0,0,1,0,-1,0,
     *     0,0,0,1,-1,0,
     *     0,0,0,0, 0,1,
     *     0,0,0,0, 0,0/
      do j=1,6
         fi(j)=0.
         do k=1,5
            fi(j)=fi(j)+sd(j,k)*ei(k)
         end do
      end do
      return
      end

c     function ifskip(q)
c     real q(*)
c     common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6)
c     data eps/0.001/
c     call getsys(isys)
c     ifskip=1
c     if(q(3).lt.-eps) return
c     if(q(2).lt.-eps) return
c     if(isys.eq.1) then
c     if(-.3090*q(1)+0.9511*q(2).gt.eps) return
c     else if(isys.eq.2) then
c     if(-.5878*q(1)+0.8090*q(2).gt.eps) return
c     else
c     call quit('invalid isys')
c     end if
c     ifskip=0
c     return
c     end

      subroutine intskp()
c     initialization of ifskip
      common /cskp/ q1(6),q2(6),q3(6),q4(6)
      integer n1(6),n2(6),n3(6),n4(6)
      data n1 /0,1,0,0,-1,0/
      data n2 /-1,0,0,1,0,0/
      data n3 /0,0,-1,0,1,0/
      data n4 /-1,0,0,0,0,0/
c     data n1 /0,-1,0,1,0,0/
c     data n2 /-1,0,0,0,-1,0/
c     data n3 /0,0,-1,0,0,1/
c     n1, n2, n3: vectors normal to the planes of independent area
c     in the reciprocal space (icosahedral)
      call getsys(isys)
      if(isys.le.2) then
         call getqi0(n1,q1)
         call getqi0(n2,q2)
         call getqi0(n3,q3)
         call getqi0(n4,q4)
         call norml(q1,3)
         call norml(q2,3)
         call norml(q3,3)
         call norml(q4,3)
      end if
      return
      end

      function ifskip(q)
      real q(*)
      common /cskp/ q1(6),q2(6),q3(6),q4(6)
c     data eps/0.001/
      data eps/0.0001/
      call getsys(isys)
      ifskip=1
      if(isys.le.2) then
         if(-0.58778525*q(1)+0.80901699*q(2).lt.-eps) return
         if(0.6882*q(1)+0.5*q(2)+0.5257*q(3).lt.-eps) return
         if(q(2).gt.eps) return
         if(q(3).lt.-eps) return
c     if(vinp(q1,q,3).gt.eps) return
c     if(vinp(q2,q,3).gt.eps) return
c     if(vinp(q3,q,3).gt.eps) return
c     if(vinp(q4,q,3).gt.eps) return
         ifskip=0
         return

c     if(q(3).lt.-eps) return
c     if(q(2).gt.eps) return
c     if(-.30901699*q(1)+0.95105652*q(2).lt.-eps) return
c     ifskip=0
c     return
      else if(isys.eq.-3) then
c     pentagonal
         if(q(3).lt.-eps) return
         if(q(2).gt.eps) return
         if(-.58778525*q(1)+0.80901699*q(2).lt.-eps) return
         ifskip=0
         return
      else if(isys.eq.4) then
c     octagonal
         if(q(3).lt.-eps) return
         if(q(2).gt.eps) return
         if(-.38268343*q(1)+0.92387953*q(2).lt.-eps) return
         ifskip=0
         return
      else if(isys.eq.5) then
c     dodecagonal
         ifskip=1
         if(q(3).lt.-eps) return
         if(q(2).gt.eps) return
         if(-.25881905*q(1)+0.96592583*q(2).lt.-eps) return
         ifskip=0
         return
      else
         call quit('isys invalid')
      end if 
      end

      subroutine reduc(ry)
c     obtain ry within a unit cell in 6D
      real ry(*)
      do i=1,6
         ry(i)=mod(ry(i),1.)
         rd=ry(i)
         if(rd.lt.-0.001) then
            ry(i)=ry(i)+1
         end if
      end do
      return
      end

      subroutine setvol()
      common /cvol/ vol
      call getsys(isys)
      call getac(al,cl)
      call getdet(det)
      if(isys.le.2) then
         vol=al**3/abs(det)   ! det is a cell volume of the reciprocal lattice
      else
         vol=al**2*cl/abs(det)
      end if
      write(6,*) 'al,det,vol=',al,det,vol
c     stop
      return
      end

      function getvol()
      common /cvol/ vol
      getvol=vol
c     write(6,*) 'getvol=',getvol
c     stop
      return
      end

      subroutine cxtoxyz(cr,dxyz,ixm,iym,izm)
      real cr(0:*)
      real dxyz(0:*)
      call cxtoxyz0c(cr,dxyz,ixm,iym,izm)  ! c order
c     call cxtoxyz0f(cr,dxyz,ixm,iym,izm)  ! fortran order
      return
      end

      subroutine cxtoxyz0c(cr,dxyz,ixm,iym,izm) ! c order
      implicit integer (h)
      real cr(0:*)
      real dxyz(0:*)
c     integer hmax(6)
      real hscl
      include 'drwmap.inc'
      real x(6),xd(6),rx(6),q(6),xy(6)

      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      common /cjbc/ jbc,xs(6),iax(6),ipers
      integer getjbc

      jbc=getjbc()
c     call gethmax(hmax)
      divx=(xmax-xmin)/(ixm)
      divy=(ymax-ymin)/(iym)
      divz=(zmax-zmin)/(izm)
      call getavr(iaver)
      if(icent.eq.0) then
         icd=1
      else
         icd=2
      end if
      iqx=iax(1)
      iqy=iax(2)
      iqz=iax(3)
      write(6,*) 'iqx,iqy,iqz=',iqx,iqy,iqz
      write(6,*) 'divx,divy,divz=',divx,divy,divz
      nt=ixm*iym*izm
      call zerovr(dxyz,nt)
c      write(6,*) '****iax=',iax(1),iax(2),iax(3)
c      write(6,*) '****ixm,iym,izm=',ixm,iym,izm
c      write(6,*) '****divx,divy,divz=',divx,divy,divz
c      write(6,*) '****hcscl,nsymo=',hscl,nsymo
c      write(6,*) '****iaver=',iaver

      n=0
      do h=0,ixm-1
         do k=0,iym-1
            do l=0,izm-1
               if(jbc.gt.0) then
                  call zerovr(q,6)
                  q(iqx)=xmin+h*divx
                  q(iqy)=ymin+k*divy
                  q(iqz)=zmin+l*divz
                  call getxe(q,x)
c     write(6,*) 'q=',q
               else 
                  call zerovr(q,6)
                  q(iqx)=xmin+h*divx
                  q(iqy)=ymin+k*divy
                  q(iqz)=zmin+l*divz
                  call ottrfm(q)
                  call getxe(q,x)
c                  call zerovr(x,6)
c                  x(iqx)=xmin+h*divx
c                  x(iqy)=ymin+k*divy
c                  x(iqz)=zmin+l*divz
c                  call ottrfm(x)
               end if
c              write(6,*) 'x=',x
c     get average density over all equivalent points
c     average density of equivalent positions
               dd=averaged(cr,x)
               dxyz(n)=dd*hscl
c              write(6,'(e12.4,$)') dd
               n=n+1
            end do
c            write(6,*) 'q=',q
c            write(6,*) 'x=',x
         end do
      end do
c     do h=0,ixm
c     write(6,'(10f8.2)') (dxyz(h,k),k=0,9)
c     end do
c     write(6,*) ' '
      return
      end

      subroutine cxtoxyz0f(cr,dxyz,ixm,iym,izm) ! fortran order
      implicit integer (h)
      real cr(0:*)
      real dxyz(0:*)
c     integer hmax(6)
      real hscl
      include 'drwmap.inc'
      real x(6),xd(6),rx(6),q(6),xy(6)

      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      common /cjbc/ jbc,xs(6),iax(6),ipers
      integer getjbc

      jbc=getjbc()
c     call gethmax(hmax)
      divx=(xmax-xmin)/(ixm)
      divy=(ymax-ymin)/(iym)
      divz=(zmax-zmin)/(izm)
      call getavr(iaver)
c     write(6,*) (cr(i),i=1,10)
      if(icent.eq.0) then
         icd=1
      else
         icd=2
      end if
      iqx=iax(1)
      iqy=iax(2)
      iqz=iax(3)
c      write(6,*) 'iqx,iqy,iqz=',iqx,iqy,iqz
c      write(6,*) 'divx,divy,divz=',divx,divy,divz
      nt=ixm*iym*izm
      call zerovr(dxyz,nt)
c      write(6,*) '****iax=',iax(1),iax(2),iax(3)
c      write(6,*) '****ixm,iym,izm=',ixm,iym,izm
c      write(6,*) '****divx,divy,divz=',divx,divy,divz
c      write(6,*) '****hcscl,nsymo=',hscl,nsymo
c      write(6,*) '****iaver=',iaver
      n=0
      do l=0,izm-1
         do k=0,iym-1
            do h=0,ixm-1
               if(jbc.gt.0) then
                  call zerovr(q,6)
                  q(iqx)=xmin+h*divx
                  q(iqy)=ymin+k*divy
                  q(iqz)=zmin+l*divz
                  call getxe(q,x)
               else 
                  call zerovr(q,6)
                  q(iqx)=xmin+h*divx
                  q(iqy)=ymin+k*divy
                  q(iqz)=zmin+l*divz
                  call ottrfm(q)
                  call getxe(q,x)
               end if
c               write(6,*) 'q=',q
c               write(6,*) 'x=',x
c     get average density over all equivalent points
               dd=averaged(cr,x)
               dxyz(n)=dd*hscl
c              write(6,'(f10.4,$)') dd
               n=n+1
            end do
         end do
      end do
c     do h=0,ixm
c     write(6,'(10f8.2)') (dxyz(h,k),k=0,9)
c     end do
c     write(6,*) ' '
      return
      end

c     get 2D electron density map (dxy) from nD electron density (cr)
      subroutine cxtoxy(cr,dxy,ixm,iym)
      real cr(0:*)
      real dxy(0:ixm,0:iym)
      real rx(6)
      real q(6),xy(6),xd(6)
      integer h,k
      include 'drwmap.inc'
      common /cjbc/ jbc,xs(6),iax(6),ipers
      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      integer getnt,getjbc
      nt=getnt()
      iqx=iax(1)
      iqy=iax(2)
      call getidm(idm)
      call wtsymo(avrr,avrt,nsymo,idm)
      call getsys(isys)
      divx=(xmax-xmin)/ixm
      divy=(ymax-ymin)/iym
      write(6,*) 'ixm,iym=',ixm,iym
      call getavr(iaver)
      write(6,*) 'iaver=',iaver
      call zeromtrxr(dxy,ixm+1,iym+1)
c     cr : electron density on a shifted lattice 
      jbc=getjbc()
      do  h=0,ixm
         do  k=0,iym
            if(jbc.gt.0) then
               call zerovr(q,6)
               q(iqx)=xmin+h*divx
               q(iqy)=ymin+k*divy
               call getxe(q,xy)
            else if(jbc.lt.0) then
               call zerovr(q,6)
               q(iqx)=xmin+h*divx
               q(iqy)=ymin+k*divy
               call ottrfm(q)
               call getxe(q,xy)
c               call zerovr(xy,6)
c               xy(iqx)=xmin+h*divx 
c               xy(iqy)=ymin+k*divy
c              call wtvctrr('xy0=',xy,6)
c               call ottrfm(xy)
c              call wtvctrr('xy1=',xy,6)
            end if
c     get average density over all equivalent points
c     average density of equivalent positions
            dd=averaged(cr,xy)
            dxy(h,k)=dd*hscl
c           write(6,*) dd
         end do
      end do
c      write(6,*) ' '
c      do k=0,ixm/2
c         write(6,'(10f10.4)') (dxy(k,l),l=0,iym/2)
c      end do
      return
      end

      function averaged(cr,xy)
c     get density at xy after averaging equivalent points
      real cr(0:*)              ! electron density
      real xy(6)                ! nD coordinate
      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      real rx(6)
      real xd(6)
c     common /ctr/ itr(6,6)
      common /cjbc/ jbc,xs(6),iax(6),ipers

      real avrtd(6,12),avrtt(6)
      data avrtd /0,0,0,0,0,0,0.5,0.5,0.5,0.5,0.5,0.5,60*0/
c     data tt /0,0,0,0,0,0/
c     call wtvctrrf('xs in averaged=',xs,6,'(6f10.4)')
      call getavr(iaver)
      call getsmtr(ismtr)
      if(iaver.ge.1) then
         nse=nsymo
      else
         nse=1
      end if
      if(nse.le.0) then
         write(6,*) 'nsymo in averaged=',nsymo
         call quit(' ')
      end if
c      write(6,*) 'xy=',xy
c      call getidm(idm)
c      call wtsymo(avrr,avrt,nsymo,idm)  ! write symmetry operator      

      dd=0
      do ns=1,nse
c     point rx is equivalent to xy
         call getrx0(xy,rx,avrr,avrt,ns,0)
         call vaddr(rx,xs,rx,6) ! origin shift
         call smtrx(rx,xd,-ismtr)
         call ctopx(xd)         ! centered lattice to primitive lattice
c     xd : coordinate of primitive lattice
         call grad(cr,xd,dnsty)
c        write(6,*) 'dnsty=',dnsty
         dd=dd+dnsty/nse
      end do
      averaged=dd
      return
      end

      subroutine getiplt(iplotd)
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      iplotd=iplot
      return
      end

      function getbrvfact(brv)
      character brv
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
      getbrvfact=fact
      return
      end

      subroutine gethkllen(hkl,qe,nt)
      integer hkl(6,*)
      real qe(*)
      do i=1,nt
         call getqi(hkl(1,i),q)
         qe(i)=vabs(q,3)
      end do
      return
      end

      integer function rcpmp1(x,rx)
c     1 for rx equivalent to x else 0
      real x(*),rx(*)
      data eps/0.00001/

      call getidm(idm)
      call  getnsymo(nsymo)
      call geticent(icent)

      rcpmp1=0
      do i=1,idm
         xx=abs(x(i)-rx(i))+eps/2
         xx=mod(xx,1.)
         if(xx.gt.eps) then
            return
         end if
      end do
      rcpmp1=1
      return
      end

      integer function isprj(hkln)
      integer hkln(*)

      integer hprj(6,6)         ! projection directions
      common /chprj/ iprj, hprj
      integer vinpi
c     return 1 if hkln have a non-zero component along projection axes
c     else return 0
      isprj=0
      if(iprj.eq.0) then
         return
      end if
      call getidm(idm)
c     write(6,*) 'idm=',idm
      do i=1,iprj               ! number of projection axes
         if(vinpi(hprj(1,i),hkln,idm).ne.0) then
            isprj=1
            return
         end if
      end do
      return
      end

