c
c        fourier or patterson map of decagonal quasi-crystals
c
c                   five-dimensional description
c                  (six-th coordinate is a dummy)
c                          input data
c     (0) file1   (2a)
c         file1:name for hkl_fo_fc file
c         file2
c         file2:name for density file
c     (1) title
c     (2) isys,a,c
c         isys: 1:icosahedral 2:icosahedral (cubic)
c               3:decagonal 4:octagonal
c         a: lattice parameter (in angstrom)  (0.8944 a icosa)
c         c: lattice parameter (in angstrom)  (6/sqrt(5) a icosa)
c     (3) nsymo,icent,brv
c         nsymo: number of generators in symmetry operations
c         icent: give 1 for centrosymmetryc
c                give 0 otherwise
c         brv  : 'p','i' or 'f'
c              ****** see (10) ******
c     (4) symmetry operator in the format of i.t.
c             for example x,y,z,t,u,v; x+1/2,y,z+1/2,t,u,v etc.
c  ********    repeat (4) nsymo times
c     (5) f000,sg0,ex1,ex2,nex1,nex2,icont
c            f000: fc at the origin
c            sg0: sigma=sg0     (sg0 : sigma=sqrt(fo)*sg0 is used)
c            ex1,ex2:  constants needed for smooth convergence
c            nex1,nex2: number of iterations
c            icont: iteration is continued from the previous result
c     (6) iplot, hmax, ismtr, nphase, iaster
c         iplot =0:  mem map
c               =1:  patterson map
c               =2:  fourier map
c               =-2: fc fourier map
c               =3:  difference fourier map
c          hmax(1-6): number of points along principal axes
c          ismtr : order of similarity transformations
c              qext => qext*tau**ismtr
c              qint => qint*tau**-ismtr
c              xext => xext*tau**-ismtr
c              xint => xint*tau**ismtr
c          nphase: number of strongest reflections for which 
c                  phase was fixed.
c          iaster: =0 all reflections
c                  =1 nonasterisked reflections
c                  =2 asterisked reflections
c     (7) qimax, frmt
c          qimax: maximum q vector to be used to draw map
c          frmt: file format for hkl, fobs, fcal(aster) sig 
c********
c     (8) ldev ng
c          ldev : 1: B&W Postscript printer 2: color Postscript printer
c          ng   :    number of graphs in a sheet
c                  ****if ng=0 no map is written****
c     (9) icase, iaver, ismth
c          icase=0: plot q1-q2 plane  (use only h1h2h3h40 reflections)
c                1: plot q1-q2 plane
c                2: plot q1-q3 plane
c                3: plot q2-q3 plane
c                4: plot q1-q4 plane
c                5: plot q2-q5 plane
c                6: plot q3-q4 plane (dihedral) or q3-q6 (icosahedral)
c                7: plot q4-q5 plane  (internal space)
c                8: plot q5-q6 plane
c                9: plot ellipsoid in the internal space (icasehedral only)
c               10: plot electron density
c                   along specified direction in q4-q5 space
c               11: plot electron density
c                   along specified direction in q5-q6 space
c               12: plot electron density
c                   along specified direction in q4-q6 space
c               if icase=>10 then direction cosine (cos, sin)
c          iaver : 0 noaverage 1 average equivalent points in the unit cell
c                    in the calculation of the density
c          ismth : 0=no smoothing
c                  1=3 point average
c                  2=5 point average
c     (9) x       :coordinates of origin
c     (10) xe, u   :coordinates of origin and displacement
c     (11) xi, v   :coordinates of origin and displacement
c     (12) q       :origin shift in external and internal space
c     (13) amagx,amagy,shftxb,shftyb,shftxa,shftya
c          amagx:  magnification for x
c          amagy:  magnification for y
c          shftxb: shift of origin along x before plot in cm
c          shftyb: shift of origin along y before plot in cm
c          shftxa: shift of origin along x after each plot in cm
c          shftya: shift of origin along y after each plot in cm
c     (14) xmin,xmax,ymin,ymax,hscl,xdiv,ydiv  or
c          ri1,ri2, hscl, dtheta, dphi (icase=9 only)
c
c          xmin:  minimum coordinate along x-axis
c          xmax:  maximum coordinate along x-axis
c          ymin:  minimum coordinate along y-axis
c          ymax:  maximum coordinate along y-axis
c          hscl: hight scale
c          xdiv: length of a mesh edge
c          ydiv: length of a mesh edge
c          ri1,ri2: radii for the first (x) and second (y) axes
c          hscl: hight scale
c          dtheta: interval of theta in polar coordinate system
c          dphi  : interval of phi in polar coordinate system
c     (16) hmin,div,lf
c         hmin: minimum hight (density) in the division
c         div : interval of hight
c         lf  : number of contour lines
c       ****if for icase=7,8*******
c     (17) px,py,pz,shftz
c          px,py,pz  : direction of the projection
c          shftz     : shift along z direction
c      *******repeat (8)-(17) ng times********
      program qcfour
      parameter (nhmax=2000)
      parameter (ixmax=100)
      real b(6),det,pvt(6)
      integer ipvt(6),indx(6,2)
      character*50 flnm1,flnm2
      character*40 frmt
      character brv
      real xmin,xmax,ymin,ymax
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      parameter (nhklt=125)   ! 5x5x5
c     parameter (nhklt=1730)  ! 12x12x12
c     parameter (nhklt=2744)  ! 14x14x14
c     parameter (nhklt=3375)  ! 15x15x15
c     parameter (nhklt=4096)  ! 16x16x16
c     parameter (nhklt=5832)  ! 18x18x18

      complex ahkl(nhklt*nhklt),bhkl(nhklt*nhklt)
      real cxyz(nhklt*nhklt),dxyz(nhklt*nhklt)
      real dxy(0:ixmax,0:ixmax)
      real hmin(1),div(1),x0(6),q(6)
      real qd(6),xe(6),xi(6),xd(6),xf(6),qe(6),qi(6),x1(6)
      integer hmax(6)
      integer lf(1)
      character*80 title
      common /ctitle/ title
      integer pcol,sctmdl
      common /cpcol/ pcol
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      common /cphs/ jp(nhmax),nphase
      integer r
      common /sym/ r(6,6,60),t(6,60),nsymo,icent
      common /bgn1/ ilm(3),icorc(3),pxj(100,3),hj(100,3)
      common /cjbc/ jbc,xs(6),iax(6)
      common /cdim/ idm,nde
      real dr,dq
c      common /cgetq/ dr(6,6),sds(6,6),dq(6,6),al,cl,er(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
c
      common /ccol/ icol,jcol
      common /cwflag/ iwflag(7)
      integer q10,qm,qc
      common /cdrwed/ q10(6),qm(6),qc(6)

      real am(0:5000),bm(0:5000)
      integer hm,km,lm,mm,nm,om
      common /const0/ pi2,dmgn,irs(6)
      data ic1/1/,ic2/1/,ic3/0/
      integer icased(6,0:12)
      parameter (tau=1.618034)
      data icased /1,2,3,4,5,6,
     *             1,2,3,4,5,6,
     *             1,3,2,4,5,6,
     *             2,3,1,4,5,6,
     *             1,4,2,3,5,6,
     *             2,5,1,3,4,6,
     *             3,4,1,2,5,6,
     *             4,5,1,2,3,6,
     *             5,6,1,2,3,4,
     *             1,2,3,4,5,6,
     *             4,5,1,2,3,6,
     *             5,6,1,2,3,4,
     *             4,6,1,2,3,5/
      data pi/3.141592/

      icorc(1)=ic1
      icorc(2)=ic2
      icorc(3)=ic3
      pcol=1
      icol=10
c
      call setinp(5)
      call setout(6)
      sctmdl=1
      read(5,'(a)') flnm1
      lfn1=istrln(flnm1)
      write(6,'(1h ,2a)') 'hkl-fo-fc file:',flnm1(1:lfn1)
      read(5,'(a)') flnm2
      open(unit=2,file=flnm1(1:lfn1))
c      lfn2=istrln(flnm2)
c      write(6,'(1h ,2a)') 'density file:',flnm2(1:lfn2)
c      open(unit=3,file=flnm2(1:lfn2),form='unformatted')
c      read(5,'(a)') title
      call freadc('title=',1,title,1)
      write(6,'(1h ,a)') title

      call freadi('isys=',1,isys,1)
      call freadr('a=',1,al,0)
      if(isys.ge.3) then
          call freadr('c=',1,cl,0)
      end if
      if(isys.le.2) then
          write(6,*) 'isys=',isys,' a=',al
      else 
          write(6,*) 'isys=',isys,' a=',al,' c=',cl
      end if

      call setsys(isys)
      call setd()
      if(isys.le.2) cl=al

      call mtrtns(al,cl)
      call mcpy(dr,er)
      call matinv(er,6,6,b,0,det,ipvt,indx,pvt)

      call getdet(det)
      if(isys.le.2) then
         idm=6
         nde=3
      else
         idm=5
         nde=2
      end if

c       sds: deformation matrix for unit vectors in 6d space
c       d,sds sent to getq through common area
c***
      call rdsymo(na,brv)
      if(icent.eq.0) call quit('only centrosymmetric case supported')
      call setbrv(brv)
c      read(5,104) f000,sg0,ex1,ex2,nex1,nex2
      call freadr('f000=',1,f000,1)
      call freadr('sg0=',1,sg0,0)
      call freadr('ex1=',1,ex1,0)
      call freadr('ex2=',1,ex2,0)
      call freadi('nex1=',1,nex1,0)
      call freadi('nex2=',1,nex2,0)
      call freadi('icont=',1,icont,0)
      write(6,203) f000,sg0,ex1,ex2,nex1,nex2,icont
 203  format(1h ,'f000=',f8.2,3x,'sg0=',f8.4,3x,'ex1=',e10.3,3x,
     *      'ex2=',e10.3,3x,'nex1=',i3,3x,'nex2=',i3,3x,'icont=',i1)
c     read(5,101) iplto, hmax,ismtr, iaster
      call freadi('iplot=',1,iplot,1)
      call freadi('hmax=',6,hmax,0)
      call freadi('ismtr=',1,ismtr,0)
      call freadi('nphase=',1,nphase,0)
      call freadi('iaster=',1,iaster,0)
      write(6,'(a,i2,a,6i4,a,i2,a,i5,a,i1)') 'iplot=',iplot,
     *     ' hmax=',hmax,' ismtr=',ismtr,' nphase=',nphase,
     *     ' iaster=',iaster
      if(isys.le.2) then
         vol=al**3/abs(det)
      else
         vol=al**2*cl/abs(det)
         hmax(6)=1
      end if

      nt=1
      do 30 i=1,idm
         nt=nt*hmax(i)
 30   continue
      if(nt.gt.nhklt*nhklt) then
         write(6,*) 'nt=',nt
         call quit('nt > nhklt*nhklt')
      end if
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
c     read(5,'(*)') qimax, frmt
      call freadr('qimax=',1,qimax,1)
      call freadc('frmt=',1,frmt,0)
      write(6,*) 'qimax=',qimax, ' frmt=',frmt
c     read(5,'(i3)') ldev, ng
      call freadi('ldev=',1,ldev,1)
      call freadi('ng=',1,ng,0)
c     read reflection indices
      call rdhklq(qimax,frmt,hmax,ismtr,iaster)
      hm=hmax(1)-1
      km=hmax(2)-1
      lm=hmax(3)-1
      mm=hmax(4)-1
      nm=hmax(5)-1
      om=hmax(6)-1
c       generate equivalent reflections
      call symstf(ahkl,bhkl,sg0,hm,km,lm,mm,nm,om,hmax,idm,f000)
c        ahkl : fo
c        bhkl : fc or fo-fc
c        sg   : sigma
      if(icont.eq.0.or.abs(iplot).gt.0) then
c        calculate initial electron density
         call mapxyz(bhkl,cxyz,hmax,nt,idm,dmin,dmax,vol)
         if(iplot.eq.0) then
c           get initial electron density cxyz > 0
            call intdns(cxyz,hmax,nt,idm,dmin)
         end if
      else if(icont.eq.1.and.iplot.eq.0) then
c         read(unit=3) (cxyz(i),i=1,nt)
c         call getdmx(cxyz,nt,dmax,vol)
         call quit('icont=1 is not allowed for sqcmem')
      end if
      if(abs(iplot).gt.0) then
         if(iplot.ne.1) then
            call rfct(ahkl,bhkl,sg0,f000,hm,km,lm,mm,nm,om,hmax,c2d,
     *                  rfact,rwfact,0)
c            write(6,'(a,i3,a,f10.1,a,f8.5)')
c     *      'ic=',ic,' c2=',c2d,' rfact=',rfact
         end if
         go to 700
      end if
      call maphkl(cxyz,bhkl,hmax,nt,idm,vol)
      f0=real(bhkl(1))/fact
c           new calculated structure factor bhkl
      call phsnga(ahkl,bhkl,nt,hmax,idm)
      call rfct(ahkl,bhkl,sg0,f000,hm,km,lm,mm,nm,om,hmax,c2d,
     *           rfact,rwfact,0)
      ic=0
         write(6,'(a,i3,a,f13.1,2(a,f8.5),a,f6.0)')
     *   'ic=',ic,' c2=',c2d,' rfact=',rfact,' rwfact=',rwfact,' f0=',f0
      if(nex1+nex2.eq.0) go to 700
      c20=c2d
      j=0
      do 122 ic=1,nex1+nex2
c     call difxyz(ahkl,bhkl,sg0,f000,exyz,hmax,nt,idm,vol)
************bhkl is replaced by ahkl-bhkl***********
      if(ic.eq.nex1+nex2) then
         ip=1
      else
         ip=0
      end if
      if(ic.le.nex1) then
         imode=1
      else
         imode=2
      end if
 600  continue
      call difxyz(ahkl,bhkl,sg0,f000,dxyz,hmax,nt,idm,vol)
************bhkl is replaced by ahkl-bhkl***********
c           difference density dxyz
      call expand1(dxyz,cxyz,nt,ex1,ex2,dmin,dmax,f000,imode,vol)
c           initial electron density cxyz
c           modified electron density dxyz > 0
      call maphkl(dxyz,bhkl,hmax,nt,idm,vol)
c           new bhkl obtained from dxyz
      f0=real(bhkl(1))/fact
c           new calculated structure factor bhkl
      call phsnga(ahkl,bhkl,nt,hmax,idm)
c           ahkl with new phase factors
      call rfct(ahkl,bhkl,sg0,f000,hm,km,lm,mm,nm,om,hmax,c2d,
     *          rfact,rwfact,ip)
      if(c2d.gt.c20) then
         j=0
         if(imode.eq.1) then
c           old bhkl obtained from cxyz
            call maphkl(cxyz,bhkl,hmax,nt,idm,vol)
            if(ex1.gt.1.e-8) then
               if(imode.eq.1) then
                  write(6,'(a,i3,a,f13.1,2(a,f8.5),a,f6.0,a,f8.3,a,
     *                 f10.3,a,e10.3)')
     *                 'ic=',ic,' c2=',c2d,' rfact=',rfact,
     *                 ' rwfact=',rwfact,' f0=',f0,' dmin=',dmin,
     *                 ' dmax=',dmax,' ex1=',ex1
               else
                  write(6,'(a,i3,a,f13.1,2(a,f8.5),a,f6.0,a,f8.3,a,
     *                 f10.3,a,e10.3)')
     *                 'ic=',ic,' c2=',c2d,' rfact=',rfact,
     *                 ' rwfact=',rwfact,' f0=',f0,' dmin=',dmin,
     *                 ' dmax=',dmax,' ex2=',ex2
               end if
               ex1=ex1/1.5
               go to 600
            end if
         else
            ex2=ex2/1.5
c           old bhkl obtained from cxyz
            call maphkl(cxyz,bhkl,hmax,nt,idm,vol)
            if(ex2.gt.1.e-8) then
               go to 600
            end if
         end if
      else
         if(j.eq.3) then
            if(imode.eq.1) then
               ex1=ex1*1.5
               ex1=min(1.e-2,ex1)
            else
               ex2=ex2*1.5
               ex2=min(1.e-2,ex2)
            end if
            j=0
         else
            j=j+1
         end if
      end if
      call vrcp(dxyz,cxyz,nt)
      if(imode.eq.1) then
       write(6,'(a,i3,a,f13.1,2(a,f8.5),a,f6.0,a,f8.3,a,f10.3,a,e10.3)')
     *        'ic=',ic,' c2=',c2d,' rfact=',rfact,' rwfact=',rwfact,
     *        ' f0=',f0,' dmin=',dmin,' dmax=',dmax,' ex1=',ex1
      else
       write(6,'(a,i3,a,f13.1,2(a,f8.5),a,f6.0,a,f8.3,a,f10.3,a,e10.3)')
     *        'ic=',ic,' c2=',c2d,' rfact=',rfact,' rwfact=',rwfact,
     *        ' f0=',f0,' dmin=',dmin,' dmax=',dmax,' ex2=',ex2
      end if
      if(abs(c2d-c20).lt.0.1) go to 700
      c20=c2d
 122  continue
 700  continue
      if(iplot.eq.0) then
c         write(unit=3) (cxyz(i),i=1,nt)
c         close(unit=3)
      end if
      if(ng.le.0) call quit(' ')
      call plots(0.0,0.0,ldev)
      do 800 nplt=1,ng
c     read(5,'(7i3)') icase,iaver
         call freadi('icase=',1,icase,1)
         call freadi('iaver=',1,iaver,0)
         call freadi('ismth=',1,ismth,0)
         write(6,'(3(a,i2))') 'icase=',icase,' iaver=',iaver,
     *        ' ismth=',ismth
         if(iaver.ne.0) then
            call getsym
         end if
         call setavr(iaver)
         call setsmt(ismth)
         if(icase.ge.10) then
            call freadr('cos=',1,cosq,1)
            call freadr('sin=',1,sinq,0)
         end if
c     read(5,*) x1,x2,x3,x4,x5,x6
         call freadr('x=',6,x0,1)
         call freadr('xe=',6,xe,1)
         call freadr('u=',1,u0,0)
         call freadr('xi=',6,xi,1)
         call freadr('v=',1,v0,0)
         call freadr('q=',6,q,1)
c     read(5,'(5f8.4)') amagx,amagy,,shftxb,shftyb,shftxa,shftya
         call freadr('amagx=',1,amagx,1)
         call freadr('amagy=',1,amagy,0)
         call freadr('shftxb=',1,shftxb,0)
         call freadr('shftyb=',1,shftyb,0)
         call freadr('shftxa=',1,shftxa,0)
         call freadr('shftya=',1,shftya,0)
         if(amagx.eq.0) amagx=1
         if(amagy.eq.0) amagy=1
c     read(5,'(9f8.4)') xmax,ymax,hscl,xdiv,ydiv
         if(abs(icase).le.8) then
            call freadr('xmin=',1,xmin,1)
            call freadr('xmax=',1,xmax,0)
            call freadr('ymin=',1,ymin,0)
            call freadr('ymax=',1,ymax,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('xdiv=',1,xdiv,0)
            call freadr('ydiv=',1,ydiv,0)
         else if (abs(icase).eq.9) then
            call freadr('ri1=',1,ri1,1)
            call freadr('ri2=',1,ri2,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('dtheta=',1,dtheta,0)
            call freadr('dphi=',1,dphi,0)
         else
            call freadr('xmin=',1,xmin,1)
            call freadr('xmax=',1,xmax,0)
            call freadr('ymin=',1,ymin,0)
            call freadr('ymax=',1,ymax,0)
            call freadr('hscl=',1,hscl,0)
            call freadr('xdiv=',1,xdiv,0)
         end if
         write(6,'(1h ,a,6f10.5)')
     *        'amagx,amagy,shftxb,shftyb,shftxa,shftya=',
     *        amagx,amagy,shftxb,shftyb,shftxa,shftya
         if(abs(icase).le.8) then
            write(6,'(1h ,a,7f10.4)')
     *           'xmin,xmax,ymin,ymax,hscl,xdiv,ydiv',
     *           xmin,xmax,ymin,ymax,hscl,xdiv,ydiv
         else if(abs(icase).eq.9) then
            write(6,'(1h ,a,5f10.4)') 'ri1,ri2,hscl,dtheta,dphi=',
     *           ri1,ri2,hscl,dtheta,dphi
         else
            write(6,'(1h ,a,6f10.4)')
     *           'xmin,xmax,ymin,ymax,hscl,xdiv',
     *           xmin,xmax,ymin,ymax,hscl,xdiv
         end if
c     read(5,113) (hmin(i),div(i),lf(i),i=1,ndiv)
         ndiv=1
         do 60 i=1,ndiv
            call freadr('hmin=',1,hmin(i),1)
            call freadr('div=',1,div(i),0)
            call freadi('lf=',1,lf(i),0)
            write(6,225) hmin(i),div(i),lf(i)
 60      continue
 225     format(1h ,'hmin=',f12.6,2x,'div=',f12.6,2x,'lf=',i3)
         call getsys(isys)
         if(isys.eq.3) then
            fact=2/sqrt(5.)
         else
            fact=1
         end if
         if(abs(icase).eq.0) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*al*fact
            ymax=ymax*al*fact
         else if(abs(icase).eq.1) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*al*fact
            ymax=ymax*al*fact
         else if(abs(icase).eq.2) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*cl
            ymax=ymax*cl
         else if(abs(icase).eq.3) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*cl
            ymax=ymax*cl
         else if(abs(icase).eq.4) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*al*tau**ismtr
            ymax=ymax*al*tau**ismtr
            amagy=amagy*tau**(-ismtr)
            ydiv=ydiv*al*tau**ismtr
         else if(abs(icase).eq.5) then
            xmin=xmin*al*fact
            xmax=xmax*al*fact
            ymin=ymin*al*tau**ismtr
            ymax=ymax*al*tau**ismtr
            amagy=amagy*tau**(-ismtr)
            ydiv=ydiv*al*tau**ismtr
         else if(abs(icase).eq.6) then
            if(isys.le.2) then
               xmin=xmin*al
               xmax=xmax*al
               ymin=ymin*al*tau**ismtr
               ymax=ymax*al*tau**ismtr
               amagy=amagy*tau**(-ismtr)
               ydiv=ydiv*al*tau**ismtr
            else
               xmin=xmin*cl
               xmax=xmax*cl
               ymin=ymin*al*tau**ismtr
               ymax=ymax*al*tau**ismtr
               amagy=amagy*tau**(-ismtr)
               ydiv=ydiv*al*tau**ismtr
            end if
         else if(abs(icase).eq.7) then
            xmin=xmin*al*tau**ismtr
            xmax=xmax*al*tau**ismtr
            ymin=ymin*al*tau**ismtr
            ymax=ymax*al*tau**ismtr
            amagx=amagx*tau**(-ismtr)
            amagy=amagy*tau**(-ismtr)
            xdiv=xdiv*al*tau**ismtr
            ydiv=ydiv*al*tau**ismtr
         else if(abs(icase).eq.8) then
            xmin=xmin*al*tau**ismtr
            xmax=xmax*al*tau**ismtr
            ymin=ymin*al*tau**ismtr
            ymax=ymax*al*tau**ismtr
            amagx=amagx*tau**(-ismtr)
            amagy=amagy*tau**(-ismtr)
            xdiv=xdiv*al*tau**ismtr
            ydiv=ydiv*al*tau**ismtr
         else if(abs(icase).eq.9) then
            xmax=ri1*pi
            ymax=ri2*pi
         else
            xmin=xmin*al*tau**ismtr
            xmax=xmax*al*tau**ismtr
            ymin=ymin*al*tau**ismtr
            ymax=ymax*al*tau**ismtr
            amagx=amagx*tau**(-ismtr)
            amagy=amagy*tau**(-ismtr)
            xdiv=xdiv*al*tau**ismtr
            ydiv=ydiv*al*tau**ismtr
         end if
         if(abs(icase).le.8) then
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
         else if(abs(icase).eq.9) then
            ixm=180/dtheta+0.001
            iym=180/dphi+0.001
         else
            ixm=(xmax-xmin)/xdiv+0.001
            iym=0
         end if
         if(abs(icase).eq.10) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=xmin*cosq
            qd(5)=xmin*sinq
            qd(6)=0
         else if(abs(icase).eq.11) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(5)=xmin*cosq
            qd(6)=xmin*sinq
            qd(4)=0
         else if(abs(icase).eq.12) then
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=xmin*cosq
            qd(6)=xmin*sinq
            qd(5)=0
         else
            qd(1)=0
            qd(2)=0
            qd(3)=0
            qd(4)=0
            qd(5)=0
            qd(6)=0
         end if
         call getxe(qd,xf)
         call getxe(q,xd)
c     default value
c     
         write(6,'(1h ,5(a,i3))') 'ldev=',ldev
         write(6,'(1h ,a,2i3)') 'iplot,icase=',iplot,icase
         if(abs(icase).ge.10) then
            write(6,'(1h ,a,2f10.5)') 'cos,sin=',cosq,sinq
         end if
         write(6,'(1h ,a,6f10.5)') 'x=',x0
         write(6,303) 'xe=', (xe(j),j=1,6),'u=',u0
         write(6,303) 'xi=', (xi(j),j=1,6),'v=',v0
 303     format(1h ,a,6f8.5,2x,a,f8.5)
         write(6,'(1h ,a,6f10.5)') 'q=',q
         if(isys.le.2) then
            call getqe0(xe,qe)
         else if(isys.eq.3.or.isys.eq.4) then
            call tr6to5e(xe,x1)
            call getqe0(x1,qe)
         else if(isys.eq.5) then
            call getqe0(xe,qe)
         end if
c     call getqe0(xe,qe)
         qe(4)=0
         qe(5)=0
         qe(6)=0
         call getxe(qe,xe)
         
         if(isys.le.2) then
            call getqe0(xi,qi)
         else if(isys.eq.3.or.isys.eq.4) then
            call tr6to5e(xi,x1)
            call getqe0(x1,qi)
         else if(isys.eq.5) then
            call getqe0(xi,qi)
         end if
c     call getqe0(xi,qi)
         qi(1)=0
         qi(2)=0
         qi(3)=0
         call getxe(qi,xi)
         do i=1,6
            xs(i)=x0(i)+xe(i)*u0+xi(i)*v0-xd(i)+xf(i)
         end do
         call smtrx(xs,xd,-ismtr)
         call vcpyr(xd,xs,6)

c         if(isys.le.2) then
c            al=al*tau**ismtr
c         else
c            al=al*tau**ismtr
c         end if
         
c     draw fourier or patterson map
c     
c     
c     hmin(i): minimum hight in i-th range
c     div(i) : increase of hight in i-th range
c     lf(i)  : number of contour lines in i-th range
c     ndiv: number of ranges
c     ixm  : number of lattice points along x-axis
c     iym  : number of lattice points along y-axis
c     am,bm : work array to store the x,y coordinates on a contour line
c             (size=5000)
c     xy: hight of the function at the lattice point (i,j)
c     ixm,iym : size of to dimensional array xy
c
         iw=0
         call getsys(isys)
         if(isys.eq.3) then
            fact=2/sqrt(5.)
         else
            fact=1
         end if
c     call vcpyi(icased(1,icase),iax,6)
         if(abs(icase).le.8) then
            ixm=(xmax-xmin)/xdiv+0.001
            iym=(ymax-ymin)/ydiv+0.001
         else if(abs(icase).eq.9) then
            ixm=180/dtheta+0.001
            iym=180/dphi+0.001
         else
            ixm=(xmax-xmin)/xdiv+0.001
            iym=0
         end if
         if(ixm.gt.ixmax) ixm=ixmax
         if(iym.gt.ixmax) iym=ixmax
c     call setxyt(1.,1.,0.,0)
         if(iplot.ne.-2.and.iplot.lt.0) call quit('invalid iplot ')
         call setxyt(1.,1.,0.,0)
         do iz=0,0
            if(ixm*iym.gt.10000) call quit('ixm*iym > 10000')
            call cxtoxy(cxyz,dxy,hm,km,lm,mm,nm,om,hmax,
     *           ixm,iym,iz,hscl,nt,idm,vol)
            call map1(hmin,div,lf,ndiv,am,bm,nbuf,iw,dxy,ixm,iym,ldev)
         end do
 800  continue
      call plote('end of plot')
      if(ldev.eq.1.or.ldev.eq.2) call getzap()
      call quit(' ')
      stop
      end

      block data
      common /cnfil/ nfil
      common /cdrwed/ q10(6),qm(6),qc(6)
      common /cjbc/ jbc,xs(6),iax(6)
      common /cnopen/ nopen
      common /corien/ iorien
      common /const0/ pi2,dmgn,irs(6)
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      parameter (sr=0.894427191,ct=0.447213596)
      parameter (su=-sr,cu=-ct)
      parameter (c1=0.309016994,c2=-0.809016994)
      parameter (c3=-c2,c4=-c1)
      parameter (s1=0.951056516,s2=0.587785252)
      parameter (s3=-s2,s4=-s1)
      parameter (c1sr=c1*sr,c2sr=c2*sr,c3sr=c3*sr,c4sr=c4*sr)
      parameter (s1sr=s1*sr,s2sr=s2*sr,s3sr=s3*sr,s4sr=s4*sr)
      parameter (tau=1.618034,tav=-tau)
      parameter (sqr2=1.41421, sqr3=1.73205, tqr3=-sqr3)
      parameter (t1=0.7071, t2=-0.7071)
      data nfil /1/
      data q10/0,0,0,0,1,0/
      data qm/1,0,0,-1,0,0/
      data qc/1,1,1,1,0,0/
      data iax /1,2,3,4,5,6/
      data  d0/ 0.,   0.,   1., 0.,   0.,   1.,
     *          c2sr, s2sr, ct, c4sr, s1sr, cu,
     *          c2sr, s3sr, ct, c4sr, s4sr, cu,
     *          c1sr, s4sr, ct, c3sr, s2sr, cu,
     *          sr,   0.,   ct, su,   0.,   cu,
     *          c1sr, s1sr, ct, c3sr, s3sr, cu/
c     for cubic and trigonal
      data  d1/ 1., tau,  0.,  tau, -1., 0.,
     *          tau, 0.,  1.,  -1,  0.,  tau,
     *          tau, 0., -1.,  -1., 0.,  tav,
     *          0.,  1., tav,   0., tau, 1.,
     *          -1., tau, 0.,  tav, -1., 0.,
     *          0.,  1., tau,   0., tau, -1./
c      for decagonal  (******reciprocal lattice*******)
      data d2/  c1, s1, 0., c2, s2, 0.,
     *          c2, s2, 0., c1, s4, 0.,
     *          c2, s3, 0., c1, s1, 0.,
     *          c1, s4, 0., c2, s3, 0.,
     *          0., 0., 1., 0., 0., 0.,
     *          0., 0., 0., 0., 0., 1./       
c     for octagonal
      data d3/  1., 0., 0., 1., 0., 0.,
     *          t1, t1, 0., t2, t1, 0.,
     *          0., 1., 0., 0.,-1., 0.,
     *          t2, t1, 0., t1, t1, 0.,
     *          0., 0., 1., 0., 0., 0.,
     *          0., 0., 0., 0., 0., 1./
      data pi2/6.283184/
      data iorien/1/
      data nopen/0/
c      return
      end

      subroutine vrcp(dxyz,cxyz,nt)
      real dxyz(*),cxyz(*)
      do 100 i=1,nt
         cxyz(i)=dxyz(i)
 100  continue
      return
      end

      subroutine vccp(ahkl,bhkl,nt)
      complex ahkl(*),bhkl(*)
      do 100 i=1,nt
         bhkl(i)=ahkl(i)
 100  continue
      return
      end

      subroutine cxtoxy(cxyz,dxy,hm,km,lm,mm,nm,om,hmax,
     *      ixm,iym,iz,hscl,nt,idm,vol)
      integer hm,km,lm,mm,nm,om
      integer h,k
      real cxyz(*)
      real dxy(0:ixm,0:iym)
      integer hmax(*)
      real rx(6)

      real q(6),xy(6)
      common /ctr/ itr(6,6)
      real xmin,xmax,ymin,ymax
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
c      integer rr
c      common /sym/ rr(6,6,60),tt(6,60),nsymo,icent
      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      common /cjbc/ jbc,xs(6),iax(6)
c      do 888 k=0,km
c         write(6,'(17i4)') (int(cxyz(0,k,l,0,0,0)),l=0,lm)
c 888  continue
      call getsys(isys)
      if(abs(icase).eq.0) then
          iqx=1
          iqy=2
      else if(abs(icase).eq.1) then
          iqx=1
          iqy=2
      else if(abs(icase).eq.2) then
          iqx=1
          iqy=3
      else if(abs(icase).eq.3) then
          iqx=2
          iqy=3
      else if(abs(icase).eq.4) then
          iqx=1
          iqy=4
      else if(abs(icase).eq.5) then
          iqx=2
          iqy=5
      else if(abs(icase).eq.6) then
          if(isys.le.2) then
              iqx=3
              iqy=6
          else
              iqx=3
              iqy=4
          end if
      else if(abs(icase).eq.7) then
          iqx=4
          iqy=5
      else if(abs(icase).eq.8) then
          iqx=5
          iqy=6
      else if(abs(icase).eq.10) then
          iqx=4
          iqy=5
      else if(abs(icase).eq.11) then
          iqx=5
          iqy=6
      else if(abs(icase).eq.12) then
          iqx=4
          iqy=6
      end if
      fact1=(xmax-xmin)/ixm
      if(abs(icase).le.9) then
          fact2=(ymax-ymin)/iym
      end if
      call getavr(iaver)
      do 120 h=0,ixm
      do 120 k=0,iym
         dxy(h,k)=0
 120  continue 
c     cxyz : electron density on a shifted lattice 
      do 100 h=0,ixm
      do 100 k=0,iym
         q(1)=0
         q(2)=0
         q(3)=0
         q(4)=0
         q(5)=0
         q(6)=0
         q(iqx)=xmin+h*fact1
         q(iqy)=ymin+k*fact2
         call getxe(q,xy)
         do 210 i=1,6
            xy(i)=xy(i)+xs(i)  ! origin shift
 210     continue
c         get average density over all equivalent points
c         dxy(h,k)=0
         if(iaver.ge.1) then
            nse=nsymo
            fact=hscl/(nsymo)
         else
            nse=1
            fact=hscl
         end if
c        average density of equivalent positions
         do 110 ns=1,nse
c           point rx is equivalent to xy
            call getrx(xy,rx,avrr,avrt,ns,0)
            call ctopx(rx)
c           density rx in centered lattice is obtained from primitive lattice
            call grad(cxyz,rx,hm,km,lm,mm,nm,om,hmax,dnsty)
c           call grad0(cxyz,rx,hm,km,lm,mm,nm,om,hmax,dnsty)
            dxy(h,k)=dxy(h,k)+dnsty*fact
c            write(6,*) 'ns,dnsty=',ns,dnsty
 110     continue
 100  continue
 500  continue
c      write(6,*) ' '
c      do 889 k=0,ixm
c         write(6,'(26i4)') (int(dxy(k,l)),l=0,iym/2)
c 889  continue
c 500  continue
      return
      end

      subroutine setavr(iaver)
      common /caver/ iavers
      iavers=iaver
      return
      end

      subroutine getavr(iaver)
      common /caver/ iavers
      iaver=iavers
      return
      end
      subroutine setsmt(ismth)
      common /csmth/ ismths
      ismths=ismth
      return
      end

      subroutine getsmt(ismth)
      common /csmth/ ismths
      ismth=ismths
      return
      end

      subroutine mtrtns(al,cl)
      common /cmtrt/ gab(6,6)
      call getsys(isys)
      if(isys.le.2) then
         a=al**2+1
         b=(al**2-1)/sqrt(5.)
         gab(1,1)=a
         gab(1,2)=b
         gab(1,3)=b
         gab(1,4)=b
         gab(1,5)=b
         gab(1,6)=b
         gab(2,2)=a
         gab(2,3)=b
         gab(2,4)=-b
         gab(2,5)=-b
         gab(2,6)=b
         gab(3,3)=a
         gab(3,4)=b
         gab(3,5)=-b
         gab(3,6)=-b
         gab(4,4)=a
         gab(4,5)=b
         gab(4,6)=-b         
         gab(5,5)=a
         gab(5,6)=b
         gab(6,6)=a
      else if(isys.eq.3) then
c         a=2*al**2
         a=al**2+1
         b=a/2
         gab(1,1)=a
         gab(1,2)=b
         gab(1,3)=b
         gab(1,4)=b
         gab(1,5)=0
         gab(1,6)=0
         gab(2,2)=a
         gab(2,3)=b
         gab(2,4)=b
         gab(2,5)=0
         gab(2,6)=0
         gab(3,3)=a
         gab(3,4)=b
         gab(3,5)=0
         gab(3,6)=0
         gab(4,4)=a
         gab(4,5)=0
         gab(4,6)=0
         gab(5,5)=cl**2
         gab(5,6)=0         
         gab(6,6)=0         
      else if(isys.eq.4) then
         call quit('mtrtns is not given for octagonal')
      end if
      do 10 i=1,6
      do 10 j=i+1,6
         gab(j,i)=gab(i,j)
 10   continue
      return
      end

      function vinpg(x,y)
      real x(6),y(6)
      common /cmtrt/ gab(6,6)
c      vector product of x, y in oblique coordinate system
c      gab : metric tensor
      vinpg=0
      do 100 i=1,6
      do 100 j=1,6
         vinpg=vinpg+x(i)*gab(i,j)*y(j)
 100  continue
      return
      end

      subroutine vcovr(x,y)
      real x(6),y(6)
      common /cmtrt/ gab(6,6)
c      get covariant elements y from contravariant elements
c      gab : metric tensor

      do 100 i=1,6
         y(i)=0
         do 110 j=1,6
            y(i)=y(i)+gab(i,j)*x(j)
 110     continue
 100  continue
      return
      end

      subroutine grad0(dxyz,rx,hm,km,lm,mm,nm,om,hmax,dnsty)
      integer hm,km,lm,mm,nm,om
      real dxyz(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      real rx(*)
      integer hmax(*)
      real rd(6)
      integer h0(6)
      integer h1(6)
      integer id(6)
      real grd(6),dx(6)
      data eps/0.0001/

      do 220 i=1,6
         rd(i)=mod(rx(i),1.)
         if(rd(i).lt.0) rd(i)=rd(i)+1.   ! 0 <= rd(i) < 1
 220  continue
c     h0(i) (i=1,6)  grid point nearest to rd(i)
      do 300 i=1,6
         rd(i)=rd(i)*hmax(i)       ! 0 <= rd(i) < hmax(i)
         h0(i)=rd(i)+0.5           ! h0(i)-1/2 < rd(i) < h0(i)+1/2
         dx(i)=rd(i)-h0(i)         ! -1/2 < dx(i) < 1/2
         h0(i)=mod(h0(i),hmax(i))  ! 0 <= h0(i) < hmax(i)
 300  continue
c     h1(i) (i=1,6) second nearest points along 6 axes
      do 100 i=1,6
         if(abs(dx(i)).lt.eps) then
            id(i)=0
         else
            id(i)=sign(1.0,dx(i))   !  id(i)=-1, 0, or 1
         end if
         h1(i)=h0(i)+id(i)
         h1(i)=mod(h1(i),hmax(i))
         if(h1(i).lt.0) h1(i)=h1(i)+hmax(i)  ! 0 <= h1(i) < hmax(i)
 100  continue
c      write(6,'(a,6i4,6i4,6f8.5)') 'h0,h1,dx',h0,h1,dx
      dnst0 =dxyz(h0(1),h0(2),h0(3),h0(4),h0(5),h0(6))
      grd(1)=dxyz(h1(1),h0(2),h0(3),h0(4),h0(5),h0(6))
      grd(2)=dxyz(h0(1),h1(2),h0(3),h0(4),h0(5),h0(6))
      grd(3)=dxyz(h0(1),h0(2),h1(3),h0(4),h0(5),h0(6))
      grd(4)=dxyz(h0(1),h0(2),h0(3),h1(4),h0(5),h0(6))
      grd(5)=dxyz(h0(1),h0(2),h0(3),h0(4),h1(5),h0(6))
      grd(6)=dxyz(h0(1),h0(2),h0(3),h0(4),h0(5),h1(6))
      do 200 i=1,6
         grd(i)=(grd(i)-dnst0)*id(i)
 200  continue
      dnsty=dnst0
      do 210 i=1,6
         dnsty=dnsty+grd(i)*dx(i)
 210  continue
c      if(dnsty.lt.0) then
c         write(6,'(a,18i4)') 'h0,h1=',h0,h1,id
c         write(6,'(a,6f10.4)') 'grd=',grd
c         write(6,'(a,6f10.4)') 'dx=',dx
c         write(6,'(a,3f10.4)') 'dnst0,delt,dnsty=',dnst0,delt,dnsty
c      end if
      return
      end

      subroutine getsym
      integer rg(6,6,2)
      real    tg(6,2)
      integer avrr
      common /avrsym/ avrr(6,6,10),avrt(6,10),nsymo
      integer imps(60)
      character*30 crri(2),crrd(2)
      data crri /'x,z,t,u,v,y',
     *           '-u,y,z,-v,-x,-t'/
      data crrd /'y,z,t,-x-y-z-t,u,v',
     *           't,z,y,x,u,v'/
      call getsys(isys)
      nsymo=2
      do 10 k=1,nsymo
         if(isys.le.2) then
            call decode(crri(k),rg,tg,k)
            idm=6
         else if(isys.eq.3) then
            call decode(crrd(k),rg,tg,k)
            idm=5
         else if(isys.eq.4) then
            call quit('octagonal is not suported in getsym')
            idm=5
         else
            call quit('invalid isys')
         end if
 10   continue
c      do 998 k=1,2
c      do 999 i=1,6
c      write(6,'(6i3,f10.5)') (rg(i,j,k),j=1,6),tg(i,k)
c 999  continue
c      write(6,*) ' '
c 998  continue
      ige=1
      if(ige.eq.1) then
           call ordr(rg,imps,nsymo,idm)
           impt=1
           do 13 i=1,idm
           avrt(i,1)=0.0
           do 13 j=1,idm
           avrr(i,j,1)=0
           avrr(i,i,1)=1
 13        continue
           do 50 ns=1,nsymo
           imp=imps(ns)
c          modified to include the pure translation
           if(imp.eq.1) then
             if(tg(1,ns).ne.0.or.tg(2,ns).ne.0.or.tg(3,ns).ne.0
     *      .or.tg(4,ns).ne.0.or.tg(5,ns).ne.0.or.tg(6,ns).ne.0)
     *       imp=2
           end if
           do 11 i=2,imp
           do 11 j=1,impt
           mp1=j+(i-2)*impt
           mp2=j+(i-1)*impt
 11        call mpso(rg,tg,ns,avrr,avrt,mp1,mp2,idm)
 50        impt=impt*imp
           nsymo=impt
c      write(6,*) 'avrr=',avrr
c      write(6,*) 'avrt=',avrt
           call wtsymo(avrr,avrt,nsymo,idm)
      end if
 81   continue
      return
      end

      subroutine grad(dxyz,rx,hm,km,lm,mm,nm,om,hmax,dnsty)
      integer hm,km,lm,mm,nm,om
      real dxyz(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      real rx(*)
      integer hmax(*)
      real rd(6)
      integer h0(6),h1(6),h2(6),h(6),k(6),l(6),m(6)
      integer id(6)
      real grd0(6),grd1(6),grd2(6),dx(6)
      real dgrd(6,6)
      data eps/0.0001/
      do 220 i=1,6
         rd(i)=mod(rx(i),1.)
         if(rd(i).lt.0) rd(i)=rd(i)+1.   ! coordinates in the unit cell
 220  continue
c     h0(i) (i=1,6)  nearest grid point for q(i)
      do 300 i=1,6
         rd(i)=rd(i)*hmax(i)       ! 0 <= rd(i) < hmax(i)
         h0(i)=rd(i)+0.5           ! h0(i)-1/2 < rd(i) < h0(i)+1/2
         dx(i)=rd(i)-h0(i)         ! -1/2 < dx(i) < 1/2
         h0(i)=mod(h0(i),hmax(i))  ! 0 <= h0(i) < hmax(i)
         if(abs(dx(i)).gt.0.5) call quit('dx > 1/2')
         if(abs(h0(i)).gt.hmax(i)) call quit('h0 > hmax')
 300  continue
c     h1(i),h2(i) (i=1,6) second nearest points along 6 axes
      do 100 i=1,6
         if(abs(dx(i)).lt.eps) then
            id(i)=0
         else
            id(i)=1
         end if
         h1(i)=h0(i)+id(i)
         h2(i)=h0(i)-id(i)
         h1(i)=mod(h1(i),hmax(i))
         h2(i)=mod(h2(i),hmax(i))
         if(h1(i).lt.0) h1(i)=h1(i)+hmax(i)
         if(h2(i).lt.0) h2(i)=h2(i)+hmax(i)
 100  continue
c     write(6,'(a,6i4,6i4,6f8.5)') 'h0,h1,dx',h0,h1,dx
      dnst0 =dxyz(h0(1),h0(2),h0(3),h0(4),h0(5),h0(6))
      do 330 i=1,6
         do 310 j=1,6
            if(j.eq.i) then
               h(j)=h1(j)
               k(j)=h2(j)
            else
               h(j)=h0(j)
               k(j)=h0(j)
            end if
 310     continue
         grd1(i)= (dxyz(h(1),h(2),h(3),h(4),h(5),h(6))-dnst0)
         grd2(i)=-(dxyz(k(1),k(2),k(3),k(4),k(5),k(6))-dnst0)
 330  continue
c         for i=1
c      grd1(1)= (dxyz(h1(1),h0(2),h0(3),h0(4),h0(5),h0(6))-dnst0)
c      grd2(1)=-(dxyz(h2(1),h0(2),h0(3),h0(4),h0(5),h0(6))-dnst0)
      do 200 i=1,6
         tmp1=dnst0+grd1(i)/2     ! density at i+0.5
         tmp2=dnst0-grd2(i)/2     ! density at i-0.5
         grd0(i)=tmp1-tmp2        ! average gradient at i
         dgrd(i,i)=grd1(i)-grd2(i)
 200  continue
      do 320 i=1,6
         do 350 j=1,6
         if(j.eq.i) go to 350
            do 340 n=1,6
               h(n)=h0(n)
               k(n)=h0(n)
               l(n)=h0(n)
               m(n)=h0(n)
 340        continue
         h(i)=h1(i)
         l(i)=h1(i)
         k(i)=h2(i)
         m(i)=h2(i)
         l(j)=h1(j)
         m(j)=h1(j)
         dnst00 =dxyz(h(1),h(2),h(3),h(4),h(5),h(6))
         dnst01 =dxyz(l(1),l(2),l(3),l(4),l(5),l(6))
         dnst02 =dxyz(k(1),k(2),k(3),k(4),k(5),k(6))
         dnst03 =dxyz(m(1),m(2),m(3),m(4),m(5),m(6))
c           for i=1, j=2
c        dnst00 =dxyz(h1(1),h0(2),h0(3),h0(4),h0(5),h0(6))
c        dnst01 =dxyz(h1(1),h1(2),h0(3),h0(4),h0(5),h0(6))
c        dnst02 =dxyz(h2(1),h0(2),h0(3),h0(4),h0(5),h0(6))
c        dnst03 =dxyz(h2(1),h1(2),h0(3),h0(4),h0(5),h0(6))
         grd3   =dnst01-dnst00
         grd4   =dnst03-dnst02
c        tmp3=dnst00+grd3/2       ! density at i+1, j+0.5
c        tmp4=dnst02+grd4/2       ! density at i-1, j+0.5
         tmp3=tmp1+grd3/2         ! density at i+0.5, j+0.5
         tmp4=tmp2+grd4/2         ! density at i-0.5, j+0.5
         grdxy1 =(tmp3-tmp4)      ! avrage gradient along i-th axis at i,j+0.5
         l(j)=h2(j)
         m(j)=h2(j)
         dnst00 =dxyz(h(1),h(2),h(3),h(4),h(5),h(6))
         dnst01 =dxyz(l(1),l(2),l(3),l(4),l(5),l(6))
         dnst02 =dxyz(k(1),k(2),k(3),k(4),k(5),k(6))
         dnst03 =dxyz(m(1),m(2),m(3),m(4),m(5),m(6))
c           for i=1, j=2
c        dnst00 =dxyz(h1(1),h0(2),h0(3),h0(4),h0(5),h0(6))
c        dnst01 =dxyz(h1(1),h2(2),h0(3),h0(4),h0(5),h0(6))
c        dnst02 =dxyz(h2(1),h0(2),h0(3),h0(4),h0(5),h0(6))
c        dnst03 =dxyz(h2(1),h2(2),h0(3),h0(4),h0(5),h0(6))
         grd5   =-(dnst01-dnst00)
         grd6   =-(dnst03-dnst02)
c        tmp5=dnst00-grd5/2      ! density at i+1, j-0.5
c        tmp6=dnst02-grd6/2      ! density at i-1, j-0.5
         tmp5=tmp1-grd5/2        ! density at i+0.5, j-0.5
         tmp6=tmp2-grd6/2        ! density at i-0.5, j-0.5
         grdxy2 =(tmp5-tmp6)     ! average gradient along i-th axis at i,j-0.5
         dgrd(i,j)=grdxy1-grdxy2 ! change in dgdx along j-th axis
 350     continue
 320   continue
c      call getqir(grd,grdq)
c      call getqe0(dx,dxq)
      delt=0
      do 210 i=1,6
         delt=delt+grd0(i)*dx(i)
 210  continue
      do 230 i=1,6
      do 230 j=1,6
         delt=delt+dgrd(i,j)*dx(i)*dx(j)
 230  continue
      dnsty=dnst0+delt
c      if(dnsty.lt.0) then
c         write(6,'(a,18i4)') 'h0,h1=',h0,h1,id
c         write(6,'(a,6f10.4)') 'grd=',grd
c         write(6,'(a,6f10.4)') 'dx=',dx
c         write(6,'(a,3f10.4)') 'dnst0,delt,dnsty=',dnst0,delt,dnsty
c      end if
      return
      end

      subroutine getqir(h,q)
      real h(*)
      real q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 100 i=1,6
           q(i)=0.
           do 101 j=1,6
                q(i)=q(i)+h(j)*dq(j,i)
 101       continue
      if(isys.le.2) then
          if(i.le.3) q(i)=q(i)/al
      else
          if(i.le.2) q(i)=q(i)/al
          if(i.eq.3) q(i)=q(i)/cl
      end if          
 100  continue
      return
      end

      subroutine symstf(ahkl,bhkl,sg0,hm,km,lm,mm,nm,om,hmax,idm,
     *      f000)
      integer hm,km,lm,mm,nm,om
      complex ahkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      complex bhkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      parameter (nhmax=2000)
      real fod(nhmax)
      integer st(nhmax,0:1)
      integer hmax(*)
      common /cjbc/ jbc,xs(6),iax(6)
      real dmgn
      common /const0/ pi2,dmgn,irs(6)
      common /kwave/ y(3,3,3),vc(3)
      integer rr
      common /sym/ rr(6,6,60),tt(6,60),nsymo,icent
      complex fctmp
      complex fobs,fcal
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),fcal(nhmax),sig(nhmax),is
      common /cphs/ jp(nhmax),nphase
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      integer hr(6),hi(6)
      integer h,k,l,m,n,o
      character brv
      call getbrv(brv)
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then   
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
      sg0=sg0*fact
      f000=f000*fact
c      initialization
      do 101 h=0,hm
      do 101 k=0,km
      do 101 l=0,lm
      do 101 m=0,mm
      do 101 n=0,nm
      do 101 o=0,om
         ahkl(h,k,l,m,n,o)=(0.0,0.0)
         bhkl(h,k,l,m,n,o)=(0.0,0.0)
 101  continue
      ahkl(0,0,0,0,0,0)=cmplx(f000,0.)
      bhkl(0,0,0,0,0,0)=cmplx(f000,0.)
c      sg(0,0,0,0,0,0)=0.1
      do i=1,is
         fod(i)=abs(fobs(i))
      end do
      call qsort(fod,st,jp,is)
      do 102 jd=is,1,-1
         j=jp(jd)
c         write(6,*) 'jd,j,fod(jd)',jd,j,fod(jd)
c        generation of equivalent reflections
         call vcpyi(hkln(1,j),hi,6)
c        transform indices from centered to primitive
c        if(iskip(hi,hmax).eq.1) go to 102
         do 20 ic=0,icent
         do 20 ns=1,nsymo
            call gethr(hi,hr,rr,ns,ic)
            call ctoph(hr)
            if(iskip(hr,hmax).eq.1) go to 20
            yt=0
            do 204 l=1,idm
               yt=yt+hi(l)*tt(l,ns)
 204        continue
c        strucutre factor for equivalent reflections
            fctmp=fcal(j)*cexp(pi2*yt*(0.,-1.))
            ac=real(fctmp)
            bc=aimag(fctmp)
            fc=abs(fctmp)
            argr=1.0
            argi=0.0
            if(fc.ne.0.0) argr=ac/fc
            if(fc.ne.0.0) argi=bc/fc
            ao=abs(fobs(j))*argr
            bo=abs(fobs(j))*argi
            do 112 k=1,idm
               if(hr(k).lt.0) then
                  hr(k)=hr(k)+hmax(k)
               end if
 112        continue
            do 113 k=idm+1,6
               hr(k)=0
 113        continue
            if(iplot.le.0) then    
c            mem map
               ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  cmplx(ao,bo)*fact ! fo
               bhkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  cmplx(ac,bc)*fact ! fc
            else if(iplot.eq.1) then    
c            patterson map
               ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  fobs(j)**2*fact       ! fo**2
               bhkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *         ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))
            else if(iplot.eq.2) then
c            fourier map
               ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  cmplx(ao,bo)*fact ! fo
               bhkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *         ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))
            else if(iplot.eq.2) then
c            fc fourier map
               ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  cmplx(ao,bo)*fact ! fo
               bhkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  cmplx(ac,bc)*fact !fc
            else if(iplot.eq.3) then
c            difference fourier map
               ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *                  (cmplx(ao,bo)-cmplx(ac,bc))*fact ! fo-fc
               bhkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *         ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))
            else
               call quit('invalid iplot')
            end if
c           sg(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=sig(j)           ! sigma
c            write(6,'(6i3,2f12.6)') hr,ao,ac
 20      continue
 102  continue
      return
      end

      function iskip(hkln,hmax)
      integer hkln(*),hmax(*)
      do 100 i=1,6
         if(abs(hkln(i)).gt.hmax(i)/2) then
            iskip=1
            return
         end if
 100  continue
      iskip=0
      return
      end

      subroutine vcpyr(a,b,n)
      real a(*),b(*)
      do 100 i=1,n
         b(i)=a(i)
 100  continue
      return
      end

      subroutine vcpyi(a,b,n)
      integer a(*),b(*)
      do 100 i=1,n
         b(i)=a(i)
 100  continue
      return
      end

c           call mapxyz(bhkl,cxyz,hmax,nt,idm,dmin,dmax,vol)
      subroutine mapxyz(ahkl,cxyz,hmax,nt,idm,dmin,dmax,vol)
      complex ahkl(*)
      real cxyz(*)
      integer hmax(*)
      call fftm(ahkl,cxyz,hmax,idm,1,icon)
      sum=0
      dmin=cxyz(1)/vol
      dmax=cxyz(1)/vol
      do 10 i=1,nt
         tmp=cxyz(i)/vol
         cxyz(i)=tmp
         dmin=min(tmp,dmin)
         dmax=max(tmp,dmax)
c         write(6,*) i,cxyz(i)
 10   continue
      return
      end

      subroutine getdmx(cxyz,nt,dmax,vol)
      real cxyz(*)
      dmax=cxyz(1)
      do 10 i=1,nt
         tmp=cxyz(i)
         cxyz(i)=tmp
         dmax=max(tmp,dmax)
 10   continue
      return
      end

      subroutine nrmxyz(cxyz,hmax,nt,idm,dmin,dmax,f000,vol)
      real cxyz(*)
      integer hmax(*)
      character brv
      call getbrv(brv)
c
c           normalization of cxyz > 0
      eps=f000/1000
      fact=(f000/vol)/(f000/vol+abs(dmin)+eps)
      do 30 i=1,nt
         cxyz(i)=(cxyz(i)+abs(dmin)+eps)*fact
 30   continue
      return
      end

c      subroutine intdns(cxyz,hm,km,lm,mm,nm,om,hmax,idm,dmin)
      subroutine intdns(cxyz,hmax,nt,idm,dmin)
      real cxyz(*)
      integer hmax(*)
      character brv
      data eps /1./
      call getbrv(brv)
c
c      initial electron density cxyz > 0
      do 30 i=1,nt
         if(cxyz(i).lt.eps) then
            cxyz(i)=eps
         end if
c         cxyz(i)=(cxyz(i)+abs(dmin)+eps)
c         write(6,*) i,cxyz(i)
 30   continue

      return
      end

      subroutine difxyz(ahkl,bhkl,sg0,f000,exyz,hmax,nt,idm,vol)
      complex ahkl(*)
      complex bhkl(*)
      real exyz(*)
      integer hmax(*)
      data eps/1.e-6/
      sg2=sg0**2
      vsg=vol*sg2/f000
c       sigma=sqrt(fo)*sg0 is assumed
c     bhkl(1)=(ahkl(1)-bhkl(1))/(vsg*(sqrt(f000)))
      bhkl(1)=(ahkl(1)-bhkl(1))/vsg
      do 10 i=2,nt
         if(abs(ahkl(i)).lt.eps) then
            bhkl(i)=(0.,0.)
         else
c           bhkl(i)=(ahkl(i)-bhkl(i))/(vsg*sqrt(abs(ahkl(i))))
            bhkl(i)=(ahkl(i)-bhkl(i))/vsg
         end if
 10   continue
      call fftm(bhkl,exyz,hmax,idm,1,icon)
      return
      end

      subroutine rfct(ahkl,bhkl,sg0,f000,hm,km,lm,mm,nm,om,hmax,c2d,
     *          rfact,rwfact,ip)
      integer hm,km,lm,mm,nm,om
      integer h,k,l,m,n,o
      complex ahkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      complex bhkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      integer hmax(*)
      parameter (nhmax=2000)
      complex fobs,fcal
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),fcal(nhmax),sig(nhmax),is
      common /cjbc/ jbc,xs(6),iax(6)
      integer hi(6)
      real*8 zsum, zdif,c2,ct
      character brv
      call getbrv(brv)
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
      zsum=0
      zdif=0
      c2=0
      ct=0
      isd=1
      sg2=sg0**2
c      sigma=sqrt(fo)*sg0 is assumed
      c2=abs(ahkl(0,0,0,0,0,0)-bhkl(0,0,0,0,0,0))**2/sg2
      ct=0
      dt=0
      do 40 i=1,is
         isd=isd+1
         call vcpyi(hkln(1,i),hi,6)
         call ctoph(hi)
         if(iskip(hi,hmax).eq.1) go to 40
         h=hi(1)
         k=hi(2)
         l=hi(3)
         m=hi(4)
         n=hi(5)
         o=hi(6)
         if(h.lt.0) h=h+hmax(1)
         if(k.lt.0) k=k+hmax(2)
         if(l.lt.0) l=l+hmax(3)
         if(m.lt.0) m=m+hmax(4)
         if(n.lt.0) n=n+hmax(5)
         if(o.lt.0) o=o+hmax(6)
         tmp=abs(ahkl(h,k,l,m,n,o))
         zsum=zsum+tmp
         zdif=zdif+abs(abs(ahkl(h,k,l,m,n,o))-abs(bhkl(h,k,l,m,n,o)))
         c2=c2+abs(ahkl(h,k,l,m,n,o)-bhkl(h,k,l,m,n,o))**2/sg2
         dt=dt+abs(ahkl(h,k,l,m,n,o)-bhkl(h,k,l,m,n,o))**2
         ct=ct+abs(ahkl(h,k,l,m,n,o))**2
c        c2=c2+abs(ahkl(h,k,l,m,n,o)-bhkl(h,k,l,m,n,o))**2/
c    *              (sqrt(tmp)*sg2)

         if(ip.eq.1.and.tmp*10.gt.f000) then
            write(6,'(6i4,4f9.2)') h,k,l,m,n,o,ahkl(h,k,l,m,n,o),
     *                 bhkl(h,k,l,m,n,o)
         end if
 40   continue
      c2d=c2/isd
      rfact=zdif/zsum
      rwfact=sqrt(dt/ct)
      return
      end

      subroutine expnd0(exyz,cxyz,dxyz,nt,ex1,ex2,dmin,dmax,f000,v,
     *              imode)
      real exyz(*)
      real cxyz(*)
      real dxyz(*)
      real*8 sum
      eps=f000/1000
      if(imode.eq.2) ddd=sqrt(dmax)
      sum=0
      do 10 i=1,nt
         tmp=cxyz(i)
         if(imode.eq.1) then
            ddd=sqrt(tmp)+eps
            dxyz(i)=tmp*exp(ex1*exyz(i)/ddd)
         else if(imode.eq.2) then
            dxyz(i)=tmp*exp(ex2*exyz(i)/ddd)
c         else
c            call quit('invalid mode')
         end if
c         write(6,*) i,dxyz(i),exyz(i)
      sum=sum+dxyz(i)
 10   continue
      fact=(f000/v)/(sum/nt)
      dmin=f000/v
      dmax=f000/v
      do 20 i=1,nt
         dxyz(i)=dxyz(i)*fact
         tmp=dxyz(i)
         dmin=min(tmp,dmin)
         dmax=max(tmp,dmax)
 20   continue
      if(abs(dmin-dmax).lt.eps) then
         write(6,*) 'fact,sum,nt',fact,sum,nt
         call quit(' ')
      end if
      return
      end

      subroutine expand0(exyz,cxyz,dxyz,nt,ex1,ex2,dmin,dmax,f000,imode)
      real exyz(*)
      real cxyz(*)
      real dxyz(*)
      real*8 sum
c      write(6,*) 'enter expand'
      eps=f000/1000
      if(imode.eq.2) ddd=sqrt(dmax)
      sum=0
      dmin=cxyz(1)
      dmax=cxyz(1)
      if(imode.eq.1) then
         do 10 i=1,nt
            tmp=cxyz(i)
            ddd=sqrt(tmp)+eps
            dxyz(i)=tmp*exp(ex1*exyz(i)/ddd)
            dmin=min(tmp,dmin)
            dmax=max(tmp,dmax)
            sum=sum+dxyz(i)
 10      continue
      else if(imode.eq.2) then
         do 20 i=1,nt
            tmp=cxyz(i)
            dxyz(i)=tmp*exp(ex2*exyz(i)/ddd)
            dmin=min(tmp,dmin)
            dmax=max(tmp,dmax)
            sum=sum+dxyz(i)
 20   continue
      end if
      if(abs(dmin-dmax).lt.eps) then
         call quit(' ')
      end if
      return
      end

      subroutine expand1(dxyz,cxyz,nt,ex1,ex2,dmin,dmax,f000,imode,vol)
      real cxyz(*)
      real dxyz(*)
      real*8 sum
c      write(6,*) 'enter expand'
c      eps=f000/1000
      eps=f000/1000/vol
      if(imode.eq.2) ddd=sqrt(dmax)
      sum=0
      dmin=cxyz(1)
      dmax=cxyz(1)
      if(imode.eq.1) then
         do 10 i=1,nt
            tmp=cxyz(i)
            ddd=sqrt(tmp)+eps
            dxyz(i)=tmp*exp(ex1*dxyz(i)/ddd)
            dmin=min(tmp,dmin)
            dmax=max(tmp,dmax)
            sum=sum+dxyz(i)
 10      continue
      else if(imode.eq.2) then
         do 20 i=1,nt
            tmp=cxyz(i)
            dxyz(i)=tmp*exp(ex2*dxyz(i)/ddd)
            dmin=min(tmp,dmin)
            dmax=max(tmp,dmax)
            sum=sum+dxyz(i)
 20   continue
      end if
      if(abs(dmin-dmax).lt.eps) then
         call quit(' ')
      end if
      return
      end

      subroutine saxis(ih)
      dimension ih(6)
      return
      end

      subroutine ahtobh(ahkl,bhkl,hm,km)
      integer hm,km
      complex ahkl(2,hm,km),bhkl(2,hm,km)
      integer h,k
      do 10 h=1,hm
      do 10 k=1,km
      do 10 i=1,2
 10   bhkl(i,h,k)=ahkl(i,h,k)
      return
      end

c           call phsnga(ahkl,bhkl,nt,hmax,idm)
      subroutine phsnga(ahkl,bhkl,nt,hmax,idm)
      integer hm,km,lm,mm,nm,om
      integer hmax(*)
      complex ahkl(*)
      complex bhkl(*)
      real zahkl,zbhkl,rbhkl,ibhkl
      parameter (nhmax=2000)
      complex fobs,fcal,fctmp
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),fcal(nhmax),sig(nhmax),is
      common /cphs/ jp(nhmax),nphase
      integer rr
      common /sym/ rr(6,6,60),tt(6,60),nsymo,icent
      data pi2/6.283184/
      complex ctmp
      integer hr(6),hi(6)
      character brv
      call getbrv(brv)
      if(brv.eq.'p') then
         fact=1
      else if(brv.eq.'f') then   
         fact=32
      else if(brv.eq.'i') then
         fact=2
      else
         call quit('invalid brv')
      end if
      zsum=0
      zdif=0
      do 10 i=1,nt
         zahkl=abs(ahkl(i))
         zbhkl=abs(bhkl(i))
         ctmp=bhkl(i)
         rbhkl=real(ctmp)
         ibhkl=aimag(ctmp)
         argr=1.0
         argi=0.0
         if(zbhkl.ne.0.0) then      
            argr=rbhkl/zbhkl
            argi=ibhkl/zbhkl
         end if
         ahkl(i)=cmplx(zahkl*argr,zahkl*argi)
 10   continue         
c ************* fixed phase **************
      hm=hmax(1)-1
      km=hmax(2)-1
      lm=hmax(3)-1
      mm=hmax(4)-1
      nm=hmax(5)-1
      om=hmax(6)-1            
      do 102 jd=is,is-nphase+1,-1
         j=jp(jd)
c         write(6,*) 'jd,j,fobs(j)',jd,j,abs(fobs(j))
c        generation of equivalent reflections
         call vcpyi(hkln(1,j),hi,6)
c        transform indices from centered to primitive
c        if(iskip(hi,hmax).eq.1) go to 102
         do 20 ic=0,icent
         do 20 ns=1,nsymo
            call gethr(hi,hr,rr,ns,ic)
            call ctoph(hr)
            if(iskip(hr,hmax).eq.1) go to 20
            yt=0
            do 204 l=1,idm
               yt=yt+hi(l)*tt(l,ns)
 204        continue
c        strucutre factor for equivalent reflections
            fctmp=fcal(j)*cexp(pi2*yt*(0.,-1.))
            ac=real(fctmp)
            bc=aimag(fctmp)
            fc=abs(fctmp)
            argr=1.0
            argi=0.0
            if(fc.ne.0.0) argr=ac/fc
            if(fc.ne.0.0) argi=bc/fc
            ao=abs(fobs(j))*argr
            bo=abs(fobs(j))*argi
            do 112 k=1,idm
               if(hr(k).lt.0) then
                  hr(k)=hr(k)+hmax(k)
               end if
 112        continue
            do 113 k=idm+1,6
               hr(k)=0
 113        continue
            call pahkl(ahkl,hm,km,lm,mm,nm,om,ao,bo,hr,fact)
 20      continue
 102  continue
      return
      end

      subroutine pahkl(ahkl,hm,km,lm,mm,nm,om,ao,bo,hr,fact)
      integer hm,km,lm,mm,nm,om,hr(*)
      complex ahkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      ahkl(hr(1),hr(2),hr(3),hr(4),hr(5),hr(6))=
     *     cmplx(ao,bo)*fact    ! fo
      return
      end

c           call phsngb(bhkl,hm,km,lm,mm,nm,om,xs)
      subroutine phsngb(bhkl,hm,km,lm,mm,nm,om,xs)
      integer hm,km,lm,mm,nm,om
      integer h,k,l,m,n,o
      complex bhkl(0:hm,0:km,0:lm,0:mm,0:nm,0:om)
      real xs(6)
      real zbhkl,rbhkl,ibhkl
      complex ctmp
      common /const0/ pi2,dmgn,irs(6)
      zsum=0
      zdif=0
      do 10 h=0,hm
      do 10 k=0,km
      do 10 l=0,lm
      do 10 m=0,mm
      do 10 n=0,nm
      do 10 o=0,om
         yt=h*xs(1)+k*xs(2)+l*xs(3)+m*xs(4)+n*xs(5)+o*xs(6)
         phsfct=cexp(pi2*yt*(0.,-1.))
         zbhkl=abs(bhkl(h,k,l,m,n,o))
         ctmp=bhkl(h,k,l,m,n,o)*phsfct
         rbhkl=real(ctmp)
         ibhkl=aimag(ctmp)
         argr=1.0
         argi=0.0
         if(zbhkl.ne.0.0) then      
            argr=rbhkl/zbhkl
            argi=ibhkl/zbhkl
         end if
         bhkl(h,k,l,m,n,o)=cmplx(zbhkl*argr,zbhkl*argi)
 10   continue
      return
      end

      subroutine maphkl(cxyz,bhkl,hmax,nt,idm,vol)
      real cxyz(*)
      complex bhkl(*)
      integer hmax(*)
      call fftm(bhkl,cxyz,hmax,idm,-1,icon)
      vnt=vol/nt
      do 10 i=1,nt
         bhkl(i)=bhkl(i)*vnt
 10   continue
      return
      end

      subroutine setbrv(brv)
      character brv,brvs
      common /cbrv/ brvs
      brvs=brv
      return
      end

      subroutine getbrv(brv)
      character brv,brvs
      common /cbrv/ brvs
      brv=brvs
      return
      end

      subroutine getac(a,c)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      a=al
      c=cl
      return
      end

      subroutine rdhklq(qimax,frmt,hmax,ismtr,iaster)
      character*(*) frmt
      character aster
      integer hmax(*)
      integer h(6),hd(6),h1(6)
      parameter (nhmax=2000)
      complex fobs,fcal
      integer hkln
      common /fhkl/ hkln(6,nhmax),fobs(nhmax),fcal(nhmax),sig(nhmax),is
      real q(6)
      i=0
 100  continue
      read(2,frmt) (h1(j),j=1,6),fo,fc,aster,sg
      if(h1(1).eq.999) go to 300
      call smtrh(h1,h,ismtr)
      if(iaster.eq.0.or.
     *     (iaster.eq.1.and.aster.eq.' ').or.
     *     (iaster.eq.2.and.aster.eq.'*')) then
         call getqi0(h,q)
         qi=sqrt(q(4)**2+q(5)**2+q(6)**2)
         call vcpyi(h,hd,6)
         call ctoph(hd)
         if(qi.le.qimax.and.iskip(hd,hmax).eq.0) then
            i=i+1
            fobs(i)=cmplx(sign(fo,fc),0.)
            fcal(i)=cmplx(fc,0.)
            do 200 j=1,6
               hkln(j,i)=h(j)
 200        continue
            sig(i)=0.01
         end if
      end if
      go to 100
 300  continue
      is=i
      write(6,'(a,i5)') 'number of reflections used=',is
      close(2)
      return
      end

      subroutine smtrh(h,hd,ind)
      integer h(*),hd(*),ht(6)
      integer s0(6,6),t0(6,6),s1(6,6),t1(6,6),s(6,6)
*************icosahedral f or i*************
      data s0/ 1, 1, 1, 1, 1, 1,
     *         1, 1, 1,-1,-1, 1,
     *         1, 1, 1, 1,-1,-1,
     *         1,-1, 1, 1, 1,-1,
     *         1,-1,-1, 1, 1, 1,
     *         1, 1,-1,-1, 1, 1/
      data t0/-1, 1, 1, 1, 1, 1,
     *         1,-1, 1,-1,-1, 1,
     *         1, 1,-1, 1,-1,-1,
     *         1,-1, 1,-1, 1,-1,
     *         1,-1,-1, 1,-1, 1,
     *         1, 1,-1,-1, 1,-1/
***************decagonal*********************
      data s1/ 0, 0,-1,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1,-1, 0, 0, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t1/-1, 0,-1,-1, 0, 0,
     *         1, 0, 1, 0, 0, 0,
     *         0, 1, 0, 1, 0, 0,
     *        -1,-1, 0,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      call getsys(isys)
      if(isys.le.2) then
         if(ind.gt.0) call mcpy(s0,s)
         if(ind.lt.0) call mcpy(t0,s)
      else
         if(ind.gt.0) call mcpy(s1,s)
         if(ind.lt.0) call mcpy(t1,s)
      end if
c        similarity transformation for decagonal and icosahedral
      call vcpyi(h,ht,6)
      if(ind.eq.0) then
           call vcpyi(ht,hd,6)
      else if(ind.gt.0) then
c        ext ==> tau  int ==> tau**-1
          do 120 n=1,ind
          do 100 i=1,6
          hd(i)=0
              do 110 j=1,6
              hd(i)=hd(i)+s(i,j)*ht(j)
 110          continue
 100      continue
          if(n.ne.ind) call vcpyi(hd,ht,6)
 120      continue
          if(isys.le.2) then
              do 130 j=1,6
              hd(j)=hd(j)/2**ind
 130          continue
          end if
          return
      else if(ind.lt.0) then
c        ext ==> tau**-1  int ==> tau
          do 220 n=1,-ind
          do 200 i=1,6
          hd(i)=0
              do 210 j=1,6
              hd(i)=hd(i)+s(i,j)*ht(j)
 210          continue
 200      continue
          if(n.ne.-ind) call vcpyi(hd,ht,6)
 220      continue
          if(isys.le.2) then
              do 230 j=1,6
              hd(j)=hd(j)/2**abs(ind)
 230          continue
          end if
          return
      end if
      end

      subroutine smtrx(h,hd,ind)
      real h(*),hd(*)
      real s0(6,6),t0(6,6),s1(6,6),t1(6,6),s2(6,6),t2(6,6)
      real s(6,6),ht(6)
*************icosahedral f or i*************
      data s0/ 1, 1, 1, 1, 1, 1,
     *         1, 1, 1,-1,-1, 1,
     *         1, 1, 1, 1,-1,-1,
     *         1,-1, 1, 1, 1,-1,
     *         1,-1,-1, 1, 1, 1,
     *         1, 1,-1,-1, 1, 1/
      data t0/-1, 1, 1, 1, 1, 1,
     *         1,-1, 1,-1,-1, 1,
     *         1, 1,-1, 1,-1,-1,
     *         1,-1, 1,-1, 1,-1,
     *         1,-1,-1, 1,-1, 1,
     *         1, 1,-1,-1, 1,-1/
***************decagonal*********************
      data s1/ 0, 0,-1,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1,-1, 0, 0, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t1/-1, 0,-1,-1, 0, 0,
     *         1, 0, 1, 0, 0, 0,
     *         0, 1, 0, 1, 0, 0,
     *        -1,-1, 0,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
***************octagonal*********************
      data s2/ 1, 1, 0,-1, 0, 0,
     *         1, 1, 1, 0, 0, 0,
     *         0, 1, 1, 1, 0, 0,
     *        -1, 0, 1, 1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      data t2/-1, 1, 0,-1, 0, 0,
     *         1,-1, 1, 0, 0, 0,
     *         0, 1,-1, 1, 0, 0,
     *        -1, 0, 1,-1, 0, 0,
     *         0, 0, 0, 0, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      call getsys(isys)
      if(isys.le.2) then
         if(ind.gt.0) call mcpy(s0,s)
         if(ind.lt.0) call mcpy(t0,s)
      else if(isys.eq.3) then
         if(ind.gt.0) call mcpy(s1,s)
         if(ind.lt.0) call mcpy(t1,s)
      else if(isys.eq.4) then
         if(ind.gt.0) call mcpy(s2,s)
         if(ind.lt.0) call mcpy(t2,s)
      end if
c     if(isys.le.2) then
c         nend=ind*3
c      else
         nend=ind
c      end if
c        similarity transformation for decagonal and icosahedral
      call vcpyr(h,ht,6)
      if(ind.eq.0) then
           call vcpyr(ht,hd,6)
      else if(ind.gt.0) then
c        ext ==> tau  int ==> tau**-1
          do 120 n=1,nend
          do 100 i=1,6
          hd(i)=0
              do 110 j=1,6
              hd(i)=hd(i)+ht(j)*s(j,i)
 110          continue
 100      continue
          if(n.ne.nend) call vcpyr(hd,ht,6)
 120      continue
          if(isys.le.2) then
              do 130 j=1,6
              hd(j)=hd(j)/2**nend
 130          continue
          end if
          return
      else if(ind.lt.0) then
c        ext ==> tau**-1  int ==> tau
          do 220 n=1,-nend
          do 200 i=1,6
          hd(i)=0
              do 210 j=1,6
              hd(i)=hd(i)+ht(j)*s(j,i)
 210          continue
 200      continue
          if(n.ne.-nend) call vcpyr(hd,ht,6)
 220      continue
          if(isys.le.2) then
              do 230 j=1,6
              hd(j)=hd(j)/2**abs(nend)
 230          continue
          end if
          return
      end if
      end

      subroutine read2
      character*80 str
c      skip up to the header Nrefl  h k l etc.
      rewind 2
 100  continue
      read(2,'(a)',end=200) str
      if(str(1:6).eq.' Nrefl') then
            go to 76
      else
            go to 100
      end if
 76   continue
c      skip one line
      read(2,*)
      return
 200  call quit(' Nrefl not found')
      return
      end

      integer function check(ih,ik,h,k)
      integer ih,ik
      real h,k
      integer jh,jk
      jh=h*10
      jk=k*10
      if(h*10-jh.gt.0.5) then
          jh=jh+1
      else if(h*10-jh.lt.-.5) then
          jh=jh-1
      end if
      if(k*10-jk.gt.0.5) then
          jk=jk+1
      else if(k*10-jk.lt.-.5) then
          jk=jk-1
      end if
      if(ih.eq.jh.and.ik.eq.jk) then
          check=1
      else
          check=0
      end if
      return
      end

c      subroutine norml(n3,n)
c      real n3(*)
cc     normalization of n3
c      absh=vabs(n3,n)
c      do 10 i=1,n
c 10   n3(i)=n3(i)/absh
c      return
c      end

      subroutine savexc(xc,xold,ndiff)
      real xc(*),xold(3,*)
      xold(1,ndiff)=xc(1)
      xold(2,ndiff)=xc(2)
      xold(3,ndiff)=xc(3)
      return
      end

      subroutine savehr(hr,hold,ndiff)
      integer hr(*),hold(6,*)
      do 100 i=1,6
 100  hold(i,ndiff)=hr(i)
      return
      end

      integer function multpl(h)
      integer h(*)
      integer hr(6)
      integer hs
      common /cmult/ hs(6,120)
      integer r
      common /sym/ r(6,6,60),t(6,60),nsymo,icent
      mul=0
      do 100 ns=1,nsymo
      do 100 j=0,1
           call gethr(h,hr,r,ns,j)
           do 200 k=1,mul
           if(icomp(hr,hs(1,k)).eq.1) then
                go to 100
           end if
 200       continue
           mul=mul+1
           do 201 k=1,6
                hs(k,mul)=hr(k)
 201       continue
 100  continue
      multpl=mul
      return
      end

      integer function icomp(h,hr)
      integer h(*),hr(*)
      icomp=0
      do 100 i=1,6
           if(h(i).ne.hr(i)) return
 100  continue
      icomp=1
      return
      end

      subroutine savehc(hc,hold,ndiff)
      real hc(*),hold(6,*)
      do 100 i=1,6
 100  hold(i,ndiff)=hc(i)
      return
      end

      function atan0(y,x)
      data pi/3.141592/, eps/0.00001/
      if(abs(x).lt.eps) then
           if(y.gt.0) then
              atan0=pi/2
           else
              atan0=-pi/2
           end if
      else if(abs(y).lt.0) then
              atan0=0.
      else
              atan0=atan2(y,x)
      end if
      return
      end

      subroutine matinv(a,nm,n,b,m,determ,ipivot,index,pivot)
c
c    n*n matrix a is replaced by its inverse matrix a**-1
c    if m is nonzero vector b is replaced by a**-1*b
c
      dimension a(nm,nm),b(nm),ipivot(nm),index(nm,2),pivot(nm)
      equivalence (irow,jrow),(icolum,jcolum),(amax,t,swap)
      determ=1.0
      do 20 j=1,n
  20  ipivot(j)=0
      do 555 i=1,n
      amax=0.0
           do 105 j=1,n
           if(ipivot(j).eq.1) go to 105
           do 100 k=1,n
           if(ipivot(k)-1) 80,100,740
  80       if(abs(amax).ge.abs(a(j,k))) go to 100
           irow=j
           icolum=k
           amax=a(j,k)
 100       continue
 105  continue
      ipivot(icolum)=ipivot(icolum)+1
      if(irow.eq.icolum) go to 260
      determ=-determ
      do 200 l=1,n
      swap=a(irow,l)
      a(irow,l)=a(icolum,l)
 200  a(icolum,l)=swap
      if(m.eq.0) go to 260
           swap=b(irow)
           b(irow)=b(icolum)
           b(icolum)=swap
 260  index(i,1)=irow
      index(i,2)=icolum
      pivot(i)=a(icolum,icolum)
      determ=determ*pivot(i)
      a(icolum,icolum)=1.0
      do 350 l=1,n
 350  a(icolum,l)=a(icolum,l)/pivot(i)
      if(m.eq.0) go to 380
           b(icolum)=b(icolum)/pivot(i)
 380  do 550 l1=1,n
      if(l1.eq.icolum) go to 550
      t=a(l1,icolum)
      a(l1,icolum)=0.0
           do 450 l=1,n
 450       a(l1,l)=a(l1,l)-a(icolum,l)*t
      if(m.eq.0) go to 550
      b(l1)=b(l1)-b(icolum)*t
 550  continue
 555  continue
      do 710 i=1,n
      l=n+1-i
      if(index(l,1).eq.index(l,2)) go to 710
      jrow=index(l,1)
      jcolum=index(l,2)
           do 700  k=1,n
           swap=a(k,jrow)
           a(k,jrow)=a(k,jcolum)
 700       a(k,jcolum)=swap
 710  continue
 740  continue
      return
      end

      complex function cexpf(z)
      complex z,i
      data i/(0., 1.)/
      x=aimag(z)
      cexpf=cos(x)+i*sin(x)
      return
      end

      subroutine freadi(prfx,n,var,nl)
      integer var(*)
      character*(*) prfx
      integer ip,ep
c      parameter (BUFLEN=256)
      parameter (BUFLEN=80)
      character*80 buf
c      common /cbuf/ buf,ip
      common /cbuf1/ buf
      common /cbuf2/ ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      if(nl.eq.1) then
           read(inpu,'(a)',end=200) buf
           ip=1
      end if
      ipd=index(buf(ip:BUFLEN),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,BUFLEN
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue

          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp:BUFLEN),' ')
          ipd2=index(buf(jp:BUFLEN),',')
          if(ipd1.gt.0.and.ipd2.gt.0) then
              jp=min(ipd1,ipd2)+jp
          else if(ipd1.gt.0) then
              jp=ipd1+jp
          else if(ipd2.gt.0) then
              jp=ipd2+jp
          else
              jp=BUFLEN
          end if
c         jp: beginning of next prefix or value
 150  continue
      if(jp.eq.BUFLEN) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp),*) (var(i),i=1,n)
      ip=jp
      return
 200  continue
           write(iout,'(2a)') prfx,' not found'
           call quit(' ')
      return
      end

      subroutine freadr(prfx,n,var,nl)
      real var(*)
      character*(*) prfx
      integer ip,ep
c      parameter (BUFLEN=256)
      parameter (BUFLEN=80)
      character*80 buf
c      common /cbuf/ buf,ip
      common /cbuf1/ buf
      common /cbuf2/ ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      if(nl.eq.1) then
           read(inpu,'(a)',end=200) buf
           ip=1
      end if
      ipd=index(buf(ip:BUFLEN),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
c      if(ipd.eq.0) then
c           write(iout,'(2a)') prfx,' not found'
c           call quit(' ')
c      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,BUFLEN
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue
          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp:BUFLEN),' ')
          ipd2=index(buf(jp:BUFLEN),',')
          if(ipd1.gt.0.and.ipd2.gt.0) then
              jp=min(ipd1,ipd2)+jp
          else if(ipd1.gt.0) then
              jp=ipd1+jp
          else if(ipd2.gt.0) then
              jp=ipd2+jp
          else
              jp=BUFLEN
          end if
c         jp: beginning of next prefix or value
 150  continue
      if(jp.eq.BUFLEN) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp-1),*) (var(i),i=1,n)
      ip=jp
      return
 200  continue
          write(iout,'(2a)') prfx,' not found'
          call quit(' ')
      return
      end

      subroutine freadc(prfx,n,var,nl)
      character*(*) var(*)
      character*(*) prfx
      integer ip,ep
c      parameter (BUFLEN=256)
      parameter (BUFLEN=80)
      character*80 buf
c      common /cbuf/ buf,ip
      common /cbuf1/ buf
      common /cbuf2/ ip
      call getinp(inpu)
      call getout(iout)
      call getskp(iskp)
 100  continue
      lval=len(var(1))
      if(nl.eq.1) then
           read(inpu,fmt='(a)',end=200) buf(1:BUFLEN)
           ip=1
      end if
      ipd=index(buf(ip:BUFLEN),prfx)
      ip=ipd+ip-1
      if(ipd.eq.0) then
           if(iskp.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else if(nl.eq.0) then
               write(iout,'(2a)') prfx,' not found'
               call quit(' ')
           else
               go to 100
           end if
      end if
c      if(ipd.eq.0) then
c           write(iout,'(2a)') prfx,' not found'
c           call quit(' ')
c      end if
      ep=len(prfx)+ip-1
c     ep: end of the prefix
      if(n.eq.0) return
c       skip blanks between prefix and value or value and value
      jp=ep+1
      do 150 i=1,n
          do 10 j=jp,BUFLEN
              if(buf(j:j).ne.' ') go to 20
 10       continue
 20       continue
          jp=j
c         jp: beginning of i-th value
          ipd1=index(buf(jp+1:BUFLEN),'''')
          if(ipd1.gt.0) then
              jp=ipd1+jp+1
          else
              jp=BUFLEN
          end if
c         jp: beginning of the next prefix or value
 150  continue
      if(jp.eq.BUFLEN) then
          write(iout,'(3a)') 'value of ',prfx,' not fund'
          call quit(' ')
      end if
      read(buf(ep+1:jp-1),*) (var(i)(1:lval),i=1,n)
      ip=jp
      return
 200  continue
           write(iout,'(2a)') prfx,' not found'
           call quit(' ')
      return
      end

      function istrln(c)
      character*(*) c
      l=len(c)
      do 10 i=l,1,-1
      if(c(i:i).ne.' ') then
          istrln=i
          return
      end if
 10   continue
      istrln=0
      return
      end

      subroutine setinp(inpu)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      cinpu=inpu
      return
      end

      subroutine setout(iout)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      coutu=iout
      return
      end

      subroutine getinp(inpu)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      inpu=cinpu
      return
      end

      subroutine getout(iout)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      iout=coutu
      return
      end

      subroutine setskp(iskp)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      cskip=iskp
      return
      end

      subroutine getskp(iskp)
      integer cinpu, coutu,cskip
      common /ciount/ cinpu,coutu,cskip
      iskp=cskip
      return
      end

      subroutine quit(str)
      character*(*) str
      write(6,*) str
c      time=etime(tarray)
c      write(6,*) '******elapsed time=',time,'sec******'
      close(unit=2)
      close(unit=3)
c      call exit()
      stop
      end

      subroutine map1(hmin,div,lf,ndiv,a,b,ii,iw,xy,ixm,iym,ldev)
      dimension hmin(*),div(*),lf(*),a(0:*),b(0:*)
      dimension xy(0:ixm,0:iym)
      common /cjbc/ jbc,xs(6),iax(6)
      integer wtitl
      common /name/ wtitl,wts,angl
      character*80 title, cmap
      common /ctitle/ title
c     dimension ar(5000),br(5000)
      common /cdim/ idm,nde
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      real rc(3),xb(502),yb(502)
      save shftx1,shfty1
      data shftx1 /0./ shfty1/0./
c
c
c     hmin(i): minimum hight in i-th range
c     div(i) : increase of hight in i-th range
c     lf(i)  : number of contour lines in i-th range
c     ndiv: number of ranges
c     ixm  : number of lattice points along x-axis
c     iym  : number of lattice points along y-axis
c     a,b : work array to store the x,y coordinates on a contour line
c     xy(i,j): hight of the function at the lattice point (i,j)
c     ih,jh : size of to dimensional array xy
c
c
c     usage:  call setxyt(ax,bx,coscx,irl) before calling this routine
c             to set ax and bx (the length of x and y axes devided by
c             ixm and iym, respectively) making an angle theta
c             coscx=cosine(theta) and irl=0,1 mean right-hand  and left-hand
c             coordinate system. these are used for getting (orthogonal)
c             coordinates of lattice points specified by oblique coordinates
c
c
      jbc=5
      if(abs(jbc).eq.1) then
          cmap='Fo Fourier  '
      else if(abs(jbc).eq.2) then
          cmap='Fc Fourier  '
      else if(abs(jbc).eq.3) then
          cmap='D-Fourier   '
      else if(abs(jbc).eq.4) then
          cmap='Fo Patterson'
      else if(abs(jbc).eq.5) then
          cmap='Fc Patterson'
      end if
      dlm=2.1
c
c      call plots(0.0,0.0,ldev)
c      call plot(shftxb,shftyb,-3)
c     plot title
c      if(wtitl.eq.1) then
c           wtsd=0.2
c           call symbol(0.0,-wtsd*2.0,wtsd,title,angl,ntitle)
c           call symbol(0.0,-wtsd*4.0,wtsd,tmap,angl,nmap)
c      end if
c     plot boundary
      shftx1=shftx1+shftxb
      shfty1=shfty1+shftyb
      if(abs(icase).le.8) then
          if(icase.gt.0) then
          divx=1
          divy=1
          call drwbox(xmin,xmax,ymin,ymax,divx,divy,shftx1,shfty1)
          end if
      else if(abs(icase).eq.9) then
          call plot(shftx1,shfty1,-3)
          rc(1)=0
          rc(2)=0
          call setxyt(1.,1.,0.,0) 
          call abtoxy(rd1,rd2,1)
          call abtoxy(rc(1),rc(2),1)
          call sphere(rd1,rd2,rc)
      else
          divx=1
          divy=1
          call drwbox(xmin,xmax,ymin,ymax,divx,divy,shftx1,shfty1)
          if(ixm.ge.500) call quit('ixm > 500')
          do 20 i=0,ixm
          xb(i+1)=xmin+i*(xmax-xmin)/ixm
          yb(i+1)=(ymin+xy(i,0))/100
 20       continue
          np=ixm+1
          if(np.ge.500) call quit('np > 500')
          call abtoxy(xb,yb,np)
c          xb(np+1)=0.0
c          yb(np+1)=0.0
c          xb(np+2)=1.0
c          yb(np+2)=1.0
c          call line(xb,yb,np,1,0,0)
          call drwln(xb,yb,np)
          call plot(shftxa,shftya,-3)
c          call plote('end of plot')
c          if(ldev.eq.1.or.ldev.eq.2) call getzap()
          return
      end if
      call getsmt(ismth)
      if(ismth.eq.1) then
         call smthxy1(xy,ixm,iym)
      else if(ismth.eq.2) then
         call smthxy2(xy,ixm,iym)
      else
         call quit('invalid ismth')
      end if
      if(icase.gt.0) then
c         coordinates on a contour line
          ibtm=0
          do 10 nd=1,ndiv
          le=lf(nd)
          do 10 l=1,le
c           al: hight
            al=hmin(nd)+div(nd)*float(l-1)
             fx=(xmax-xmin)/ixm
             fy=(ymax-ymin)/iym
          call contpl(xy(0,0),al,1,ixm+1,1,iym+1,xmin,ymin,fx,fy,icon)
          if(icon.eq.1) then
             if(ibtm.eq.1) go to 30
          end if
          if(icon.eq.0) then
             if(ibtm.eq.0) ibtm=1
          end if
 10       continue
 30       continue
      end if
c     call plot(shftxa,shftya,-3)
      shftx2=shftxa+shftx1
      shfty2=shftya+shfty1
      if(icase.eq.7.or.icase.eq.8) then
          call dispfn(xy(0,0),ixm,iym,xmin,xmax,
     *                     ymin,ymax,-0.5,20.,shftx2,shfty2)
      end if
c      call plote('end of plot')
c      if(ldev.eq.1.or.ldev.eq.2) call getzap()
      return
      end

      subroutine smthxy0(cxy,ixm,iym)
      real cxy(0:ixm,0:iym)
      parameter (ixmax=100)
      real dxy(0:ixmax,0:ixmax)
      do i=1,ixm-1
         do j=1,iym-1
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=0,0
         do j=1,iym-1
            dxy(i,j)=(cxy(i,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=ixm,ixm
         do j=1,iym-1
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=1,ixm-1
         do j=0,0
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j)+2*cxy(i,j)+cxy(i,j+1))/8
         end do
      end do
      do i=1,ixm-1
         do j=iym,iym
            dxy(i,j)=(cxy(i-1,j)+2*cxy(i,j)+cxy(i+1,j))/8+
     *               (cxy(i,j-1)+2*cxy(i,j)+cxy(i,j))/8
         end do
      end do
      dxy(0,0)=cxy(0,0)
      dxy(ixm,0)=cxy(ixm,0)
      dxy(0,iym)=cxy(0,iym)
      dxy(ixm,iym)=cxy(ixm,iym)
      do i=0,ixm
         do j=0,iym
            cxy(i,j)=dxy(i,j)
         end do
      end do
      return
      end

      subroutine smthxy1(cxy,ixm,iym)
      real cxy(0:ixm,0:iym)
      parameter (ixmax=100)
      real dxy(0:ixmax,0:ixmax)
      do i=0,ixm
         i1=max(i-1,0)
         i2=i
         i3=min(i+1,ixm)
         do j=0,iym
            j1=max(j-1,0)
            j2=j
            j3=min(j+1,iym)
            dxy(i,j)=(cxy(i1,j2)+cxy(i2,j2)+cxy(i3,j2)+
     *                cxy(i2,j1)+cxy(i2,j2)+cxy(i2,j3))/6
         end do
      end do
      do i=0,ixm
         do j=0,iym
            cxy(i,j)=dxy(i,j)
         end do
      end do
      return
      end

      subroutine smthxy2(cxy,ixm,iym)
      real cxy(0:ixm,0:iym)
      parameter (ixmax=100)
      real dxy(0:ixmax,0:ixmax)
      do i=0,ixm
         i1=max(i-2,0)
         i2=max(i-1,0)
         i3=i
         i4=min(i+1,ixm)
         i5=min(i+2,ixm)
         do j=0,iym
            j1=max(j-2,0)
            j2=max(j-1,0)
            j3=j
            j4=min(j+1,iym)
            j5=min(j+2,iym)
            dxy(i,j)=(cxy(i1,j3)+cxy(i2,j3)+cxy(i3,j3)+
     *                cxy(i4,j3)+cxy(i5,j3)+
     *                cxy(i3,j1)+cxy(i3,j2)+cxy(i3,j3)+
     *                cxy(i3,j4)+cxy(i3,j5))/10
         end do
      end do
      do i=0,ixm
         do j=0,iym
            cxy(i,j)=dxy(i,j)
         end do
      end do
      return
      end

      subroutine dispfn(xy,nx,nz,xmin,xmax,zmin,zmax,ymin,ymax,
     *                                             shftxb,shftyb)
      real xy(0:nx,0:nz)
      common /cmtrx/ux,uy,uz,vx,vy,vz,wx,wy,wz
      common /const/icymin,icymax,cy0,dh
      parameter (ndmax=1000)
c      common /cynmx/ynmax(ndmax),ynmin(ndmax)
      common /cynmx/ynmax(0:ndmax),ynmin(0:ndmax)
      real xw(ndmax+2),yw(ndmax+2)
      real ud(4),vd(4)
c
c       dh:interval of hight with one color
      dh=1
      cy0=0
      icymin=ymin
      icymax=ymax
      call getamg(amagx,amagy)
      xa=xmin*amagx
      xb=xmax*amagx
      za=zmin*amagy
      zb=zmax*amagy
c      LIN
c      read(5,*) px,pz,py,shftz
      call freadr('px=',1,px,1)
      call freadr('py=',1,py,0)
      call freadr('pz=',1,pz,0)
      call freadr('shftz=',1,shftz,0)
      write(6,*) 'direction of projection and shftz',px,py,pz,shftz
      if(abs(pz).lt.abs(px)) then
          call quit('give abs(py) > abs(px)')
      end if
      rx=(xa+xb)/2
      ry=0
      rz=(za+zb)/2
c      ul=xa+3
c      ur=xb-3
      vb=za
      vt=zb
c     window(ul,vb,ur,vt)
      fx=1
      fy=1
c     xd=-ul*fx
c     yd=-vb*fy
      xd=0
      yd=0
c      define a transformation matrix
c           
c        ux,uy,uz,0
c        vx,vy,vz,0
c        wx,wy,wz,0    
c        x1,x2,x3,1
c
c     wx,wy,wz:normalized px,py,pz
      pn=sqrt(px**2+py**2+pz**2)
      wx=px/pn
      wy=py/pn
      wz=pz/pn
      if(wx.eq.0.and.wz.eq.0) then
          ux=-wy
          uy=0
          uz=0
          vx=0
          vy=0
          vz=1
          ru=-rx*wy
          rv=rz
          rzx=0
      else 
          ey=sqrt(wx**2+wz**2)
          ux=wz/ey
          uy=0
          uz=-wx/ey
          vx=wy*uz
          vy=ey
          vz=-wy*ux
          call prj(rx,ry,rz,ru,rv)
      end if

      call prj(xa,0.,za,ud(1),vd(1))
      call prj(xb,0.,za,ud(2),vd(2))
      call prj(xb,0.,zb,ud(3),vd(3))
      call prj(xa,0.,zb,ud(4),vd(4))
      umin=ud(1)
      umax=ud(1)
      do 9 i=2,4
          if(ud(i).lt.umin) umin=ud(i)
          if(ud(i).gt.umax) umax=ud(i)
 9    continue
      x3=(shftxb)/29.7
      y3=(shftyb)/29.7
      x4=(umax-umin+shftxb)/29.7
      y4=(zb-za+shftyb)/29.7
c     call setxyt(1.,1.,0.,0)
      call setxy0(1.,1.,0.,0)
      call abtoxy(x3,y3,1)
      call abtoxy(x4,y4,1)
      call window(umin,za,umax,zb)
      call viewpt(x3,y3,x4,y4)

********************
      mpl=3
      nx=nx*mpl
********************
      dz=(zb-za)/nz
      dx=(xb-xa)/nx
      dxux=dx*ux
      rzx=dz*uz/dxux
      ul=umin
      ur=umax
      if(ux.gt.0) then
c         uw=ul
          uw=umin
      else
c         uw=ur
          uw=umax
      end if
      ua=xa*ux+za*uz-ru
      ixda=int((ua-uw)/dxux+0.5)
      ixdmx=abs(int((ur-ul)/dxux+0.5))
      if(ixdmx.gt.ndmax) call quit('ixdmx > ndmax')
**************************
      do 10 ixd=0,ixdmx
         ynmax(ixd)=0
         ynmin(ixd)=1
 10   continue
      if(wz.ge.0) then
          nza=0
          nzb=nz
          nzs=1
      else
          nza=nz
          nzb=0
          nzs=-1
      end if
      do 21 iz=nza,nzb,nzs
          z=za+iz*dz-rz
          ixdaz=int(ixda+iz*rzx+0.5)
          ivis1=-1
          n=0
          do 20 ix=0,nx
              x=xa+ix*dx-rx
              modix=mod(ix,mpl)
              jx=ix/mpl
              y=xy(jx,iz)+(xy(jx+1,iz)-xy(jx,iz))*modix/mpl
              y=(y+shftz)/100-ry
              call prj(x,y,z,u,v)
              xn2=fx*(u+ru)+xd
              yn2=fy*(v+rv)+yd
              call ckhide(xn2,yn2,ix,ivis1,ixdaz,ixdmx,ihide)
              if(ihide.eq.0) then
                 n=n+1
                 if(n.gt.ndmax) call quit('n > ndmax')
                 xw(n)=xn2
                 yw(n)=yn2
              else
                 call drwln(xw,yw,n)
                 n=0
              end if
 20       continue
          call drwln(xw,yw,n)
 21   continue

********************
      nx=nx/mpl
      nz=nz*mpl
********************
      dx=(xb-xa)/nx
      dz=(zb-za)/nz
      dzuz=dz*uz
c     dzuz=abs(dz*uz)
      rxz=dx*ux/dzuz
      ul=umin
      ur=umax
      if(uz.gt.0) then
c          uw=ul
           uw=umin
      else
c          uw=ur
           uw=umax 
      end if
      ua=xa*ux+za*uz-ru
      izda=int((ua-uw)/dzuz+0.5)
      izdmz=abs(int((ur-ul)/dzuz+0.5))
      if(izdmz.gt.ndmax) call quit('izdmx > ndmax')
**************************
      do 30 izd=0,izdmz
         ynmax(izd)=0
         ynmin(izd)=1
 30   continue
      if(wx.ge.0) then
          nxa=0
          nxb=nx
          nxs=1
      else
          nxa=nx
          nxb=0
          nxs=-1
      end if
      do 41 ix=nxa,nxb,nxs
          x=xa+ix*dx-rx
          izdax=int(izda+ix*rxz+0.5)
          ivis1=-1
          n=0
          do 40 iz=0,nz
              z=za+iz*dz-rz
              modiz=mod(iz,mpl)
              y=xy(ix,iz/mpl)+(xy(ix,iz/mpl+1)-xy(ix,iz/mpl))*modiz/mpl
              y=(y+shftz)/100-ry
              call prj(x,y,z,u,v)
              xn2=fx*(u+ru)+xd
              yn2=fy*(v+rv)+yd
              call ckhide(xn2,yn2,iz,ivis1,izdax,izdmz,ihide)
              if(ihide.eq.0) then
                 n=n+1
                 if(n.gt.ndmax) call quit('n > ndmax')
                 xw(n)=xn2
                 yw(n)=yn2
              else
                 call drwln(xw,yw,n)
                 n=0
              end if
 40       continue
          call drwln(xw,yw,n)
 41   continue

      return
      end

      subroutine drwln(xw,yw,n)
      real xw(*),yw(*)
      if(n.gt.1) then
         xw(n+1)=0
         yw(n+1)=0
         xw(n+2)=1
         yw(n+2)=1
         call line(xw,yw,n,1,0,0)
      end if
      return
      end

      subroutine prj(rx,ry,rz,ru,rv)
      common /cmtrx/ux,uy,uz,vx,vy,vz,wx,wy,wz
      ru=rx*ux+ry*uy+rz*uz
      rv=rx*vx+ry*vy+rz*vz
      return
      end

      subroutine ckhide(xn2,yn2,ix,ivis1,ixdaz,ixdmx,ihide)
c     HDDRAW
c     common /const/icymin,icymax,cy0,dh
      parameter (ndmax=1000)
c      common /cynmx/ynmax(ndmax),ynmin(ndmax)
      common /cynmx/ynmax(0:ndmax),ynmin(0:ndmax)
      ihide=1
c      y2=y
      ixd=ixdaz+ix
      if(ixd.lt.0.or.ixd.gt.ixdmx) return
      if(ixd.ne.ixd1) then
         ynmx=ynmax(ixd)
         ynmn=ynmin(ixd)
      end if
      if(ynmx.lt.ynmn) then
         ynmax(ixd)=yn2
         ynmin(ixd)=yn2
         ivis=1
      else if(yn2.ge.ynmx) then
         ynmax(ixd)=yn2
         ivis=1
      else if(yn2.le.ynmn) then
         ynmin(ixd)=yn2
         ivis=1
      else
         ivis=0
      end if
      if(ivis1.gt.0.and.ivis.gt.0) then
         if(ivis1.gt.0) then
             xna=xn1
             yna=yn1
         else
             xna=xn1
             yna=yn1
         end if
         if(ivis.gt.0) then
             xnb=xn2
             ynb=yn2
         else
             xnb=xn2
             ynb=yn2
         end if
c         call draw(xna,yna,xnb,ynb)
         ihide=0
      end if
      xn1=xn2
      yn1=yn2
      ynmx1=ynmx
      ynmn1=ynmn
c      y1=y2
      ivis1=ivis
      ixd1=ixd
      return
      end

      subroutine drwbox(xmin,xmax,ymin,ymax,divx,divy,shftxb,shftyb)
      real xb(7),yb(7)
      call getamg(amagx,amagy)
      xxa=xmin*amagx
      xxb=xmax*amagx
      yya=ymin*amagy
      yyb=ymax*amagy
      dvx=divx*amagx
      dvy=divy*amagy
      xb(1)=xxa
      yb(1)=yya
      xb(2)=xxb
      yb(2)=yya
      xb(3)=xxb
      yb(3)=yyb
      xb(4)=xxa
      yb(4)=yyb
      xb(5)=xxa
      yb(5)=yya

      xb(6)=0.0
      yb(6)=0.0
      xb(7)=1.0
      yb(7)=1.0
      x1=xb(1)-0.1
      y1=yb(1)-0.1
      x2=xb(3)+0.1
      y2=yb(3)+0.1
      x3=(-0.1+shftxb)/29.7
      y3=(-0.1+shftyb)/29.7
      x4=(xb(3)-xb(1)+0.1+shftxb)/29.7
      y4=(yb(3)-yb(1)+0.1+shftyb)/29.7
c     call setxyt(1.,1.,0.,0)
      call setxy0(1.,1.,0.,0)
      call abtoxy(x3,y3,1)
      call abtoxy(x4,y4,1)
      call window(x1,y1,x2,y2)
      call viewpt(x3,y3,x4,y4)
      call setxy0(1.,1.,0.,0)
      call abtoxy(xb,yb,5)
      call line(xb,yb,5,1,0,0)
c     call axs(xmin,xmax,ymin,ymax,divx,divy)
      call axs(xxa,xxb,yya,yyb,dvx,dvy)
      x1=xb(1)
      y1=yb(1)
      x2=xb(3)
      y2=yb(3)
      x3=(shftxb)/29.7
      y3=(shftyb)/29.7
      x4=(xb(3)-xb(1)+shftxb)/29.7
      y4=(yb(3)-yb(1)+shftyb)/29.7
c     call setxyt(1.,1.,0.,0)
      call setxy0(1.,1.,0.,0)
      call abtoxy(x3,y3,1)
      call abtoxy(x4,y4,1)
      call window(x1,y1,x2,y2)
      call viewpt(x3,y3,x4,y4)
c     call setxy0(1.,1.,0.,0)
      call setxyt(1.,1.,0.,0)
      return
      end

      subroutine sphere(ri1,ri2,xy)
      real ri1,ri2,xy(*)
      call ellipse(0.,360.,0.,ri1,ri2,xy,0)
      return
      end

      subroutine getamg(amgx,amgy)
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      amgx=amagx
      amgy=amagy
      return
      end

      subroutine abtoxy(x,y,n)
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      dimension x(*),y(*)
      do 10 i=1,n
          x(i)=x(i)*ax+y(i)*bx*coscx
          y(i)=y(i)*bx*sincx
          if(irl.eq.1) y(i)=-y(i)
 10   continue
      return
      end

      subroutine xytotp(x,y,n)
      dimension x(*),y(*)
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      common /cabtxy/ xmin,xmax,ymin,ymax,ri1,ri2
      data pi/3.14159/
      do 20 i=1,n
          xd=x(i)
          yd=y(i)
          x(i)=cos(pi*xd/xmax)*ri1*ax
          y(i)=sin(pi*xd/xmax)*ri2*ax*sin(pi*yd/ymax-pi/2)
          if(irl.eq.1) y(i)=-y(i)
 20       continue
      return
      end

      subroutine axs(amin,amax,bmin,bmax,diva,divb)
      dimension ax(4),ay(4)
      ax(3)=0.0
      ay(3)=0.0
      ax(4)=1.0
      ay(4)=1.0
      offsx=mod(amin,diva)
      offsy=mod(bmin,diva)
      im=(amax-amin+offsx)/diva+0.001
      jm=(bmax-bmin+offsy)/divb+0.001
      do 60 i=0,im
      ax(1)=amin-offsx +diva*float(i)
      ay(1)=bmin
      call abtoxy(ax,ay,1)
      ax(2)=ax(1)
      ay(2)=ay(1)-0.05
 60   call line(ax,ay,2,1,0,0)
      do 61 j=0,jm
      ax(1)=amin
      ay(1)=bmin-offsy +divb*float(j)
      call abtoxy(ax,ay,1)
      ax(2)=ax(1)-0.05
      ay(2)=ay(1)
 61   call line(ax,ay,2,1,0,0)
      do 70 i=0,im
      ax(1)=amin-offsx +diva*float(i)
      ay(1)=bmax
      call abtoxy(ax,ay,1)
      ax(2)=ax(1)
      ay(2)=ay(1)+0.05
 70   call line(ax,ay,2,1,0,0)
      do 71 j=0,jm
      ax(1)=amax
      ay(1)=bmin-offsy +divb*float(j)
      call abtoxy(ax,ay,1)
      ax(2)=ax(1)+0.05
      ay(2)=ay(1)
 71   call line(ax,ay,2,1,0,0)
      return
      end

      subroutine fdmin(i1,i2,i,m1,dmin,a,b)
      dimension a(0:*),b(0:*)
      m1=i1
      do 10 j=i1,i2
      d=(a(j)-a(i))**2+(b(j)-b(i))**2
      if(j.eq.i1) then
          dmin=d
          m1=j
      else if(d.lt.dmin) then
          dmin=d
          m1=j
      end if
 10   continue
      return
      end

      subroutine intch(x1,y1,x2,y2)
      x=x1
      y=y1
      x1=x2
      y1=y2
      x2=x
      y2=y
      return
      end

      subroutine clin(a,b,n,ltyp,pitch)
      dimension a(*),b(*),ad(5002),bd(5002),d1(4)
      if(ltyp.eq.1) then
          a(n+1)=0.0
          b(n+1)=0.0
          a(n+2)=1.0
          b(n+2)=1.0
          call line(a,b,n,1,0,0)
          return
      end if
      if(ltyp.eq.2) then
           d1(1)=pitch/2
           d1(2)=d1(1)
           d1(3)=d1(1)
           d1(4)=d1(1)*5
      else if(ltyp.eq.3) then
           d1(1)=pitch
           d1(2)=d1(1)
           d1(3)=d1(1)
           d1(4)=d1(1)
      end if
      k=0
      is=2
      j=1
      ad(1)=a(1)
      bd(1)=b(1)
      dab=0.0
      dx=0.0
      i=1
      i1=1
 12   continue
      k=k+1
      k=mod(k-1,4)+1
      ad(1)=ad(j)
      bd(1)=bd(j)
      j=1
      dd=-dx
      do 10 i=is,n
          j=j+1
          if(j.gt.5000) then
              call quit(' ad(j),bd(j), j>5000 in clin ')
          end if
          da=a(i)-a(i-1)
          db=b(i)-b(i-1)
          dab=sqrt(da**2+db**2)
          d=dd+dab
          ad(j)=a(i)
          bd(j)=b(i)
          if(i.eq.n.and.d.lt.d1(k)) go to 11
          if(d.lt.d1(k)) go to 10
          dx=d1(k)-dd
          ad(j)=a(i-1)+da*dx/dab
          bd(j)=b(i-1)+db*dx/dab
          is=i
          if((k.eq.1.or.k.eq.3).and.i.ne.n) go to 12
 11       if((k.eq.1.or.k.eq.3).and.i.eq.n) go to 10
          ad(j+1)=0.0
          bd(j+1)=0.0
          ad(j+2)=1.0
          bd(j+2)=1.0
          call line(ad,bd,j,1,0,0)
          if(i.ne.n) go to 12
 10   dd=d
      return
      end

      subroutine setxyt(a,b,cosc,i)
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      ax=a*amagx
      bx=b*amagy
      coscx=cosc
      sincx=sqrt(1-coscx**2)
      irl=i
      return
      end

      subroutine setxy0(a,b,cosc,i)
      common /cabxy/ ax,bx,coscx,sincx,irl,amagx,amagy
      ax=a
      bx=b
      coscx=cosc
      sincx=sqrt(1-coscx**2)
      irl=i
      return
      end


c s.contpl : sub to contour plot *********************************
c  contpl(hh,h,is,ie,js,je,xmin,ymin,fx,fy)                      *
c    *subroutine to plot contour line of height h                *
c     within (is-ie),(js-je) of 2d array hh(i,j)                 *
c     fx: increment of x in cm per data                          *
c     fy: increment of y in cm per data                          *
c     the above fx and fy values are evaluated by setting        *
c     the cell edges of the x and y axes to be edge cm.          *
c    *contour points: got by linear interpolation of adjacent    *
c     lattice points (thus, at most 1 contour point between      *
c     adjacent lattice points, and 0, 1 or at most 2 contour     *
c     lines in a unit cell of the lattice)                       *
c    *if 2 contour points on a unit cell boundary, these are     *
c     connected uncoditionally                                   *
c    *if 4 contour points on a unit cell boundary (thus, 2       *
c     contour lines in a cell), point connection be determined   *
c     on condition that line be less curved                      *
c    *to do above (check curvature), need 1 former point,        *
c     thus, 1st point to start a contour line be chosen among    *
c     2-contour-point cells                                      *
c    *contour lines are classified into 2 types                  *
c     1) line(s) running from boundary to boundary               *
c     2) loop(s) self-closed                                     *
c---- arrays ----------------------------------------------------*
c   (a,ia): coordinate data of contour points                    *
c           (x,iy): for y-axes data (m points)                   *
c           (y,ix): for x-axes data (n-m points)                 *
c   (x,y) : coordinate data for 'line' routine                   *
c---- control indices -------------------------------------------*
c    m  : number of y-axes data           (a,ia array)           *
c    n  : number of total data (y + x)    (a,ia array)           *
c    im : start of 'live' y-axes data     (a,ia array)           *
c    in : end of 'live' x-axes data       (a,ia array)           *
c    imm: temporary 'im'                  (a,ia array)           *
c    inn: tnmporary 'in'                  (a,ia array)           *
c    ist: start point of line             (a,ia array)           *
c    ib : boundary index                                         *
c    np : number of 'next' points                                *
c    j  : index of line to be plotted     (x,y array)            *
c---- co-subroutines --------------------------------------------*
c   contpn(h,is,ie,js,je)                                       *
c       find contour points of height h within (is-ie),(js-je)   *
c       set (a,ia) array and m,n                                 *
c   ib=bndry(i,is,ie,js,je)  (function)                          *
c       if i-th point is not on boundary, ib=0                   *
c       if i is on upper- or left- boundary, ib= 1               *
c       if i is on lower- or right-boundary, ib=-1               *
c   exchng(i,im,in,j,fx,fy)                                      *
c       if(j.eq.-1), only exchange                               *
c       if(j.le.-2), not exchange, j=-j                          *
c       if(j.ne.-1), j=j+1, set x(j)=fx*(x-coordinate of i) &    *
c                               y(j)=fy*(y-coordinate of i)      *
c       if(i.le.m), exchange im and i, set i=im, im=im+1         *
c       if(i.gt.m), exchange i and in, set i=in, in=in-1         *
c   search(ib,i0,i1,im,in,np)                                    *
c       search (i0)->(i1)->'next'? among im-in                   *
c       if(i0=0), get search direction from ib                   *
c       if(i0.ne.0), ib: not used                                *
c       np=number of found 'next' points                         *
c       if(i0.ge.1), i0=optimal 'next' among np points           *
c       if(np.ge.1), set i0=i1, i1='next'                        *
c    line(x,y,j,inc,ltp,iem) : plotter routine 'line'            *
c*****************************************************************
c
      subroutine contpl(h,al,is,ie,js,je,xmin,ymin,fx,fy,icon)
      parameter (ndata=3000)
c     real hh(ie,je),p(2,ndata)
      real h(ie,je),p(2,ndata)
c     common /cont/ ch(ndata),x(ndata*10),y(ndata*10),a(ndata*10),
      common /cont/ x(ndata*10),y(ndata*10),a(ndata*10),
     *              ia(ndata*10),m,n
*     a function to rotate the figure in a 180-degree arc
      parameter (edge=18.0)
c     yy(t)=edge*sinphi-t

c *** find points to form contour line of height h
      icon=0
      call contpn(h,al,is,ie,js,je)
      if(n.le.0) then
         icon=1
         return
      end if
      if(n.gt.ndata) call quit('n > ndata') 
      im=1 
      in=n 
      ib=1 
c  select first point to start contour line 
   10 continue 
      j=-1
      jb=0
   15 imm=im 
      inn=in
   20 continue
      do 30 i1=imm,inn 
      ib=bndry(i1,is,ie,js,je)
      if(ib.ne.0) goto 40 
   30 continue
      if(jb.eq.1) then
      jb=2
      goto 15
      endif
      i1=imm
   40 continue
      call exchng(i1,imm,inn,j,fx,fy)
      i0=0 
c  search next (second) point 
      call search(ib,i0,i1,im,in,np)
      if(np.gt.1.and.jb.le.1) then
      jb=1 
      goto 20
      endif
      j=0 
c  evaluate first point coordinate 
      call exchng(i0,im,in,j,fx,fy) 
c  store first point location 
      ist=i0 
   50 continue 
c  evaluate next point coordinate 
      call exchng(i1,im,in,j,fx,fy) 
c  check if terminal point 
      ib=bndry(i1,is,ie,js,je) 
      if(im.gt.in) goto 60 
      if(ib.ne. 0) goto 70 
c  search next point 
      call search(ib,i0,i1,im,in,np) 
      if(np.ne. 0) goto 50 
   60 continue 
c  check if terminal point is connected to first point 
      call search(ib,i0,i1,ist,ist,np) 
      if(np.eq.1) then
      j=-j 
      call exchng(i1,im,in,j,fx,fy)
      endif
   70 continue 

c *** plot found contour line
      do 80 i=1,j
          p(1,i)=x(i)+xmin
          p(2,i)=y(i)+ymin
   80 continue
      call smooth(p,j,x,y,npoint)

      call getcas(icase)
      if(abs(icase).le.8) then
          call abtoxy(x(1),y(1),npoint)
      else if(abs(icase).eq.9) then
          call xytotp(x(1),y(1),npoint)
      end if

      if(al.gt.0.0) then
          call clin(x,y,npoint,1,0.1)
      else if(al.eq.0.0) then
          call clin(x,y,npoint,2,0.1)
      else if(al.lt.0.0) then
          call clin(x,y,npoint,3,0.1)
      end if
      if(im.lt.in) goto 10
      end

************************************************************************

      subroutine getcas(ic)
      real shftxb,shftyb,shftxa,shftya
      common /cplt/ icase,iplot,shftxb,shftyb,shftxa,shftya,cosq,sinq
      ic=icase
      return
      end

      subroutine smooth(p,n,x,y,npoint)
      parameter (ndata=3000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata),rslt(is,ntotal),x(*),y(*)

      iz=3
      call spline(p,n,iz,rslt,npoint)
      do 10 i=1,npoint
         x(i)=rslt(1,i)
         y(i)=rslt(2,i)
   10 continue
      end

************************************************************************

      subroutine spline(p,n,iz,rslt,npoint)
**    input data
*             p: coordinates of points
*             n: number of data
*            iz: number of interpolated points
**    output data
*          rslt: coordinates of total points
*        npoint: number of total points

      parameter (ndata=3000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata),rslt(is,ntotal),pi(is),pe(is)
c      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

*     corrected (iz in the original program = number of interpolated
*     points + 1)
      iz2=iz+1
      ic1=1
      ic2=1
*     pi and pe is not used in the present program because ic1=ic2=1
*     input pi here if ic1=2
*     input pe here if ic2=2
      call endpnt(p,n,ic1,ic2,pi,pe)
      call gauss(p,n)
      call curgen(p,n,iz2,rslt)
      npoint=(n-1)*iz2+1
      end

************************************************************************

      subroutine endpnt(p,n,ic1,ic2,pi,pe)
      parameter (ndata=3000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata),pi(is),pe(is)
      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

      if (ic1 .eq. 1) then
         a(1,2)=1.0
         a(1,3)=0.5
      else
         a(1,2)=1.0
         a(1,3)=0.0
         do 10 k=1,is
            b(k,1)=pi(k)
            u(k,1)=pi(k)
   10    continue
      end if

      do 30 k=1,n-1
         xd=0.0
         do 20 j=1,is
            xd=xd+(p(j,k+1)-p(j,k))*(p(j,k+1)-p(j,k))
   20    continue
         xl(k)=sqrt(xd)
   30 continue
      if (ic1 .ne. 2) then
         do 40 k=1,is
            b(k,1)=(3.0/(2.0*xl(1)))*(p(k,2)-p(k,1))
   40    continue
      end if
      if (ic2 .eq. 1) then
         a(n,1)=1.0
         a(n,2)=2.0
         do 50 k=1,is
            b(k,n)=(3.0/xl(n-1))*(p(k,n)-p(k,n-1))
   50    continue
      else
         a(n,1)=0.0
         a(n,2)=1.0
         do 60 k=1,is
            b(k,n)=pe(k)
            u(k,n)=pe(k)
   60    continue
      end if
      end

************************************************************************

      subroutine gauss(p,n)
      parameter (ndata=3000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata)
      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

      do 20 j=2,n-1
         a(j,1)=xl(j)
         a(j,2)=2.0*(xl(j)+xl(j-1))
         a(j,3)=xl(j-1)
         do 10 k=1,is
            b(k,j)=(3.0*(xl(j-1)*xl(j-1)*(p(k,j+1)-p(k,j)))
     &      +3.0*xl(j)*xl(j)*(p(k,j)-p(k,j-1)))/(xl(j)*xl(j-1))
   10    continue
   20 continue

      do 50 i=2,n
         d=a(i-1,2)/a(i,1)
         a(i,1)=0.0
         a(i,2)=a(i,2)*d-a(i-1,3)
         a(i,3)=a(i,3)*d
         do 30 k=1,is
            b(k,i)=b(k,i)*d-b(k,i-1)
   30    continue
         q=a(i,2)
         a(i,2)=1.0
         a(i,3)=a(i,3)/q
         do 40 k=1,is
            b(k,i)=b(k,i)/q
   40    continue
   50 continue
      do 70 k=1,is
         u(k,n)=b(k,n)
         do 60 j=1,n-1
            u(k,n-j)=b(k,n-j)-a(n-j,3)*u(k,n+1-j)
   60    continue
   70 continue
      end

************************************************************************

      subroutine curgen(p,n,iz,rslt)
      parameter (ndata=3000,is=2,ntotal=(ndata-1)*10+1)
      real p(is,ndata),rslt(is,ntotal),f(4,is)
      common /spl1/ a(ndata,3),b(is,ndata),u(is,ndata),xl(ndata)

      i=1
      do 40 j=1,n-1
         do 10 k=1,is
            f(1,k)=p(k,j)
            f(2,k)=u(k,j)
            f(3,k)=(3.0/(xl(j)*xl(j)))*(p(k,j+1)-p(k,j))
     &      -(1.0/xl(j))*(u(k,j+1)+2.0*u(k,j))
            f(4,k)=(-2.0/(xl(j)*xl(j)*xl(j)))*(p(k,j+1)-p(k,j))
     &      +(1.0/(xl(j)*xl(j)))*(u(k,j+1)+u(k,j))
   10    continue
         t=0.0
         do 30 l=1,iz
            do 20 k=1,is
               rslt(k,i)=((f(4,k)*t+f(3,k))*t+f(2,k))*t+f(1,k)
   20       continue
            i=i+1
            t=t+xl(j)/iz
   30    continue
   40 continue
      do 50 k=1,is
         rslt(k,i)=p(k,n)
   50 continue
      end

************************************************************************

c s.contpn: find contour points (max:225x73)
c
c     (a,ia): coordinate data of contour points
c             (x,iy): for y-axes data (m points)
c             (y,ix): for x-axes data (n-m points)
c
      subroutine contpn(hh,h,i1,i2,j1,j2)
      real hh(i2,j2)
      parameter (ndata=3000)
c     common /cont/ ch(ndata),x(ndata*10),y(ndata*10),a(ndata*10),
      common /cont/ x(ndata*10),y(ndata*10),a(ndata*10),
     *              ia(ndata*10),m,n
c     common /origin/i0,j0
c     set the origin (1988.7.1)
      data i0/0/
      data j0/0/

      m=0
      do 10 j=j1-j0,j2-j0
      do 10 i=i1-i0,i2-i0-1
      if((hh(i,j)-h)*(hh(i+1,j)-h).gt.0.) goto 10
      if(hh(i,j).ge.h.and.hh(i+1,j).ge.h) goto 10
      m=m+1
      d=(hh(i,j)-h)/(hh(i,j)-hh(i+1,j))
      if(d.lt.0.01) d=0.01
      if(d.gt.0.99) d=0.99
c     a(m)=i+i0+d
      a(m)=float(i)+float(i0)+d
      ia(m)=j+j0
c  20 continue
   10 continue
      n=m
      do 40 i=i1-i0,i2-i0
      do 40 j=j1-j0,j2-j0-1
      if((hh(i,j)-h)*(hh(i,j+1)-h).gt.0.) goto 40
      if(hh(i,j).ge.h.and.hh(i,j+1).ge.h) goto 40
      n=n+1
      if(n.gt.ndata*10) goto 90
      d=(hh(i,j)-h)/(hh(i,j)-hh(i,j+1))
      if(d.lt.0.01) d=0.01
      if(d.gt.0.99) d=0.99
      a(n)=float(j)+float(j0)+d
      ia(n)=i+i0
c  50 continue
   40 continue
c     display(10,1010,line=14) h,i1,i2,j1,j2,m,n
      write(6,1010) h,i1,i2,j1,j2,m,n
 1010 format(' h=',f7.1,' (',2i4,') (',2i4,') m=',i4,' n=',i4)
      return
c  90 display(10,1020,line=14) h,i1,i2,j1,j2
   90 write(6,1020) h,i1,i2,j1,j2
 1020 format(' h=',f7.1,' (',2i4,')  (',2i4,') overflow')
      return
      end

************************************************************************

c s.bndry : support contplt routine
c     check if boundary point
      function bndry(i,is,ie,js,je)
      parameter (ndata=3000)
c     common /cont/ ch(ndata),x(ndata*10),y(ndata*10),a(ndata*10),
      common /cont/ x(ndata*10),y(ndata*10),a(ndata*10),
     *              ia(ndata*10),m,n
      bndry=0
      if(i.le.m) then
      if(ia(i).eq.js) bndry=1
      if(ia(i).eq.je) bndry=-1
      else
      if(ia(i).eq.is) bndry=1
      if(ia(i).eq.ie) bndry=-1
      endif
      return
      end

************************************************************************

c s.exchng : support contplt routine
c     replace data in (a,ia) array for contour plot
c
      subroutine exchng(i,im,in,j,fx,fy)
      parameter (ndata=3000)
c     common /cont/ ch(ndata),x(ndata*10),y(ndata*10),a(ndata*10),
      common /cont/ x(ndata*10),y(ndata*10),a(ndata*10),
     *              ia(ndata*10),m,n
      if(j.eq.-1) goto 10
      ib=1
      if(j.le.-2) then
      ib=-1
      j=-j
      endif
      j=j+1
      if(i.le.m) then
      x(j)=(a(i)-1.)*fx
      y(j)=(ia(i)-1)*fy
*     write(6,'(4f6.2,2f8.5)') x(j),y(j),a(i),float(ia(i)),fx,fy
      else
      x(j)=(ia(i)-1)*fx
      y(j)=(a(i)-1.)*fy
*     write(6,'(4f6.2,2f8.5)') x(j),y(j),float(ia(i)),a(i),fx,fy
      endif
      if(ib.eq.-1) return
   10 continue
      if(i.le.m) ib=im
      if(i.gt.m) ib=in
      buf=a(ib)
      a(ib)=a(i)
      a(i)=buf
      ibuf=ia(ib)
      ia(ib)=ia(i)
      ia(i)=ibuf
      i=ib
      if(i.le.m) im=im+1
      if(i.gt.m) in=in-1
      return
      end

************************************************************************

c s.search : support contplt routine
c     search 'next' point
c
      subroutine search(ib,i0,i1,im,in,np)
      parameter (ndata=3000)
c     common /cont/ ch(ndata),x(ndata*10),y(ndata*10),a(ndata*10),
      common /cont/ x(ndata*10),y(ndata*10),a(ndata*10),
     *              ia(ndata*10),m,n
      dimension ik(3)
      ixy1=a(i1)
      iyx1=ia(i1)
      if(i0.le.0) goto 10
      if(i0.le.m) then
       x0=a(i0)
       ix0=x0
       iy0=ia(i0)
       y0=iy0
      else
       ix0=ia(i0)
       x0=ix0
       y0=a(i0)
       iy0=y0
      endif
      if(i1.le.m) then
       ib=iyx1-iy0
       x1=a(i1)
       y1=iyx1
      else
       ib=iyx1-ix0
       x1=iyx1
       y1=a(i1)
      endif
   10 continue
      if(ib.le.0) ib=-1
      if(ib.gt.0) ib=0
      k=0
      do 20 i=im,in
      if(i.eq.i1) goto 20
      if(i1.le.m.and.i.le.m) goto 20
      if(i1.gt.m.and.i.gt.m) goto 20
      if(ifix(a(i)).ne.iyx1+ib) goto 20
      if(.not.(ia(i).eq.ixy1.or.ia(i).eq.ixy1+1)) goto 20
      k=k+1
      ik(k)=i
   20 continue
      if(ib.eq.0) ib=1
      do 30 i=im,in
      if(i1.le.m.and.i.gt.m) goto 30
      if(i1.gt.m.and.i.le.m) goto 30
      if(ia(i).ne.iyx1+ib) goto 30
      if(ifix(a(i)).ne.ixy1) goto 30
      k=k+1
      ik(k)=i
   30 continue
      np=k
      if(np.eq.0) return
      if(np.eq.1) then
       i0=i1
       i1=ik(1)
       return
      endif
      d1=(x1-x0)**2+(y1-y0)**2
      tmin=10.
      do 40 k=1,np
      i=ik(k)
      if(i.le.m) then
       if(i1.le.m) goto 40
       x2=a(i)
       y2=ia(i)
      else
       if(i1.gt.m) goto 40
       x2=ia(i)
       y2=a(i)
      endif
      d=(x2-x1)**2+(y2-y1)**2
      d2=(x2-x0)**2+(y2-y0)**2
      t=(d1+d-d2)/sqrt(d*d1)
      if(t.ge.tmin) goto 40
       imin=i
       tmin=t
   40 continue
   50 continue
      i0=i1
      i1=imin
      return
      end

      subroutine ctoph(h)
      integer h(*)
      integer hd(6)
      character brv
c      transformation of indices from centered to primitive
      call getbrv(brv)
      if(brv.eq.'p') then
         return
      else if(brv.eq.'f') then
         call ftoph(h,hd)
         call vcpyi(hd,h,6)
      else if(brv.eq.'i') then
         call quit('brv=i is not allowed in the present ver.')
      else
         call quit('invalid brv')
      end if
      return
      end

      subroutine ptoch(h)
      integer h(*)
      integer hd(6)
      character brv
c      transformation of indices from primitive to centered
      call getbrv(brv)
      if(brv.eq.'p') then
         return
      else if(brv.eq.'f') then
         call ptofh(h,hd)
         call vcpyi(hd,h,6)
      else if(brv.eq.'i') then
         call quit('brv=i is not allowed in the present ver.')
      else
         call quit('invalid brv')
      end if
      return
      end

      subroutine ctopx(x)
      real x(*)
      real xd(6)
      character brv
c      transformation of indices from centered to primitive
      call getbrv(brv)
      if(brv.eq.'p') then
         return
      else if(brv.eq.'f') then
         call ftopx(x,xd)
         call vcpyr(xd,x,6)
      else if(brv.eq.'i') then
         call quit('brv=i is not allowed in the present ver.')
      else
         call quit('invalid brv')
      end if
      return
      end

      subroutine ptocx(x)
      real x(*)
      real xd(6)
      character brv
c      transformation of indices from primitive to centered
      call getbrv(brv)
      if(brv.eq.'p') then
         return
      else if(brv.eq.'f') then
         call ptofx(x,xd)
         call vcpyr(xd,x,6)
      else if(brv.eq.'i') then
         call quit('brv=i is not allowed in the present ver.')
      else
         call quit('invalid brv')
      end if
      return
      end


      subroutine ptofx(x,y)
      real x(*),y(*)
      integer qm(6,6)
      data qm / 0, 1, 0, -1, 0, 0,
     *          0, 0, 1, 0, 0, -1,
     *          1, 0, 0, 0, 1, 0,
     *          0, 0, 0, 0, -1, 1,
     *          0, 0, 0, 1, -1, 0,
     *          0, 0, 0, 1, 0, 1/
c      coordinate transformation from primitive to face-centerd
      do i=1,6
         y(i)=0
         do j=1,6
            y(i)=y(i)+qm(i,j)*x(j)/2
         end do
      end do
      return
      end

      subroutine ftopx(x,y)
      real x(*),y(*)
      integer qm(6,6)
      data qm / 0, 0, 2, 1, 1, -1,
     *          2, 0, 0, -1, 1, 1,
     *          0, 2, 0, 1, -1, 1,
     *          0, 0, 0, -1, 1, 1,
     *          0, 0, 0, -1, -1, 1,
     *          0, 0, 0, 1, -1, 1/
c      coordinate transformation from face-centerd to primitive
      do i=1,6
         y(i)=0
         do j=1,6
            y(i)=y(i)+qm(i,j)*x(j)
         end do
      end do
      return
      end

      subroutine ftoph(x,y)
      integer x(*),y(*)
      integer qm(6,6)
      data qm / 0, 1, 0, -1, 0, 0,
     *          0, 0, 1, 0, 0, -1,
     *          1, 0, 0, 0, 1, 0,
     *          0, 0, 0, 0, -1, 1,
     *          0, 0, 0, 1, -1, 0,
     *          0, 0, 0, 1, 0, 1/
c      coordinate transformation from face-centerd to primitive
      do i=1,6
         y(i)=0
         do j=1,6
            y(i)=y(i)+x(j)*qm(j,i)
         end do
         y(i)=y(i)/2
      end do
      return
      end

      subroutine ptofh(x,y)
      integer x(*),y(*)
      integer qm(6,6)
      data qm / 0, 0, 2, 1, 1, -1,
     *          2, 0, 0, -1, 1, 1,
     *          0, 2, 0, 1, -1, 1,
     *          0, 0, 0, -1, 1, 1,
     *          0, 0, 0, -1, -1, 1,
     *          0, 0, 0, 1, -1, 1/
c      coordinate transformation from primitive to face-centerd
      do i=1,6
         y(i)=0
         do j=1,6
            y(i)=y(i)+x(j)*qm(j,i)
         end do
      end do
      return
      end

      subroutine setd()
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      dimension ipv(6),ind(6,2),piv(6),b(6)
      common /cgetd/ det
      save icount
      data icount /0/
      data tau /1.618034/

      icount=icount+1
      if(icount.eq.1) then
c        initialize
         do 3 i=1,6
         do 3 j=1,6
 3       d0(i,j)=d0(i,j)/2
         do 4 i=1,6
         do 4 j=1,6
 4       d1(i,j)=d1(i,j)/sqrt(1+tau**2)/2
         do 5 i=1,6
         do 5 j=1,4
 5       d2(i,j)=d2(i,j)/sqrt(5.)
         do 6 i=1,6
         do 6 j=1,4
 6       d3(i,j)=d3(i,j)/2
         call trnsp(d0)
         call trnsp(d1)
         call trnsp(d2)
         call trnsp(d3)
      end if
      if(isys.eq.1) then
          call mcpy(d0,dq)
      else if(isys.eq.2) then
          call mcpy(d1,dq)
      else if(isys.eq.3.or.isys.eq.4) then
          call mcpy(d2,dq)
      else if(isys.eq.5) then
          call mcpy(d3,dq)
      else
          call quit('wrong isys')
      end if
      call mcpy(dq,dr)
      call matinv(dr,6,6,b,0,det,ipv,ind,piv)
      return
      end

      subroutine getdet(det0)
      common /cgetd/ det
      det0=det
      return
      end

      subroutine setdei()
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      common /cphasn/ amij1(3,3),amij2(3,3),amij3(3,3)
      common /cphonn/ bmij1(3,3)
      real fij1(3,3),fij2(3,3),fij3(3,3),fij4(3,3),fij5(3,3),fij6(3,3)
      integer q50(6,6),q30(6,10),q20(6,15),q10d0(6),q2d0(6),qmd0(6)
      common /cprjax/ q5p(6,6),q3p(6,10),q2p(6,15),q10d(6),q2d(6),qmd(6)
      real ctoh(6,6),htoc(6,6),ded(6,6)
      dimension ipv(6),ind(6,2),piv(6),b(6)
      real wmij2(3,3),wmij3(3,3),wmij4(3,3)
      save icount
      data icount /0/
c     for pentagonal D5 and decagonal
      data fij1 /0,0,0,0,0,0,0,0,1/
      data fij2 /1,0,0,0,0,0,0,0,0/
      data fij3 /0,0,0,0,1,0,0,0,0/
c     for cubic T and trigonal 
      data fij4 /1,0,0,0,1,0,0,0,1/
      data fij5 /0,0,0,0,0,0,0,0,1/
      data fij6 /1,0,0,0,1,0,0,0,0/
      data ctoh /0.0,     -0.816496, 0.816496, 0., 0., 0.,
     *           0.816496,-0.816496, 0.0,      0., 0., 0.,
     *           0.577350, 0.577350, 0.577350, 0., 0., 0.,
     *           0., 0., 0., 0.0,     -0.816496, 0.816496,
     *           0., 0., 0., 0.816496,-0.816496, 0.0,
     *           0., 0., 0., 0.577350, 0.577350, 0.577350/
      data q50/1,0,0,0,0,0,    ! 1
     *         0,1,0,0,0,0,    ! 2
     *         0,0,1,0,0,0,    ! 3
     *         0,0,0,1,0,0,    ! 4
     *         0,0,0,0,1,0,    ! 5
     *         0,0,0,0,0,1/    ! 6
      data q30/1,1,1,0,0,0,    ! 1
     *         1,0,1,1,0,0,    ! 2
     *         1,0,0,1,1,0,    ! 3
     *         1,0,0,0,1,1,    ! 4
     *         1,1,0,0,0,1,    ! 5  3-fold axis in cubic approximant
     *         0,1,1,0,-1,0,   ! 6
     *         0,0,1,1,0,-1,   ! 7
     *         0,-1,0,1,1,0,   ! 8
     *         0,0,-1,0,1,1,   ! 9
     *         0,1,0,-1,0,1/   ! 10
      data q20/1,1,0,0,0,0,    ! 1
     *         1,0,1,0,0,0,    ! 2
     *         1,0,0,1,0,0,    ! 3
     *         1,0,0,0,1,0,    ! 4  y-axis in cubic approximant
     *         1,0,0,0,0,1,    ! 5
     *         0,1,1,0,0,0,    ! 6  x-axis in cubic approximant
     *         0,0,1,1,0,0,    ! 7
     *         0,0,0,1,1,0,    ! 8
     *         0,0,0,0,1,1,    ! 9
     *         0,1,0,0,0,1,    ! 10
     *         0,1,0,0,-1,0,   ! 11
     *         0,0,1,0,0,-1,   ! 12
     *         0,-1,0,1,0,0,   ! 13
     *         0,0,-1,0,1,0,   ! 14  normal to 3-fold axis in cubic apprx.
     *         0,0,0,-1,0,1/   ! 15  z-axis in cubic approximant
      data q10d0 /0,0,0,0,1,0/
      data q2d0  /1,1,1,1,0,0/
      data qmd0  /1,0,0,-1,0,0/
      icount=icount+1
      if(icount.eq.1) then
         call trnsp(ctoh)
         call mcpy(ctoh,htoc)
         call matinv(htoc,6,6,b,0,det,ipv,ind,piv)
         call getsys(isys)
         do 41 i=1,6
            call getqi0(q50(1,i),q5p(1,i))
            call norml(q5p(1,i),3)
            call norml(q5p(4,i),3)
 41      continue
         do 42 i=1,10
            call getqi0(q30(1,i),q3p(1,i))
            call norml(q3p(1,i),3)
            call norml(q3p(4,i),3)
 42      continue
         do 43 i=1,15
            call getqi0(q20(1,i),q2p(1,i))
            call norml(q2p(1,i),3)
            call norml(q2p(4,i),3)
 43      continue
         call getqi0(q10d0,q10d)
         call norml(q10d(1),3)
         call norml(q10d(4),3)
         call getqi0(qmd0,qmd)
         call norml(qmd(1),3)
         call norml(qmd(4),3)
         call getqi0(q2d0,q2d)
         call norml(q2d(1),3)
         call norml(q2d(4),3)
         if(isys.eq.1) then
c          Y. Ishii, Phys Rev. B39 (1989) 11862
c          D5 distortion
            call mcpy3(fij1,amij1)
            call mcpy3(fij2,amij2)
            call mcpy3(fij3,amij3)
            call mcpy3(fij1,bmij1)
         else if(isys.eq.2) then
c          T distortion  : eps1
            call mcpy3(fij4,amij1)
c          D3 phason distortion : eps2
            call mcpy3(fij5,amij2)
            call mcpy3(fij6,amij3)
c          D3 phonon distortion : eps4
            call mcpy3(fij5,bmij1)
            call getwmij(amij2,wmij2,ctoh,htoc)
            call getwmij(amij3,wmij3,ctoh,htoc)
            call getwmij(bmij1,wmij4,ctoh,htoc)
         else if(isys.eq.3) then
c          orthorhombic phason strain
            call mcpy3(fij1,amij1)
            call mcpy3(fij2,amij2)
            call mcpy3(fij3,amij3)
         end if
         write(6,*) 'amij1='
         write(6,'(3f9.5)') ((amij1(i,j),j=1,3),i=1,3)
         if(isys.eq.2) then
            write(6,*) 'amij2='
            write(6,'(3f9.5)') ((wmij2(i,j),j=1,3),i=1,3)
            write(6,*) 'amij3='
            write(6,'(3f9.5)') ((wmij3(i,j),j=1,3),i=1,3)
            write(6,*) 'bmij1='
            write(6,'(3f9.5)') ((wmij4(i,j),j=1,3),i=1,3)
         else
            write(6,*) 'amij2='
            write(6,'(3f9.5)') ((amij2(i,j),j=1,3),i=1,3)
            write(6,*) 'amij3='
            write(6,'(3f9.5)') ((amij3(i,j),j=1,3),i=1,3)
            write(6,*) 'bmij1='
            write(6,'(3f9.5)') ((bmij1(i,j),j=1,3),i=1,3)
         end if
      end if
      if(isys.eq.1.or.isys.eq.3) then
         do i=1,6
            de(i,i)=1
            di(i,i)=1
         end do
         do i=4,6
            do j=1,3
               de(i,j)=eps1*amij1(i-3,j)+eps2*amij2(i-3,j)+
     *                 eps3*amij3(i-3,j)
            end do
         end do
         do i=1,3
            do j=1,3
               de(i,j)=de(i,j)+eps4*bmij1(i,j)
            end do
         end do
      else if(isys.eq.2) then
         do i=1,6
            de(i,i)=1
         end do
         do i=4,6
            do j=1,3
               de(i,j)=eps2*amij2(i-3,j)+eps3*amij3(i-3,j)
            end do
         end do
         do i=1,3
            do j=1,3
               de(i,j)=de(i,j)+eps4*bmij1(i,j)
            end do
         end do
         call mtmpl(de,ctoh,ded)
         call mtmpl(htoc,ded,de)
         do i=4,6
            do j=1,3
               de(i,j)=de(i,j)+eps1*amij1(i-3,j)
            end do
         end do         
      end if
      call mcpy(de,di)
      call matinv(di,6,6,b,0,det,ipv,ind,piv)
      write(6,*) 'deformation matrix de'
      do 407 i=1,6
      write(6,'(6f8.4)') (de(i,j),j=1,6)
 407  continue
      write(6,*) 'deformation matrix di'
      do 408 i=1,6
      write(6,'(6f8.4)') (di(i,j),j=1,6)
 408  continue
      return
      end

      subroutine getwmij(amij,wmij,ctoh,htoc)
      real amij(3,3),wmij(3,3),ctoh(6,6),htoc(6,6),tmp(6,6)
      do i=1,3
         do j=1,3
            tmp(i,j)=0
            do k=1,3
               tmp(i,j)=tmp(i,j)+amij(i,k)*ctoh(k,j)
            end do
         end do
      end do
      do i=1,3
         do j=1,3
            wmij(i,j)=0
            do k=1,3
               wmij(i,j)=wmij(i,j)+htoc(i,k)*tmp(k,j)
            end do
         end do
      end do
      return
      end

      subroutine mcpy3(s1,s2)
      real s1(3,3)
      real s2(3,3)
      do 100 i=1,3
         do 110 j=1,3
            s2(i,j)=s1(i,j)
 110     continue
 100  continue
      return
      end

      subroutine norml(x,n)
      real x(*)
c     normalization of x
      absh=vabs(x,n)
      do 10 i=1,n
 10   x(i)=x(i)/absh
      return
      end

      subroutine getqi0(h,q)
      integer h(*)
      real q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 100 i=1,6
           q(i)=0.
           do 101 j=1,6
                q(i)=q(i)+h(j)*dq(j,i)
 101       continue
      if(isys.le.2) then
          if(i.le.3) then
             q(i)=q(i)/al
          else
             q(i)=q(i)/al
          end if
      else
          if(i.le.2) then
             q(i)=q(i)/al
          else if(i.eq.3) then
             q(i)=q(i)/cl
          else if(i.le.5) then
             q(i)=q(i)/al
          end if
      end if          
 100  continue
      return
      end

      subroutine getqi(h,q)
      integer h(*)
      real q(*)
      real qd(6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 100 i=1,6
           qd(i)=0.
           do 101 j=1,6
                qd(i)=qd(i)+h(j)*dq(j,i)
 101       continue
 100  continue
c     phason distortion in reciprocal space
      do 200 i=1,6
           q(i)=0.
           do 201 j=1,6
                q(i)=q(i)+qd(j)*di(j,i)
 201       continue
      if(isys.le.2) then
          if(i.le.3) then
             q(i)=q(i)/al
          else
             q(i)=q(i)/al
          end if
      else
          if(i.le.2) then
             q(i)=q(i)/al
          else if(i.eq.3) then
             q(i)=q(i)/cl
          else if(i.le.5) then
             q(i)=q(i)/al
          end if
      end if          
 200  continue
      return
      end

      subroutine getqe0(h,q)
      real h(*),q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 100 i=1,6
           q(i)=0.
           do 101 j=1,6
                q(i)=q(i)+dr(i,j)*h(j)
 101       continue
      if(isys.le.2) then
          if(i.le.3) then
             q(i)=q(i)*al
          else
             q(i)=q(i)*al
          end if
      else
          if(i.le.2) then
             q(i)=q(i)*al
          else if(i.eq.3) then
             q(i)=q(i)*cl
          else if(i.le.5) then
             q(i)=q(i)*al
          end if
      end if
 100  continue
      return
      end

      subroutine getqe(h,q)
      real h(*),q(*)
      real qd(6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 100 i=1,6
           qd(i)=0.
           do 101 j=1,6
                qd(i)=qd(i)+dr(i,j)*h(j)
 101       continue
 100  continue
      do 200 i=1,6
           q(i)=0.
           do 201 j=1,6
                q(i)=q(i)+de(j,i)*qd(j)
 201       continue
      if(isys.le.2) then
          if(i.le.3) then
             q(i)=q(i)*al
          else
             q(i)=q(i)*al
          end if
      else
          if(i.le.2) then
             q(i)=q(i)*al
          else if(i.eq.3) then
             q(i)=q(i)*cl
          else if(i.le.5) then
             q(i)=q(i)*al
          end if
      end if
 200  continue
      return
      end

      subroutine getxe(q,x)
      real q(*),x(*)
      real qd(6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      call getsys(isys)
      do 200 i=1,6
      qd(i)=q(i)
      if(isys.le.2) then
          if(i.le.3) then
             qd(i)=q(i)/al
          else
             qd(i)=q(i)/al
          end if
      else
          if(i.le.2) then
             qd(i)=q(i)/al
          else if(i.eq.3) then
             qd(i)=q(i)/cl
          else if(i.le.5) then
             qd(i)=q(i)/al
          end if
      end if
 200  continue
      do 100 i=1,6
          x(i)=0.
          do 101 j=1,6
               x(i)=x(i)+dq(i,j)*qd(j)
 101      continue
 100  continue
      return
      end

      subroutine getsys(isysd)
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      isysd=isys
      return
      end            

      subroutine setsys(isysd)
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),isys
      isys=isysd
      return
      end            

      function ivicmp(h,hr,n)
      integer h(*),hr(*)
      ivicmp=0
      do 50 i=1,n
      if(h(i).ne.hr(i)) return
 50   continue
      ivicmp=1
      return
      end

      function ivrcmp(x,y,n,eps)
      real x(*),y(*)
      ivrcmp=0
      do 50 i=1,n
      if(abs(x(i)-y(i)).gt.eps) return
 50   continue
      ivrcmp=1
      return
      end

      function vabs(x,n)
c*****absolute value of n-dimensional vector x
      dimension x(*)
      vabs=0
      do 10 i=1,n
 10   vabs=vabs+x(i)**2
      vabs=sqrt(vabs)
      return
      end

      function vinp(x,y,n)
c*****inner product of n-dimensional vactors, x,y
      dimension x(*),y(*)
      vinp=0
      do 10 i=1,n
 10   vinp=vinp+x(i)*y(i)
      return
      end

      function vdinp(h,x,n)
c*****inner product of n-dimensional integral and real vectors
      integer h(*)
      real x(*)
      vdinp=0
      do 10 i=1,n
 10   vdinp=vdinp+h(i)*x(i)
      return
      end

c*********************************
      subroutine trcs(x,y,n1,n2,n3)
      dimension x(*),y(*),n1(*),n2(*),n3(*)
      real n1,n2,n3
c*****transformation of coordinate systems********************
c     x: coordinates in primary system
c     y: coordinates in plotter system
c        y(1),y(2) are x and y coordinates in plotter system
      y(1)= (n1(1)*x(1)+n1(2)*x(2)+n1(3)*x(3))
      y(2)= (n2(1)*x(1)+n2(2)*x(2)+n2(3)*x(3))
      y(3)= (n3(1)*x(1)+n3(2)*x(2)+n3(3)*x(3))
      return
      end

      subroutine qsort(x,st,ip,nx)
c     quick sort (ascending order of x)
c     nx: the number of data x
c     ip: the initial order
c     st: a work array
      real x(nx)
      integer r,s,st(nx,0:1),ip(nx)
      do 10 i=1,nx
 10   ip(i)=i
      s=1
      st(1,0)=1
      st(1,1)=nx
    1 l=st(s,0)
      r=st(s,1)
      s=s-1
    2 i=l
      j=r
      xt=x((l+r)/2)
    3 if(i.lt.nx) then
         if(x(i).lt.xt) then
            i=i+1
            go to 3
         endif
      endif
    4 if(j.gt.1) then
         if(xt.lt.x(j)) then
            j=j-1
            go to 4
         endif
      endif
      if(i.le.j) then
        temp=x(j)
        x(j)=x(i)
        x(i)=temp
        itemp=ip(j)
        ip(j)=ip(i)
        ip(i)=itemp
        if(i.le.nx.and.j.ge.1) then
           i=i+1
           j=j-1
           go to 3
        end if
      endif
      if(j-l.le.r-i) go to 5
      if(i.lt.r) then
         s=s+1
         st(s,0)=i
         st(s,1)=r
      endif
      r=j
      go to 6
    5 if(l.lt.j) then
         s=s+1
         st(s,0)=l
         st(s,1)=j
      endif
      l=i
    6 if(l.lt.r) go to 2
      if(s.ne.0) go to 1
      return
      end

      subroutine tr5to6(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *         0,1,0,0,-1,0,
     *         0,0,1,0,-1,0,
     *         0,0,0,1,-1,0,
     *         1,1,1,1, 1,0,
     *         0,0,0,0, 0,1/
      do 12 j=1,6
         fi(j)=0.
         if(j.le.5) then
             do 13 k=1,4
                 fi(j)=fi(j)+sd(j,k)*ei(k)
 13          continue
         else if(j.eq.6) then
                 fi(6)=ei(5)
         end if
 12   continue
      return
      end

      subroutine tr6to5e(ei,fi)
      real fi(*),ei(*)
      integer sd(6,6)
      data sd/ 4,-1,-1,-1, 1, 0,
     *        -1, 4,-1,-1, 1, 0,
     *        -1,-1, 4,-1, 1, 0,
     *        -1,-1,-1, 4, 1, 0,
     *        -1,-1,-1,-1, 1, 0,
     *         0, 0, 0, 0, 0, 1/
      do 12 j=1,6
         fi(j)=0.
         if(j.le.4) then
             do 13 k=1,5
             fi(j)=fi(j)+sd(j,k)*ei(k)/5
 13          continue
         else if (j.eq.5) then
             fi(5)=ei(6)
         end if
 12   continue
      return
      end

      subroutine tr6to5i(ei,fi)
      integer fi(*),ei(*)
      integer sd(6,6)
      data sd/ 1,0,0,0,-1,0,
     *         0,1,0,0,-1,0,
     *         0,0,1,0,-1,0,
     *         0,0,0,1,-1,0,
     *         0,0,0,0, 0,1,
     *         0,0,0,0, 0,0/
      do 12 j=1,6
         fi(j)=0.
         do 13 k=1,5
             fi(j)=fi(j)+sd(j,k)*ei(k)
 13      continue
 12   continue
      return
      end

      function ifskip(q)
      real q(*)
c      common /cgetq/ dr(6,6),sds(6,6),dq(6,6),al,cl,isys
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4
      data eps/0.001/
      call getsys(isys)
      ifskip=1
      if(q(3).lt.-eps) return
      if(q(2).lt.-eps) return
      if(isys.eq.1) then
          if(-.3090*q(1)+0.9511*q(2).gt.eps) return
      else if(isys.eq.2) then
          if(-.5878*q(1)+0.8090*q(2).gt.eps) return
      else
          call quit('invalid isys')
      end if
      ifskip=0
      return
      end

