c
c      program pltint
c                   plot fo and fc for quasicrystals
c      (1)  title  (nchar <= 80)
c      (2)  job, iaster, ifori
c                 0: fo/fo   ! fo of two data sets
c                 1: fo/sig
c                -1: fo/sig (log log plot)
c                 2: fo/fc plot
c                -2: fo/fc plot  (log log plot)
c                 3: qi/fofc (only for quasicrystal)
c                 4: qe/fofc (only for quasicrystal)
c                -4: qe/fo   (only for quasicrystal)  ! new
c                    in the 2-fold plane for isys=1,2
c                    in the n-fold plane for dihedral (n=8,10, or 12)
c                 5: qe,qi/fo(fo circle for intensity for quasicrystal)
c                     
c               iaster: =0 all reflections
c                       =1 nonasterisked reflections
c                       =2 asterisked reflections
c                ifori: 1 if input data is F, 2 for I 
c      (3)  isys,idm           (for crystals isys=0 )
c           idm : dimension for modulated structure (idm=3,4,5 or 6)

c           isys,a,(c) (for quasicrystals isys > 0)
c           isys: 0 crystal (including modulated crystals)
c                 1:icosahedral or pentagonal 2:  cubic or trigonal
c                 3:decagonal -3:pentagonal 4:octagonal 5: dodecagonal
c                 (1,2: icosahedral with phason distortion)
c                 (4,5 are not implimented at present)
c           a   : lattice constant
c           c   : lattice constant along 10, 8 fold axis
c                  for decagonal and octagonal
c                               quasicrystals only
c      (4)  ldev,amag,fmag
c           amag : magnification of the figure
c           fmag : magnification of the intensity
c        ****** if job != -4 skip (4') *****
c      (4') nsymo, icent, brv
c           nsymo: number of generators
c           icent: 0 for non-centrosymmetric, 1 for centrosymmetric
c           brv  : 'p', 'i' or 'f' for primitive, body-center, or face-center
c           symmetry operator  (header)
c           x,y,z,t,u,v etc. (symmetry operator in the IT format)
c      (5)  flnm0
c           flnm0 : filename for fo,sig or fo,fc or hkl,fo,sig (job 1 or other)
c      (6)  format for
c              fo,sig or fo, fc  (job=0,-1,1,-2,2)
c              hkl(1)-hkl(5/6),fo, sig or fc (job=3,4,5)

      real lambda(3)
      character(80) title
      character brv
      common /cprjax/ q5p(6,6),q3p(6,10),q2p(6,15),q10d(6),q2d(6),qmd(6)
      integer rr
      common /sym/ rr(6,6,60),tt(6,60),nsymo,icent
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      common /cldev/ ldev,icol1,icol2,icase,amag,fmag

      character(80) flnm0
      data de/36*0./, di/36*0./
      data pi2/6.28318/, eps/0.0001/, eps1/0.001/
c         lambda: wave length for mo,cu,cr
      data lambda/0.71073,1.54184,2.29100/
c     initialize
      call setinp(5)
      call setout(6)
c**** get dr,dq,sds ***
c      read(5,*) job,iaster
      call freadcs('title=',1,title,1)
      call freadis('job=',1,job,1)
      if(job.lt.-2.and.job.gt.5) then
         write(6,*) 'job=',job
         call quit('job should be -2<=job<=5')
      end if
      call freadis('iaster=',1,iaster,0)
      call freadis('ifori=',1,ifori,0)
      write(6,*) 'job=',job,' iaster=',iaster,' ifori=',ifori
      if(ifori.ne.1.and.ifori.ne.2) then
         call quit('ifori should be 1 or 2')
      end if
c     read(5,*) isys,a,c
      call freadis('isys=',1,isys,1)
      if(isys.eq.0) then
         call freadis('idm=',1,idm,0)
         if(job.ge.3) then
            call quit('job >=3 is only for quasicrystals')
         end if
      else if(isys.gt.0) then
         call freadrs('a=',1,al,0)
         if(isys.ge.3) then
            call freadrs('c=',1,cl,0)
         end if
      end if
      if(isys.eq.0) then
          write(6,*) '  isys=',isys,' idm=',idm
      else if(isys.le.2) then
          write(6,*) '  isys=',isys,' a=',al
      else if(isys.ge.3) then
          write(6,*) '  isys=',isys,' a=',al,' c=',cl
      end if
      call setsys(isys)
      if(isys.gt.0) then
         call setd()
      end if
      call freadis('ldev=',1,ldev,1)
      call freadrs('amag=',1,amag,0)
      call freadrs('fmag=',1,fmag,0)
      write(6,*) 'ldev=',ldev,' amag=',amag,' fmag=',fmag
      if(job.eq.-4) then
         call rdsymo(1,brv)
      end if
      if(isys.eq.0) then
         call setidm(idm)      
      else if(isys.le.2) then
         call setidm(6)
      else
         call setidm(5)
      end if

c      read(5,'(a)') flnm0
      call freadc('flnm0=',1,flnm0,1)
      lfn0=istrln(flnm0)
      write(6,'(a)') flnm0(1:lfn0)
      call openfile(4,flnm0,'OLD')
      fmin=0.
      icount=0
      call plots(0.,0.,ldev)
      call fillarea(1)
      call pltfoc(job,title,iaster,ifori)
      call plote('end of plot')
      stop
      end

      subroutine pltfo(istl,icol1,icol2,yw,rc)
      real yw(*)
      if(rc.lt.0.001) return
      if(istl.eq.0.and.icol1.ge.0) then
           call fillcolor(icol1)
           if(icol1.eq.10) then
               call fillarea(0)
               call setcol(0)
           end if
           call circle(0.,360.,rc,yw,0)
           if(icol1.eq.10) then
               call fillarea(1)
               call setcol(10)
           end if
      else if(istl.eq.1.and.icol2.ge.0) then
           call fillcolor(icol2)
           if(icol2.eq.10) then
               call fillarea(0)
               call setcol(0)
           end if
           call circle(0.,360.,rc,yw,0)
           if(icol2.eq.10) then
               call fillarea(1)
               call setcol(10)
           end if
      end if
      return
      end

      function isatel(h,brv)
      integer h(*)
      character brv
      if(brv.eq.'f') then
c        h(j) is all even or all odd
         if(mod(h(1),2).eq.0) then
             isatel=0
         else
             isatel=1
         end if
         return
      else if(brv.eq.'i') then
c        ism is even or odd
         ism=h(1)+h(2)+h(3)+h(4)+h(5)+h(6)
         if(mod(ism,2).eq.0) then
             isatel=0
         else
             isatel=1
         end if
         return             
      else if(brv.eq.'s') then
c        ism is even or odd
         ism=h(1)+h(2)+h(3)+h(4)
         if(mod(ism,5).eq.0) then
             isatel=0
         else
             isatel=1
         end if
         return             
      else
         isatel=0
         return
      end if
      return
      end

      subroutine countn(n,iaster,aster)
      character aster
      if(iaster.eq.0) then
         n=n+1
      else if(iaster.eq.1) then
         if(aster.eq.' ') then
            n=n+1
         end if
      else if(iaster.eq.2) then
         if(aster.eq.'*') then
            n=n+1
         end if
      end if
      return
      end

      function getfomax()
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      getfomax=fomax
      return
      end

      function getfcmax()
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      getfcmax=fcmax
      return
      end

      subroutine initminmax()
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      fomax=0
      fcmax=0
      fomin=0
      fcmin=0
      sgmin=0
      sgmax=0
      qimax=0
      qemax=0
      return
      end

      subroutine calminmax(nref)  ! minimum and maximum values
      include 'nhkl.inc'
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      if(nref.le.0) then
         write(6,*) 'nref=',nref
         call quit('strange nref in calminmax')
      end if
      do n=1,nref
         if(n.eq.1) then
            write(6,*) 'fo(n)',fo(n)
            fomin=fo(n)
            fomax=fo(n)          
            fcmin=abs(fc(n))
            fcmax=abs(fc(n))
            sgmin=sg(n)
            sgmax=sg(n)
            qemax=qe(n)
            qimax=qi(n)
         else
            fomin=min(fo(n),fomin)
            fomax=max(fo(n),fomax)
            fcmin=min(abs(fc(n)),fcmin)
            fcmax=max(abs(fc(n)),fcmax)
            sgmin=min(sg(n),sgmin)
            sgmax=max(sg(n),sgmax)
            qemax=max(qe(n),qemax)
            qimax=max(qi(n),qimax)
         end if
      end do
      call wtminmax()
      return
      end

      subroutine wtminmax()
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      write(6,*) 'fomin=',fomin
      write(6,*) 'fomax=',fomax
      write(6,*) 'fcmin=',fcmin
      write(6,*) 'fcmax=',fcmax
      write(6,*) 'sgmin=',sgmin
      write(6,*) 'sgmax=',sgmax
      write(6,*) 'qemax=',qemax
      write(6,*) 'qimax=',qimax
      return
      end

      subroutine setmgn(mgnd)
      real mgnd
      real mgn
      common /cgetxwyw/ mgn,fmnx,fmxx,fmny,fmxy,qemx,qimx
      mgn=mgnd
      return
      end

      subroutine getdmxy(dmx,dmy,job)
      common /cxymnmx/ xmn,xmx,ymn,ymx
      dmx=xmx-xmn
      dmy=ymx-ymn
      write(6,*) 'xmn,xmx,ymn,ymx,dmx,dmy=',xmn,xmx,ymn,ymx,dmx,dmy
      return
      end

      subroutine setxymnmx(job)  ! set minimum and maximum value
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      common /cxymnmx/ xmn,xmx,ymn,ymx
      real log10
      xmn=0                     ! offset for x-axis
      ymn=0                     ! offset for y-axis
      if(job.eq.0) then
         xmx=fomax
         ymx=fcmax
      else if(job.eq.-1) then
         xmn=log10(fomin)
         xmx=log10(fomax)
         ymn=log10(sgmin)
         ymx=log10(sgmax)
      else if(job.eq.1) then
         xmx=fomax
c        ymn=sgmin
         ymx=sgmax
      else if(job.eq.-2) then
         xmn=log10(min(fomin,fcmin))
         xmx=log10(max(fomax,fcmax))
         ymn=xmn
         ymx=xmx
      else if(job.eq.2) then
         xmx=max(fomax,fcmax)
         ymx=xmx
      else if(job.eq.3) then
         xmx=qimax
         ymx=fomax
      else if(job.eq.4) then
         xmx=qemax
         ymx=fomax
      else if(job.eq.-4) then
         xmx=qemax
         ymx=qemax
      else if(job.eq.5) then
         xmx=qemax
         ymx=qimax
      else
         write(6,*) 'job=',job
         call quit('job invalid')
      end if
      write(6,*) 'xmn,xmx=',xmn,xmx
      return
      end

      subroutine getdltxy(job,dmx,dmy,dltx,dlty)
      dltx=getdlt(dmx)
      dlty=getdlt(dmy)
      return
      end

      function getfmag()
      common /cldev/ ldev,icol1,icol2,icase,amag,fmag
      getfmag=fmag
      return
      end

      subroutine pltfoc(job,title,iaster,ifori)
      character(*) title
      include 'nhkl.inc'
      character aster
      real xw(nhkl),yw(nhkl),zw(nhkl)
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      real qd(6)
      character(60) frm
      common /cldev/ ldev,icol1,icol2,icase,amag,fmag
      common /cxymnmx/ xmn,xmx,ymn,ymx
      real mgn
      common /cgetxwyw/ mgn,fmnx,fmxx,fmny,fmxy,qemx,qimx
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      real xx(2)
      integer getidm
      call fillcolor(0)
      call freadcs('format=',1,frm,1)
      write(6,*) 'format=',frm
      idm=getidm()
      call initfocqs()
      call initminmax()
      nref=0
      n=0
      m=0
      no=0
c     iw=1 ! for write input data
      iw=0 ! just read
 100  continue
      call rdpltinf(job,frm,m,iend,inew,ifori,iw)
      if(iend.eq.1) then
         go to 200
      end if
      if(inew.eq.1) then
         call wtpltinf(job,frm,m)
      end if
      call countn(n,iaster,aster)
      go to 100

 200  continue
      nref=m                    ! number of reflections to be plotted
      call calminmax(nref)      ! calculate minimum and maximum
c     call wtminmax()
      call getsys(isys)
      call setxymnmx(job)   ! set minimum and  maximum values for scale
      call getdmxy(dmx,dmy,job) ! get interval for x and y axis
      call setfmxyq(job)    ! for resetxymx for plot
      call resetxymx(xmx,ymx,job) ! reset xmx ymx for wtbox
c     write(6,*) 'xmx,ymx=',xmx,ymx
      call setmgn(15.0)
      lenx=15
      leny=15
c     write box
      call wtbox(xmn,xmx,ymn,ymx,dmx,dmy,job,title)
      call setnewpen(job)
      if(abs(job).eq.2) then
         call diagline(job)   ! plot a diagonal line
      end if
c     set x and y axis values
      call setxyzw(nref,xw,yw,zw,nmx,job)
c     plot normalized observed intensity
      call fillcolor(icol1)
      write(6,*) 'nmx=',nmx
      if(nmx.gt.nhkl) then
         write(6,*) 'nhkl=',nhkl,' nmx=',nmx
         call quit('nmx > nhkl')
      end if

c      do i=1,nmx
c         write(6,*) 'xw,yw,zw=',xw(i),yw(i),zw(i)
c      end do
      call drwiobs(xw,yw,zw,nmx,job)
      return
      end

      integer function getixax(dmx)
      if(dmx.ge.1) then
         getixax=1
      else
         getixax=0
      end if
      return
      end

      subroutine setnewpen(job)
      if(abs(job).eq.1) then
         call newpen(1)
      else
         call newpen(2)
      end if
      return
      end

      subroutine initfocqs()
      include 'nhkl.inc'
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      call zerov(fo,nhkl)
      call zerov(fc,nhkl)
      call zerov(qe,nhkl)
      call zerov(qi,nhkl)
      call zerov(sg,nhkl)
      return
      end

      subroutine diagline(job)
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      if(job.gt.1) then
          call plot(0.,0.,3)
          call plot(lenx,leny,2)
      else if(job.lt.-1) then
          call plot(0.,0.,3)
          call plot(lenx,leny,2)
      end if
      return
      end

      subroutine drwopencircle(xw,yw,rw,nmx)
      real xw(*),yw(*),rw(*)
      real xx(2)
      do i=1,nmx
         xx(1)=xw(i)
         xx(2)=yw(i)
         rc=rw(i)
         call fillarea(0)
         call fillcolor(0)
         call circle(0.,360.,rc,xx,0)
      end do
      return
      end

      subroutine drwfilledcircle(xw,yw,rw,nmx,icol)
      real xw(*),yw(*),rw(*)
      real xx(2)
      do i=1,nmx
         xx(1)=xw(i)
         xx(2)=yw(i)
         rc=rw(i)
         call fillarea(1)
         call fillcolor(icol)
         call circle(0.,360.,rc,xx,0)
      end do
      return
      end

      subroutine drwfofclabel(he)
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      real xx(2)
      real xw(3),yw(3)
      character(80) title
      x=lenx-3.
      y=leny-2.
      ang=0
      nch=2
      he=0.4
      title='Fo'
c     write(6,*) 'x,y,he=',x,y,he
      call csymbol(x,y,he,title,ang,nch)
      xw(1)=x+0.7
      yw(1)=y+0.2
      xw(2)=0.0
      yw(2)=0.0
      xw(3)=1.0
      yw(3)=1.0
      call line1(xw,yw,1,1,-1,3,he)
      x=lenx-3
      y=leny-3
      ang=0
      nch=2
      title='Fc'
      call csymbol(x,y,he,title,ang,nch)
      xx(1)=x+0.7
      xx(2)=y+0.2
      rc=he/2
      ang0=0.
      ang1=360.
      call circle(ang0,ang1,rc,xx,0)
      return
      end

      subroutine line2(xw,yw,he,nmx)
      real xw(*),yw(*)
      real xx(2)
      do i=1,nmx
         xx(1)=xw(i)/xw(nmx+2)
         xx(2)=yw(i)/yw(nmx+2)
         rc=he/2
         call fillarea(0)
         call fillcolor(0)
         call circle(0.,360.,rc,xx,0)
      end do
      return
      end

      subroutine drwiobs(xw,yw,zw,nmx,job)      
      real xw(*),yw(*),zw(*)
      real xx(2)
c     write(6,*) 'job=',job
      if(job.eq.3.or.job.eq.4) then
         he=0.2   ! symbol size
         call line1(xw,yw,nmx,1,-1,3,he)
         call line2(xw,zw,he,nmx) ! for circle
         call drwfofclabel(he)
      else if(job.eq.-4) then
c     call drwfilledcircle(xw,yw,zw,nmx,10)
         call drwopencircle(xw,yw,zw,nmx)            
      else if(job.eq.5) then
         call drwfilledcircle(xw,yw,zw,nmx,10)
         call drwopencircle(xw,yw,zw,nmx)            
      else
         he=0.2
         call line1(xw,yw,nmx,1,-1,3,he)
      end if
      return
      end

      subroutine wtpltinf(job,frm,m)
      character(*) frm
      include 'nhkl.inc'
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      integer getidm
      idm=getidm()
      if(job.eq.0) then
         write(6,frm) fo(m),fc(m)
      else if(abs(job).eq.1) then
         write(6,frm) fo(m),sg(m)
      else if(abs(job).eq.2) then
         write(6,frm) fo(m),fc(m)
      else if(job.eq.3.or.job.eq.4) then
         write(6,frm) (hd(j,m),j=1,idm),fo(m),fc(m)
      else if(job.eq.5) then
         if(isys.eq.0) then
            write(6,frm) (hd(j,m),j=1,idm),fo(m)
         else if(isys.le.2) then
            write(6,'(6i4,f10.3,2f10.5)')
     *        (hd(j,m),j=1,idm),fo(m),qe(m),qi(m)         
         else
            write(6,'(5i4,f10.3,2f10.5)')
     *        (hd(j,m),j=1,idm),fo(m),qe(m),qi(m)         
         end if
      end if
      return
      end

      subroutine rdpltinf(job,frm,m,iend,inew,ifori,iw)
      character(*) frm
      include 'nhkl.inc'
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      real iob,isg
      integer getidm
      real q(6)
      integer hdm(6)
      idm=getidm()
      iend=0
      inew=0
      if(abs(job).le.2) then
         read(4,frm,end=200) fom,fcm
         if(iw.eq.1) then
            write(6,frm) fom,fcm
         end if
      else
         read(4,frm,end=200) (hdm(j),j=1,idm),fom,fcm
         if(iw.eq.1) then
            write(6,frm) (hdm(j),j=1,idm),fom,fcm
         end if
      end if

      m=m+1
c     write(6,*) 'm=',m
      if(ifori.eq.2) then       ! intensit to F
         fo(m)=sqrt(fom)
         fc(m)=sqrt(fcm)
         isg=fcm
         sg(m)=isg/(2*fo(m))
      else
         fo(m)=fom
         fc(m)=fcm
         isg=fcm
         sg(m)=isg
      end if

      if(abs(job).ge.3) then
         call vcpyi(hdm,hd(1,m),6)
         call getqi0(hdm,q)
         qe(m)=vabs(q(1),3)
         qi(m)=vabs(q(4),idm-3)
      end if

      if(abs(job).eq.1) then
         if(job.eq.-1) then
            if(fo(m).le.0.or.sg(m).le.0) then
               m=m-1
               return
            end if
         end if
      else if(abs(job).eq.2) then
         if(job.eq.-2) then
            if(fo(m).eq.0.or.fc(m).eq.0) then
               m=m-1
               return
            end if
         end if
      end if
      inew=1
      return

 200  iend=1
      return
      end

      subroutine setxyzw(nref,xw,yw,zw,nmx,job)  ! set xw,yw etc.
      real xw(*),yw(*),zw(*)
      real mgn
      common /cgetxwyw/ mgn,fmnx,fmxx,fmny,fmxy,qemx,qimx
      include 'nhkl.inc'
      integer hd(6,nhkl)
      real    fo(nhkl),fc(nhkl),qi(nhkl),qe(nhkl),sg(nhkl)
      common /cfofcsg/ hd,fo,fc,qe,qi,sg
      real q(6)
      integer hr(6)
      integer getnsymo
      data eps /0.00001/
c     write(6,*) 'nsymo=',getnsymo()
c     stop
c      write(6,*) '******nref=',nref
      nmx=0
      fomax=getfomax()
c      write(6,*) 'fomax=',fomax
      rscl=0.025
      do n=1,nref
         fcn=abs(fc(n))
         fon=abs(fo(n))
         sgn=abs(sg(n))
         if(job.eq.0) then
            nmx=nmx+1
            xw(nmx)=fon*mgn/fmxx
            yw(nmx)=fcn*mgn/fmxy
            zw(nmx)=0
         else if(job.eq.1) then
            nmx=nmx+1
            xw(nmx)=fon*mgn/fmxx
            yw(nmx)=sgn*mgn/fmxy
            zw(nmx)=0
c     write(6,*) 'xw,yw,fon,sgn=',xw(nmx),yw(nmx),fon,sgn
         else if(job.eq.-1) then
            xwd=(log10(fon)-log10(fmnx))/(log10(fmxx)-log10(fmnx))
            ywd=(log10(sgn)-log10(fmny))/(log10(fmxy)-log10(fmny))
            nmx=nmx+1
            xw(nmx)=xwd*mgn
            yw(nmx)=ywd*mgn
            zw(nmx)=0
c     write(6,*) 'xw,yw,fon,sgn=',xw(nmx),yw(nmx),fon,sgn
         else if(job.eq.2) then
            nmx=nmx+1
            xw(nmx)=fcn*mgn/fmxx
            yw(nmx)=fon*mgn/fmxx
            zw(nmx)=0
         else if(job.eq.-2) then
            xwd=(log10(fcn)-log10(fmnx))/(log10(fmxx)-log10(fmnx))
            ywd=(log10(fon)-log10(fmnx))/(log10(fmxx)-log10(fmnx))
            nmx=nmx+1
            xw(nmx)=xwd*mgn
            yw(nmx)=ywd*mgn
            zw(nmx)=0
         else if(job.eq.3) then
            qin=qi(n)
            nmx=nmx+1
            xw(nmx)=qin*mgn/qimx
            yw(nmx)=fon*mgn/fmxx
            zw(nmx)=fcn*mgn/fmxx
         else if(job.eq.4) then
            qen=qe(n)
            nmx=nmx+1
            xw(nmx)=qen*mgn/qemx
            yw(nmx)=fon*mgn/fmxx
            zw(nmx)=fcn*mgn/fmxx
         else if(job.eq.-4) then
            nsymo=getnsymo()
            do i=1,nsymo
               do ic=0,1
                  call gethr0(hd(1,n),hr,i,ic)
                  call getqi0(hr,q)
                  if(q(1).ge.0.and.q(2).ge.0.and.
     *                 q(3).gt.-eps.and.q(3).lt.eps) then
                     nmx=nmx+1
                     xw(nmx)=q(1)*mgn/qemx
                     yw(nmx)=q(2)*mgn/qemx
                     zw(nmx)=fon*mgn/fmxx*getfmag()
                  end if
               end do
            end do
         else if(job.eq.5) then
            qen=qe(n)
            qin=qi(n)
            nmx=nmx+1
            xw(nmx)=qen*mgn/qemx
            yw(nmx)=qin*mgn/qimx
            zw(nmx)=fon*mgn/abs(fomax)*rscl
         end if
c         write(6,*) 'xw,yw,zw=',xw(nmx),yw(nmx),zw(nmx)
      end do
c      write(6,*) '****nmx=',nmx
      xw(nmx+1)=0.
      yw(nmx+1)=0.
      zw(nmx+1)=0.
c      if(job.ne.-2) then
c      if(job.gt.0) then
      if(job.ge.0) then
         xw(nmx+2)=1.1
         yw(nmx+2)=1.1
         zw(nmx+2)=1.1
      else
         xw(nmx+2)=1.05
         yw(nmx+2)=1.05
         zw(nmx+2)=1.05
      end if
      return
      end

      function getdlt(dmx)  ! interval of scale
      if(dmx.ge.1000000) then
         getdlt=200000
      else if(dmx.ge.500000.and.dmx.lt.1000000) then
         getdlt=100000
      else if(dmx.ge.200000.and.dmx.lt.500000) then
         getdlt=50000
      else if(dmx.ge.100000.and.dmx.lt.200000) then
         getdlt=25000
      else if(dmx.ge.50000.and.dmx.lt.100000) then
         getdlt=10000
      else if(dmx.ge.20000.and.dmx.lt.50000) then
         getdlt=5000
      else if(dmx.ge.10000.and.dmx.lt.20000) then
         getdlt=2500
      else if(dmx.ge.5000.and.dmx.lt.10000) then
         getdlt=1000
      else if(dmx.ge.2000.and.dmx.lt.5000) then
         getdlt=500
      else if(dmx.ge.1000.and.dmx.lt.2000) then
         getdlt=250
      else if(dmx.ge.500.and.dmx.lt.1000) then
         getdlt=100
      else if(dmx.ge.200.and.dmx.lt.500) then
         getdlt=50
      else if(dmx.ge.100.and.dmx.lt.200) then
         getdlt=25
      else if(dmx.ge.50.and.dmx.lt.100) then
         getdlt=10
      else if(dmx.ge.20.and.dmx.lt.50) then
         getdlt=5
      else if(dmx.ge.10.and.dmx.lt.20) then
         getdlt=2
      else if(dmx.ge.5.and.dmx.lt.10) then
         getdlt=1
      else if(dmx.ge.2.and.dmx.lt.5) then
         getdlt=0.5
      else if(dmx.ge.1.and.dmx.lt.2) then
         getdlt=0.25
      else if(dmx.ge.0.5.and.dmx.lt.1) then
         getdlt=0.1
      else if(dmx.ge.0.2.and.dmx.lt.0.5) then
         getdlt=0.05
      else if(dmx.ge.0.1.and.dmx.lt.0.2) then
         getlogdlt=0.025
      else if(dmx.ge.0.05.and.dmx.lt.0.1) then
         getlogdlt=0.01
      else if(dmx.lt.0.05) then
         getlogdlt=0.005
      end if
      return
      end

      subroutine setfmxyq(job)
      common /cminmax/ fomin,fomax,fcmin,fcmax,sgmin,sgmax,qemax,qimax
      real mgn
      common /cgetxwyw/ mgn,fmnx,fmxx,fmny,fmxy,qemx,qimx
      if(job.eq.0) then
         fmxx=fomax
         fmnx=fomin
         fmxy=fcmax
         fmny=fcmin
         qemx=qemax
         qimx=qimax
      else if(abs(job).gt.1) then
         fmxx=max(fomax,fcmax)
         fmnx=min(fomin,fcmin)
         fmxy=fmxx
         fmny=fmnx
         qemx=qemax
         qimx=qimax
      else
         fmxx=fomax
         fmnx=fomin
         fmxy=sgmax
         fmny=sgmin
         qemx=qemax
         qimx=qimax
      end if
      return
      end

      subroutine resetxymx(xmx,ymx,job)
      common /cgetxwyw/ mgn,fmnx,fmxx,fmny,fmxy,qemx,qimx
c      if(abs(job).gt.1) then
      if(job.eq.0) then
         fmx=fmxx
         fmy=fmxy
      else if(abs(job).gt.1) then
         fmx=fmxx
         fmy=fmxx
      else
         fmx=fmxx
         fmy=fmxy
      end if
      if(job.eq.0) then
         xmx=fmx*1.1
         ymx=fmy*1.1
      else if(job.eq.1) then
         xmx=fmx*1.1
         ymx=fmy*1.1
      else if(job.eq.-1) then
         xmx=log10(fmx)*1.05
         ymx=log10(fmy)*1.05
      else if(job.eq.2) then
         xmx=fmx*1.1
         ymx=xmx
      else if(job.eq.-2) then
         xmx=log10(fmx)*1.05
         ymx=xmx
      else if(job.eq.3) then
         xmx=qimx*1.1
         ymx=fmx*1.1
      else if(job.eq.4) then
         xmx=qemx*1.1
         ymx=fmx*1.1
      else if(job.eq.5) then
         xmx=qemx*1.1
         ymx=qimx*1.1
      end if
      write(6,*) '****xmx,ymx=',xmx,ymx
      return
      end

      function getdx(dltx,lenx,xmn,xmx)
      real dltx,lenx,xmn,xmx
c     write(6,*) 'dltx,lenx,xmn,xmx in getdx=',dltx,lenx,xmn,xmx
      getdx=dltx*lenx/(xmx-xmn)   ! length for dltx in normalized coord. sys.
c     write(6,*) 'getdx=',getdx
      return
      end

      subroutine getxmn0(xmn0,eps1,eps2,xmn,ixax)  ! round xmin
      data eps /0.0001/
      if(ixax.eq.1) then
         if(xmn.lt.0) then
            xmn0=xmn+eps
            eps1=-0.0001
            eps2=-0.001
         else
            xmn0=xmn-eps
            eps1=0.0001
            eps2=0.001
         end if
      else
         xmn0=xmn
         eps1=0
         eps2=0
      end if
      return
      end

      function getx0(dltx,xmn0,lenx,xmx,xmn)  ! length to be cut
      real dltx,xmn0,lenx,xmx,xmn
      getx0=(dltx-mod(xmn0,dltx))*lenx/(xmx-xmn) ! length to be cut 
      return
      end

      function getxx0(xmn0,dltx,it,lenx,xmx,xmn)
      real xmn0,dltx,lenx,xmx,xmn
      getxx0=(-mod(xmn0,dltx*it))*lenx/(xmx-xmn) ! offset
      return
      end

      integer function getnxs(leng,eps,dx,it)
      real leng,eps,dx
      integer it
      nxs=(leng+eps)/(dx*it)+1
      if(nxs.lt.2) then
         write(6,*) 'nxs=',nxs,'leng,dx=',leng,dx,' it=',it
         call quit('nxs too small')
      end if
      getnxs=nxs
      return
      end

      subroutine wtbox(xmn,xmx,ymn,ymx,dmx,dmy,job,title)
c     xmn xmx : minimum and maximum value for x-axis
c     ymn ymx : minimum and maximum value for y-axis
c     dmx  : length of the interval for x-axis
c     dmy  : length of the interval for y-axis
      character(*) title
      character(80) titled
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      character(30) scale
      real leng
      integer getnxs,getixax
      data pi/3.141592/
      data eps/0.0001/
      call getdltxy(job,dmx,dmy,dltx,dlty) ! length for interval
c     dltx : interval for x axis scale
c     dlty : interval for y axis scale
      ixax=getixax(dltx)
      iyay=getixax(dlty)
      write(6,*) 'dmx,dmy,dltx,dlty=',dmx,dmy,dltx,dlty
      write(6,*) 'ixax,iyay=',ixax,iyay
      call setfont('/Helvetica-Bold-Scand')
      fact=pi/180
      xbtm=0.
      ybtm=0.
      x1=xbtm-1.6
      y1=ybtm-1.6
      x2=lenx+0.6
      y2=leny+2.8
      x0=0.1
      y0=0.1
      x3=x1/29.7+x0
      y3=y1/29.7+y0
      x4=x2/29.7+x0
      y4=y2/29.7+y0
      call window(x1,y1,x2,y2)
      call viewpt(x3,y3,x4,y4)
c     write box
      call draw(xbtm,ybtm,lenx,ybtm)
      call draw(lenx,ybtm,lenx,leny)
      call draw(lenx,leny,xbtm,leny)
      call draw(xbtm,leny,xbtm,ybtm)
c     write(6,*) '****after call drw'
c     return

c     write x axis
      dx=getdx(dltx,lenx,xmn,xmx)
      it=1
      ang=0.
c     iside=1
      iside=-1
      ixy=1
      call drwaxis(ixy,ang,dx,iside,dltx,xmx,xmn,ixax,lenx,leny,
     *     ybtm,it,job)

c     write y axis
      dy=getdx(dlty,leny,ymn,ymx)
      it=1
      ang=90.
      iside=1
      ixy=2

      call drwaxis(ixy,ang,dy,iside,dlty,ymx,ymn,iyay,leny,lenx,
     *     xbtm,it,job)

c     write title from input data
      dyt=0.2
      chs=0.4
      ang=0.
      istr=istrln(title)
      call symbol(xbtm,leny+dyt,chs,title,ang,istr)
      return
      end

      subroutine drwaxis(ixy,ang,dx,iside,dltx,xmx,xmn,
     *     ixax,lenx,leny,ybtm,it,job)
      real lenx,leny
      real leng
      character(80) titled
      integer getnxs
c     dltx : interval of the scale      
      call getxmn0(xmn0,eps1,eps2,xmn,ixax)
      x0=getx0(dltx,xmn0,lenx,xmx,xmn) ! length to be cut
      x1=mod(xmn0,dltx)         ! fractional part of offset
      ix=xmn0/dltx              ! offset 
      leng=lenx-x0+dx           ! cut length corresponding to xmn0
      xx0=getxx0(xmn0,dltx,it,lenx,xmx,xmn)
c     write(6,*) '******x0,x1,ix,xx0=',x0,x1,ix,xx0
      nxs=getnxs(leng,eps,dx,it)
      iq0=(int((xmn0)/(dltx*it)))*dltx*it+eps1  ! first digit in scale 
      iq1=(int((xmn0)/(dltx*it)))+eps1
c     write(6,*) '******iq0,iq1=',iq0,iq1
      call drwscale(ixy,x1,iq0,iq1,xx0,dx,dltx,nxs,it,ixax,ang,titled)
      if(ixy.eq.1) then
c     call axis(x0,ybtm,leng,dx,ang,iside,0.1)
c     call axis(x0,leny,leng,dx,ang,iside,0.1)
         call axis(xx0,ybtm,leng,dx*it,ang,iside,0.15)
         call axis(xx0,leny,leng,dx*it,ang,iside,0.15)
         call getxtitle(titled,xx,job,nch)
         call csymbol(xx,ybtm-1.4,0.4,titled,ang,nch)
      else
c     call axis(ybtm,x0,leng,dx,ang,iside,0.1)
c     call axis(leny,x0,leng,dx,ang,iside,0.1)
         call axis(ybtm,xx0,leng,dx*it,ang,iside,0.15)
         call axis(leny,xx0,leng,dx*it,ang,iside,0.15)
         call getytitle(titled,yy,job,nch)
         call csymbol(ybtm-1.0,xx,0.4,titled,ang,nch)
      end if
      return
      end

      real function getqx(x0,i,iq1,dltx,it)
      getqx=iq1*dltx+dltx*it*i
      return
      end

      integer function getiqx(x0,i,iq0,iq1,dltx,it)
      iq0d=iq0+dltx*it*i
      if(iq0d.ge.0) then
         iqx=iq0d+0.001
      else
         iqx=iq0d-0.001
      end if
      getiqx=iqx
      return
      end

      function getxx(i,xx0,dx,it,ixax)
      if(ixax.eq.1) then
         xx1=xx0+dx*it*i
         xx=xx1
      else if(ixax.eq.0) then
         xx1=xx0+dx*it*i
         xx=xx1
      end if
      getxx=xx
      return
      end

      subroutine drwscale(ixory,y0,jq0,jq1,yy0,dy,dlty,nys,it,iyay,ang,
     *     titled)
      character(*) titled
      character(30) scale
      real mag
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      integer getiqx
      chg=0.4                   ! character size (height)
      if(ixory.eq.1) then       ! xaxis
         xgap=9
         ygap=0.7
      else                      ! yaxis
         xgap=0.2
         ygap=0
      end if
      call getmag(dlty,mag,titled)
c     write(6,*) 'nys=',nys
      write(6,*) 'jq0,jq1,dlty,nys=',jq0,jq1,dlty,nys
      do i=0,nys
         if(iyay.eq.1) then     ! integer scale
            iqy=getiqx(yy0,i,jq0,jq1,dlty,it)
            yy=getxx(i,yy0,dy,it,iyay) ! offset
            call getscale1(iqy,scale,nch,mag)
         else if(iyay.eq.0) then ! real number scale
            qy=getqx(yy0,i,jq1,dlty,it)
            yy=getxx(i,yy0,dy,it,iyay) ! offset
            call getscale2(qy,dlty,scale,nch,mag)
         else
            write(6,*) 'iyay',iyay
            call quit('iyay invalid')
         end if
         if(yy.gt.0.0.and.yy.lt.leny) then
            if(ixory.eq.1) then
               call csymbol(yy,ybtm-ygap,chg,scale,ang,nch)
            else if(ixory.eq.2) then
               call csymbol(xbtm-xgap,yy,chg,scale,ang,nch)
            end if
         end if
      end do
      return
      end

      subroutine getmag(dltx,mag,titled)
      real mag
      character(*) titled  
      character(20) str,chr
      real log10
      data eps / 0.00001/
      titled=''
      mag=1
      iexp=log10(dltx)
      write(6,*) 'iexp=',iexp
      if(iexp.gt.2) then
         mag=10**iexp*(1+eps)
         magw=10**(iexp)
         write(chr,'(i1)') iexp+1
         str='(a,i'//chr(1:1)//',a)'
         write(titled,fmt=str) ' (x',magw,')'
      else if(iexp.le.-1) then
         magw=10**(-iexp)
         mag=1.0/magw
         write(chr,'(i1)') -iexp+1
         str='(a,i'//chr(1:1)//',a)'
         write(titled,fmt=str) ' (x 1/',magw,')'
      end if
      write(6,*) 'mag,titled=',mag,titled
      return
      end

      subroutine getscale1(iqx,scale,nch,mag)  ! integer value scale
      character(*) scale
      real mag
      integer imag
      imag=mag
      iqxm=iqx/imag
      if(iqxm.lt.-10) then
         write(scale,'(i3)') iqxm
         nch=3
      else if(iqxm.lt.0) then
         write(scale,'(i2)') iqxm
         nch=2
      else if(iqxm.lt.10) then
         write(scale,'(i1)') iqxm
         nch=1
      else if(iqxm.lt.100) then
         write(scale,'(i2)') iqxm
         nch=2
      else if(iqxm.lt.1000) then
         write(scale,'(i3)') iqxm
         nch=3
      else if(iqxm.lt.10000) then
         write(scale,'(i4)') iqxm
         nch=4
      end if
c      write(6,*) 'scale1=',scale,' nch=',nch
c      write(6,*) 'iqx=',iqx
      return
      end

      subroutine getscale2(q,dlt,scale,nch,mag)  ! real value
      character(*) scale
      character ch1,ch2
      character ch0
      character(15) frmt
      real mag
      real log10
      data eps /0.0001/
c     write(6,*) 'q,mag=',q,mag
      qd=q/mag
      dltd=dlt/mag
      i1=log10(abs(qd)+eps)
      if(i1.eq.0) then
         i1=1   ! for 0.xx
      else if(i1.lt.0) then
         i1=2   ! for -0.xx
      end if
      if(abs(dltd-0.1).le.eps) then
         i2=1
      else if(abs(dltd-0.25).le.eps) then
         i2=2
      else if(abs(dltd-0.5).le.eps) then
         i2=1
      else
         i2=0
      end if
      i3=i1+i2+2
c     write(6,*) 'qd,dlt=',qd,dltd
c     write(6,*) 'i1,i2,i3=',i1,i2,i3
      if(i3.lt.10) then
         write(ch1,'(i1)') i3
      else if(i3.lt.100) then
         write(ch1,'(i2)') i3
      end if
      write(ch2,'(i1)') i2
      frmt='(f'//ch1//'.'//ch2//')'
c     write(6,*) 'frmt=',frmt
      nch=i3
      if(qd.ge.0) then
         write(scale,frmt) qd+0.001
      else
         write(scale,frmt) qd-0.001
      end if
c      write(6,*) 'scale2=',scale,' nch=',nch
c      write(6,*) 'qd=',qd
      return
      end

      subroutine getxtitle(titled,xx,job,nch)
      character(*) titled
      character(15) title
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      if(job.eq.0) then
         title='Fo'
         xx=lenx/2
      else if(job.eq.1) then
         title='Fo'
         xx=lenx/2
      else if(job.eq.-1) then
         title='log(Fo)'
         xx=lenx/2
      else if(job.eq.2) then
         title='Fc'
         xx=lenx/2
      else if(job.eq.-2) then
         title='log(Fc)'
         xx=lenx/2
      else if(job.eq.3) then
         title='Qi'
         xx=lenx/2
      else if(job.eq.4) then
         title='Qe'
         xx=lenx/2
      else if(job.eq.-4) then
         title='Q(1)'
         xx=lenx/2
      else if(job.eq.5) then
         title='Qe'
         xx=lenx/2
      end if
      lstr1=istrln(title)
      lstr2=istrln(titled)
      titled=title(1:lstr1)//titled(1:lstr2)
      nch=istrln(titled)
      return
      end

      subroutine getytitle(titled,yy,job,nch)
      character(*) titled
      character(15) title
      real lenx,leny
      common /clenxy/ lenx,leny,xbtm,ybtm
      if(job.eq.0) then
         title='Fo'
         yy=leny/2
      else if(job.eq.1) then
         title='sig(Fo)'
         yy=leny/2
      else if(job.eq.-1) then
         title='log(sig(Fo))'
         yy=leny/2
      else if(job.eq.2) then
         title='Fo'
         yy=leny/2
      else if(job.eq.-2) then
         title='log(Fo)'
         yy=leny/2
      else if(job.eq.3.or.job.eq.4) then
         title='Fo & Fc'
         yy=leny/2
      else if(job.eq.-4) then
         title='Q(2)'
         yy=leny/2
      else if(job.eq.5) then
         title='Qi'
         yy=leny/2
      end if
      lstr1=istrln(title)
      lstr2=istrln(titled)
      titled=title(1:lstr1)//titled(1:lstr2)
      nch=istrln(titled)
      return
      end

      subroutine getqi0(h,q)
      integer h(*)
      real q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      call getsys(isys)
      do i=1,6
         q(i)=0.
         do j=1,6
            q(i)=q(i)+h(j)*dq(j,i)
         end do
         if(isys.le.2) then
            if(i.le.3) then
               q(i)=q(i)/al
            end if
         else
            if(i.le.2) then
               q(i)=q(i)/al
            else if(i.eq.3) then
               q(i)=q(i)/cl
            else if(i.le.5) then
               q(i)=q(i)
            else
               q(i)=q(i)
            end if
         end if          
      end do
      return
      end

      subroutine getqi(h,q)
      integer h(*)
      real q(*)
      real qd(6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      call getsys(isys)
      do i=1,6
         qd(i)=0.
         do j=1,6
            qd(i)=qd(i)+h(j)*dq(j,i)
         end do
         if(isys.le.2) then
            qd(i)=qd(i)/al
         else
            if(i.le.2) then
               qd(i)=qd(i)/al
            else if(i.eq.3) then
               qd(i)=qd(i)/cl
            else if(i.le.5) then
               qd(i)=qd(i)/al
            else
               qd(i)=qd(i)/cl
            end if
         end if          
      end do
c     phason distortion in reciprocal space
      do i=1,6
         q(i)=0.
         do j=1,6
            q(i)=q(i)+qd(j)*di(j,i)
         end do
      end do
      return
      end

      subroutine getqe0(h,q)
      real h(*),q(*)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      call getsys(isys)
      do i=1,6
         q(i)=0.
         do j=1,6
            q(i)=q(i)+dr(i,j)*h(j)
         end do
         if(isys.le.2) then
            q(i)=q(i)*al
         else
            if(i.le.2) then
               q(i)=q(i)*al
            else if(i.eq.3) then
               q(i)=q(i)*cl
            else if(i.le.5) then 
               q(i)=q(i)*al
            else
               q(i)=q(i)*cl
            end if
         end if
      end do
      return
      end

      subroutine getqe(h,q)
      real h(*),q(*)
      real qd(6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      call getsys(isys)
      do i=1,6
         qd(i)=0.
         do  j=1,6
            qd(i)=qd(i)+dr(i,j)*h(j)
         end do
         if(isys.le.2) then
            qd(i)=qd(i)*al
         else
            if(i.le.2) then
               qd(i)=qd(i)*al
            else if(i.eq.3) then
               qd(i)=qd(i)*cl
            else if(i.le.5) then 
               qd(i)=qd(i)*al
            else
               qd(i)=qd(i)*cl
            end if
         end if
      end do
c     phason distortion in real space
      do i=1,6
         q(i)=0.
         do j=1,6
            q(i)=q(i)+de(i,j)*qd(j)
         end do
      end do
      return
      end

      subroutine quit(str)
      character(*) str
      write(6,*) str
      close(1)
      close(2)
      stop
      end 

      integer function check(ih,ik,h,k)
      integer ih,ik
      real h,k
      integer jh,jk
      jh=h*10
      jk=k*10
      if(h*10-jh.gt.0.5) then
          jh=jh+1
      else if(h*10-jh.lt.-.5) then
          jh=jh-1
      end if
      if(k*10-jk.gt.0.5) then
          jk=jk+1
      else if(k*10-jk.lt.-.5) then
          jk=jk-1
      end if
      if(ih.eq.jh.and.ik.eq.jk) then
          check=1
      else
          check=0
      end if
      return
      end

      function ifskip(q)
      real q(*)
      data eps/0.001/
      ifskip=1
      if(q(3).lt.-eps) return
      if(q(2).lt.-eps) return
      if(-.3090*q(1)+0.9511*q(2).gt.eps) return
      ifskip=0
      return
      end

      subroutine vcpy(dd,de)
      real dd(*),de(*)
      do 10 j=1,6
      de(j)=dd(j)
 10   continue
      return
      end

      subroutine vicpy(dd,de)
      integer dd(*),de(*)
      do 10 j=1,6
      de(j)=dd(j)
 10   continue
      return
      end

      subroutine getdq(dqd)
      real dqd(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      call mcpyr6(dq,dqd)
      return
      end

      subroutine setomtr(phi,theta,psi,omtrx)
      real omtrx(3,3)
      omtrx(1,1)= cos(psi)*cos(phi)-cos(theta)*sin(phi)*sin(psi)
      omtrx(1,2)= cos(psi)*sin(phi)+cos(theta)*cos(phi)*sin(psi)
      omtrx(1,3)= sin(psi)*sin(theta)
      omtrx(2,1)=-sin(psi)*cos(phi)-cos(theta)*sin(phi)*cos(psi)
      omtrx(2,2)=-sin(psi)*sin(phi)+cos(theta)*cos(phi)*cos(psi)
      omtrx(2,3)= cos(psi)*sin(theta)
      omtrx(3,1)= sin(theta)*sin(phi)
      omtrx(3,2)=-sin(theta)*cos(phi)
      omtrx(3,3)= cos(theta)
      return
      end

      subroutine setd()
      common /cgethk/ d0(6,6),d1(6,6),d2(6,6),d3(6,6),d4(6,6)
      common /cgetq/ dr(6,6),dq(6,6),de(6,6),di(6,6),
     *          al,cl,eps1,eps2,eps3,eps4,eps5,eps6
      real  piv(6),b(6)
      integer ipv(6),ind(6,2)
      save icount
      parameter (sr=0.894427191,ct=0.447213596)
      parameter (su=-sr,cu=-ct)
      parameter (c1=0.309016994,c2=-0.809016994)
      parameter (c3=-c2,c4=-c1)
      parameter (s1=0.951056516,s2=0.587785252)
      parameter (s3=-s2,s4=-s1)
      parameter (c1sr=c1*sr,c2sr=c2*sr,c3sr=c3*sr,c4sr=c4*sr)
      parameter (s1sr=s1*sr,s2sr=s2*sr,s3sr=s3*sr,s4sr=s4*sr)
      parameter (tau=1.618034,tav=-tau)
      parameter (sqr2=1.41421, sqr3=1.73205, tqr3=-sqr3)
      parameter (t1=0.70710678, t2=-0.70710678)
      parameter (u1=0.8660254, u2=-u1) 
      data  d0/ 0.,   0.,   1., 0.,   0.,   1.,
     *          c2sr, s2sr, ct, c4sr, s1sr, cu,
     *          c2sr, s3sr, ct, c4sr, s4sr, cu,
     *          c1sr, s4sr, ct, c3sr, s2sr, cu,
     *          sr,   0.,   ct, su,   0.,   cu,
     *          c1sr, s1sr, ct, c3sr, s3sr, cu/
c     for cubic and trigonal
      data  d1/ 1., tau,  0.,  tau, -1., 0.,
     *          tau, 0.,  1.,  -1,  0.,  tau,
     *          tau, 0., -1.,  -1., 0.,  tav,
     *          0.,  1., tav,   0., tau, 1.,
     *          -1., tau, 0.,  tav, -1., 0.,
     *          0.,  1., tau,   0., tau, -1./
c      for decagonal  (******reciprocal lattice*******)
      data d2/  c1, s1, 0., c2, s2, 0.,
     *          c2, s2, 0., c1, s4, 0.,
     *          c2, s3, 0., c1, s1, 0.,
     *          c1, s4, 0., c2, s3, 0.,
     *          0., 0., 1., 0., 0., 0.,
     *          0., 0., 0., 0., 0., 1./       
c     for octagonal
      data d3/  1., 0., 0., 1., 0., 0.,
     *          t1, t1, 0., t2, t1, 0.,
     *          0., 1., 0., 0.,-1., 0.,
     *          t2, t1, 0., t1, t1, 0.,
     *          0., 0., 1., 0., 0., 0.,
     *          0., 0., 0., 0., 0., 1./
c     for dodecagonal
      data d4/  1., 0., 0., 1., 0., 0.,
     *          u1, 0.5, 0., u2, 0.5, 0.,
     *          0.5, u1, 0., 0.5, u2, 0.,
     *          0., 1.,  0., 0., 1.,  0.,
     *          0., 0.,  1., 0., 0.,  0.,
     *          0., 0.,  0., 0., 0.,  1./
      data icount /0/
      call getsys(isys)
      if(isys.eq.0) then
         call quit('isys=0 in setd')
      end if
      icount=icount+1
      if(icount.eq.1) then
c     initialize
         do i=1,6
            do j=1,6
               d0(i,j)=d0(i,j)/2
            end do
         end do
         do i=1,6
            do j=1,6
               d1(i,j)=d1(i,j)/sqrt(1+tau**2)/2
            end do
         end do
         do i=1,6
            do j=1,4
               d2(i,j)=d2(i,j)/sqrt(5.)
            end do
         end do
         do i=1,6
            do j=1,4
               d3(i,j)=d3(i,j)/2
            end do
         end do
         do i=1,6
            do j=1,4
               d4(i,j)=d4(i,j)/sqrt(2.)
            end do
         end do
         call setinp(5)
         call setout(6)
         call trnspr6(d0)
         call trnspr6(d1)
         call trnspr6(d2)
         call trnspr6(d3)
         call trnspr6(d4)
      end if

      if(isys.eq.1) then
         call wtmtrxrf('d0',d0,6,'(6f10.5)')
         call mcpyr6(d0,dq)
         call wtmtrxrf('dq',dq,6,'(6f10.5)')
      else if(isys.eq.2) then
         call wtmtrxrf('d1',d1,6,'(6f10.5)')
         call mcpyr6(d1,dq)
         call wtmtrxrf('dq',dq,6,'(6f10.5)')
      else if(isys.eq.3.or.isys.eq.-3) then
         call mcpyr6(d2,dq)
      else if(isys.eq.4) then
         call mcpyr6(d3,dq)
      else if(isys.eq.5) then
         call mcpyr6(d4,dq)
      else
         write(6,*) 'isys=',isys
         call quit('wrong isys')
      end if
      call wtmtrxrf('dq',dq,6,'(6f10.5)')
      call mcpyr6(dq,dr)
      call wtmtrxrf('dr before matinv',dr,6,'(6f10.5)')
      call matinv(dr,6,6,b,0,det,ipv,ind,piv)
      call wtmtrxrf('dr after matinv',dr,6,'(6f10.5)')
      return
      end

      function vinp(x,y,n)
c*****inner product of n-dimensional vactors, x,y
      dimension x(*),y(*)
      vinp=0
      do i=1,n
         vinp=vinp+x(i)*y(i)
      end do
      return
      end

      subroutine openfile(iun,flnm,st)
      integer iun
      character(*) flnm,st
      lflnm=istrln(flnm)
      lst=istrln(st)
      open (unit=iun,file=flnm(1:lflnm),status=st(1:lst),err=910)
      go to 810
 910  write(6,*) 'can''t open ',flnm(1:lflnm)
      call exit(1)
 810  continue
      return
      end

      subroutine getjsatel(jsateld)
      common /cjsatel/jsatel
      jsateld=jsatel
      return
      end

      subroutine setjsatel(jsateld)
      common /cjsatel/ jsatel
      jsatel=jsateld
      return
      end            

      subroutine getsys(isysd)
      common /cisys/ isys
      isysd=isys
      return
      end

      subroutine setsys(isysd)
      common /cisys/ isys
      isys=isysd
      return
      end            

c      integer function getidm()
c      common /cidm/idm
c      getidm=idm
c      return
c      end

c      subroutine setidm(idmd)
c      common /cidm/idm
c      idm=idmd
c      return
c      end

      integer function getish(n) ! dymmy subroutine for using symmetry.f
      getish=0
      return
      end

      subroutine gettrc(ltype,trc,ntrc) ! dymmy subroutine for using symmetry.f
      real trc(6)
      return
      end

      integer function getisyd(n) ! dymmy subroutine for using symmetry.f
      getisyd=0
      return
      end
