********************************************************
c       Polyhd-to-VRML
c       (C) Yamamoto 1999
c       usage: xyz2wrl xyz_file wrl_file mode
c       xyz_file : input x,y,z coordinates file
c       x,y,z coordinates of vertices of a triangle in a line
c       wrl_file: output vrml file
c       mode 0 non-smoothing 1 smoothing
********************************************************
        include 'impr8.inc'
	parameter (nmax=200000)
	real*8 x(nmax),y(nmax),z(nmax)
        integer icor(nmax),ind(nmax)
	character*80 flnm1,flnm2
        character cmode
	iarg=iargc()

	if(iarg.ne.3) then
	   write(6,*) 'Usage: xyz2wrl xyz_file wrl_file mode'
	   write(6,*) 'xyz_file  : input xyz file'
	   write(6,*) 'wrl_file  : output wrl file'
	   write(6,*) 'mode      : 0/1 smoothing off/on'
	   stop
	end if

	call getarg(1,flnm1)
	l1=istrln(flnm1)
	open(unit=2,file=flnm1(1:l1))

	n=0
 200	continue
	read(2,*,end=100) (x(n+i),y(n+i),z(n+i),i=1,3)
	n=n+3
	if(n.gt.nmax) then
	   write(6,*) 'n > nmax'
	   stop
	end if
	go to 200
 100	continue
	nvrtx=n
        write(6,*) 'nvrtx=',nvrtx
	close(unit=2)

	call getarg(3,cmode)
	if(cmode.eq.'0') then
	   mode=0
	else if(cmode.eq.'1') then
	   mode=1
	else
	   write(6,*) 'invalid mode'
	   stop
	end if

c       find independent vectors
        write(6,*) 'mode=',mode

        if(mode.eq.1) then
	   call indvct(x,y,z,nvrtx,nind,icor,ind)
	   write(6,*) 'nind=',nind
	end if

	call getarg(2,flnm2)
	l2=istrln(flnm2)
	open(unit=3,file=flnm2(1:l2))

	write(3,*) '#VRML V2.0 utf8 '
	write(3,*) 'WorldInfo { title "poly2vrml" } '
	write(3,*) 'NavigationInfo { type "EXAMINE" } '
	write(3,*) 'DEF BKG Background { skyColor [ 0.0 0.0 0.5 ] } '
	write(3,*) 'Transform { children [ Shape{ '
	write(3,*) 'appearance Appearance { material Material '
	write(3,*) '{ diffuseColor  0.00  1.00  0.00   specularColor'
	write(3,*) ' 1 1 1 shininess 0.9 } } '
	write(3,*) 'geometry IndexedFaceSet{ coord Coordinate{ point[ '

        if(mode.eq.0) then
	   do i=1,nvrtx
	      write(3,'(3f10.5,a)') x(i),y(i),z(i),','
	   end do
	   write(3,*) ']} coordIndex[' !//VRML end
	   do i=1,nvrtx,3
	      i1=i -1
	      i2=i
	      i3=i+1
	      write(3,'(3(i5,a),i3,a)') i1,',',i2,',',i3,',',-1,','
	   end do
	else
	   do ii=1,nind
	      i=ind(ii)
	      write(3,'(3f10.5,a)') x(i),y(i),z(i),','
	   end do
	   write(3,*) ']} coordIndex[' !//VRML end
	   do i=1,nvrtx,3
	      i1=icor(i)  -1
	      i2=icor(i+1)-1
	      i3=icor(i+2)-1
	      write(3,'(3(i5,a),i3,a)') i1,',',i2,',',i3,',',-1,','
	   end do
	end if
   	write(3,*) '] }}]}'
	close(unit=3)
	stop
	end

	integer function istrln(c)
        include 'impr8.inc'
	character*(*) c
	l=len(c)
	do 10 i=l,1,-1
	   if(c(i:i).ne.' ') then
	      istrln=i
	      return
	   end if
 10	continue
	istrln=0
	return
	end

	subroutine indvct(x,y,z,n,nind,icor,ind)
        include 'impr8.inc'
c       find independent vectors
        real*8 x(*),y(*),z(*)
        integer icor(*),ind(*)
	parameter (nmax=200000)
	integer st(2*nmax),ns(nmax),iskp(nmax)
	real*8 t(nmax)
        data eps /0.000001/
c       icor : independent vector number

        do i=1,n
	   t(i)=z(i)
	end do
        call qsort(t,st,ns,n)

	nind=0	
	do id=1,n
           i=ns(id)
           if(iskp(i).eq.0) then
c       find independent vertices        
	      nind=nind+1
              iskp(i)=1
              icor(i)=nind
	      ind(nind)=i
           else
	      go to 150
	   end if
c       find vertices same as ii-th one
	   do jd=id+1,n
	      j=ns(jd)
              if(iskp(j).eq.0) then
		 ieq=isame(x(i),y(i),z(i),x(j),y(j),z(j))
		 if(ieq.eq.1) then  
		    iskp(j)=1
		    icor(j)=icor(i)
		 else
		    if(t(jd)-t(id).gt.eps) then
		       go to 150
		    end if
		 end if
	      end if
	   end do
 150	   continue
	end do
	return
	end

	integer function isame(x1,y1,z1,x2,y2,z2)
        include 'impr8.inc'
	data eps /0.000001/
	d=sqrt((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)
	if(d.lt.eps) then
	   isame=1
	else
	   isame=0
	end if
	return
	end

	subroutine qsort(x,st,ip,nx)
	include 'impr8.inc'
c       quick sort (ascending order of x)
c       nx: the number of data x
c       ip: the initial order
c       st: a work array
	real*8 x(nx)
	integer r,s,st(nx,0:1),ip(nx)
	do i=1,nx
	   ip(i)=i
	end do
	s=1
	st(1,0)=1
	st(1,1)=nx
 1	l=st(s,0)
	r=st(s,1)
	s=s-1
 2	i=l
	j=r
	xt=x((l+r)/2)
 3	if(i.lt.nx) then
	   if(x(i).lt.xt) then
	      i=i+1
	      go to 3
	   endif
	endif
 4	if(j.gt.1) then
	   if(xt.lt.x(j)) then
	      j=j-1
	      go to 4
	   endif
	endif
	if(i.le.j) then
	   temp=x(j)
	   x(j)=x(i)
	   x(i)=temp
	   itemp=ip(j)
	   ip(j)=ip(i)
	   ip(i)=itemp
	   if(i.le.nx.and.j.ge.1) then
	      i=i+1
	      j=j-1
	      go to 3
	   end if
	endif
	if(j-l.le.r-i) go to 5
	if(i.lt.r) then
	   s=s+1
	   st(s,0)=i
	   st(s,1)=r
	endif
	r=j
	go to 6
 5	if(l.lt.j) then
	   s=s+1
	   st(s,0)=l
	   st(s,1)=j
	endif
	l=i
 6	if(l.lt.r) go to 2
	if(s.ne.0) go to 1
	return
	end






